@@ -20,13 +20,15 @@
  *
  */
 
-#include <hydrogen/basics/sample.h>
+
 
 #include <limits>
 
 #include <hydrogen/hydrogen.h>
 #include <hydrogen/Preferences.h>
 #include <hydrogen/helpers/filesystem.h>
+#include <hydrogen/basics/sample.h>
+
 #ifdef H2CORE_HAVE_RUBBERBAND
 #include <rubberband/RubberBandStretcher.h>
 #define RUBBERBAND_BUFFER_OVERSIZE  500
@@ -52,36 +54,36 @@ Sample::Sample( const QString& filepath,  int frames, int sample_rate, float* da
 	__data_r( data_r ),
 	__is_modified( false )
 {
-	/*
-	if( !(filepath.lastIndexOf( "/" ) >0) ) {
-		ERRORLOG( QString( "sample path : %1 is not ok" ).arg( filepath ) );
-		sleep(1);
-		0/0;
-	}
-	*/
 	assert( filepath.lastIndexOf( "/" ) >0 );
 }
 
-Sample::Sample( Sample* other ): Object( __class_name ),
-	__filepath( other->get_filepath() ),
-	__frames( other->get_frames() ),
-	__sample_rate( other->get_sample_rate() ),
+Sample::Sample( Sample* pOther ): Object( __class_name ),
+	__filepath( pOther->get_filepath() ),
+	__frames( pOther->get_frames() ),
+	__sample_rate( pOther->get_sample_rate() ),
 	__data_l( 0 ),
 	__data_r( 0 ),
-	__is_modified( other->get_is_modified() ),
-	__loops( other->__loops ),
-	__rubberband( other->__rubberband )
+	__is_modified( pOther->get_is_modified() ),
+	__loops( pOther->__loops ),
+	__rubberband( pOther->__rubberband )
 {
 	__data_l = new float[__frames];
 	__data_r = new float[__frames];
-	memcpy( __data_l, other->get_data_l(), __frames );
-	memcpy( __data_r, other->get_data_r(), __frames );
-	EnvelopePoint pt;
-	PanEnvelope* pan = other->get_pan_envelope();
-	for( int i=0; i<pan->size(); i++ ) __pan_envelope.push_back( pan->at( i ) );
-	PanEnvelope* velocity = other->get_velocity_envelope();
-	for( int i=0; i<velocity->size(); i++ ) __velocity_envelope.push_back( velocity->at( i ) );
+	memcpy( __data_l, pOther->get_data_l(), __frames );
+	memcpy( __data_r, pOther->get_data_r(), __frames );
 
+	PanEnvelope* pPan = pOther->get_pan_envelope();
+	for( int i=0; i<pPan->size(); i++ )
+	{
+		__pan_envelope.push_back( pPan->at( i ) );
+	}
+
+
+	PanEnvelope* pVelocity = pOther->get_velocity_envelope();
+	for( int i=0; i<pVelocity->size(); i++ )
+	{
+		__velocity_envelope.push_back( pVelocity->at( i ) );
+	}
 }
 
 Sample::~Sample()
@@ -90,6 +92,14 @@ Sample::~Sample()
 	if( __data_r!=0 ) delete[] __data_r;
 }
 
+void Sample::set_filename( const QString& filename )
+{
+	QFileInfo Filename = QFileInfo( filename );
+	QFileInfo Dest = QFileInfo( __filepath );
+	__filepath = QDir(Dest.absolutePath()).filePath( Filename.fileName() );
+}
+
+
 Sample* Sample::load( const QString& filepath )
 {
 	if( !Filesystem::file_readable( filepath ) ) {
@@ -449,16 +459,19 @@ bool Sample::exec_rubberband_cli( const Rubberband& rb )
 		double durationtime = 60.0 / Hydrogen::get_instance()->getNewBpmJTM() * rb.divider/*beats*/;
 		double induration = get_sample_duration();
 		if ( induration != 0.0 ) ratio = durationtime / induration;
+
 		rubberoutframes = int( __frames * ratio + 0.1 );
 		_INFOLOG( QString( "ratio: %1, rubberoutframes: %2, rubberinframes: %3" ).arg( ratio ).arg ( rubberoutframes ).arg ( __frames ) );
 
-		QObject* parent = 0;
-		QProcess* rubberband = new QProcess( parent );
+		QObject*	pParent = 0;
+		QProcess*	pRrubberbandProc = new QProcess( pParent );
+
 		QStringList arguments;
 		QString rCs = QString( " %1" ).arg( rb.c_settings );
 		float pitch = pow( 1.0594630943593, ( double )rb.pitch );
 		QString rPs = QString( " %1" ).arg( pitch );
 		QString rubberResultPath = QDir::tempPath() + "/tmp_rb_result_file.wav";
+
 		arguments << "-D" << QString( " %1" ).arg( durationtime ) 	//stretch or squash to make output file X seconds long
 				  << "--threads"					//assume multi-CPU even if only one CPU is identified
 				  << "-P"						//aim for minimal time distortion
@@ -466,8 +479,10 @@ bool Sample::exec_rubberband_cli( const Rubberband& rb )
 				  << "-c" << rCs					//"crispness" levels
 				  << outfilePath 					//infile
 				  << rubberResultPath;					//outfile
-		rubberband->start( program, arguments );
-		while( !rubberband->waitForFinished() ) {
+
+		pRrubberbandProc->start( program, arguments );
+
+		while( !	pRrubberbandProc->waitForFinished() ) {
 			//_ERRORLOG( QString( "prozessing" ));
 		}
 		if ( QFile( rubberResultPath ).exists() == false ) {
@@ -475,29 +490,31 @@ bool Sample::exec_rubberband_cli( const Rubberband& rb )
 			return false;
 		}
 
-		Sample* rubberbanded = Sample::load( rubberResultPath.toLocal8Bit() );
-		if( rubberbanded==0 ) {
+		Sample* p_Rubberbanded = Sample::load( rubberResultPath.toLocal8Bit() );
+		if( p_Rubberbanded==0 ) {
 			return false;
 		}
-		if( QFile( outfilePath ).remove() );
-//			_INFOLOG("remove outfile");
-		if( QFile( rubberResultPath ).remove() );
-//			_INFOLOG("remove rubberResultFile");
-		__frames = rubberbanded->get_frames();
-		__data_l = rubberbanded->get_data_l();
-		__data_r = rubberbanded->get_data_r();
-		rubberbanded->__data_l = 0;
-		rubberbanded->__data_r = 0;
+
+		QFile( outfilePath ).remove();
+
+		QFile( rubberResultPath ).remove();
+
+		__frames = p_Rubberbanded->get_frames();
+		__data_l = p_Rubberbanded->get_data_l();
+		__data_r = p_Rubberbanded->get_data_r();
+		p_Rubberbanded->__data_l = 0;
+		p_Rubberbanded->__data_r = 0;
 		__is_modified = true;
 		__rubberband = rb;
-		delete rubberbanded;
+		delete p_Rubberbanded;
 	}
 	return true;
 }
 
 Sample::Loops::LoopMode Sample::parse_loop_mode( const QString& string )
 {
-	char* mode = string.toLocal8Bit().data();
+	QByteArray byteArray =	string.toLocal8Bit();
+	char* mode = byteArray.data();
 	for( int i=Loops::FORWARD; i<=Loops::PINGPONG; i++ ) {
 		if( 0 == strncasecmp( mode, __loop_modes[i], sizeof( __loop_modes[i] ) ) ) return ( Loops::LoopMode )i;
 	}
