@@ -24,15 +24,18 @@
 
 #include <cassert>
 
-#include <hydrogen/basics/adsr.h>
+
 #include <hydrogen/LocalFileMng.h>
 #include <hydrogen/Preferences.h>
-
+#include <hydrogen/event_queue.h>
 #include <hydrogen/fx/Effects.h>
 #include <hydrogen/globals.h>
+#include <hydrogen/timeline.h>
 #include <hydrogen/basics/song.h>
+#include <hydrogen/basics/drumkit_component.h>
 #include <hydrogen/basics/sample.h>
 #include <hydrogen/basics/instrument.h>
+#include <hydrogen/basics/instrument_component.h>
 #include <hydrogen/basics/instrument_list.h>
 #include <hydrogen/basics/instrument_layer.h>
 #include <hydrogen/basics/pattern.h>
@@ -72,13 +75,11 @@ Song::Song( const QString& name, const QString& author, float bpm, float volume
 	, __humanize_velocity_value( 0.0 )
 	, __swing_factor( 0.0 )
 	, __song_mode( PATTERN_MODE )
+	, __components( NULL )
 {
 	INFOLOG( QString( "INIT '%1'" ).arg( __name ) );
 
-	//m_bDelayFXEnabled = false;
-	//m_fDelayFXWetLevel = 0.8;
-	//m_fDelayFXFeedback = 0.5;
-	//m_nDelayFXTime = MAX_NOTES / 8;
+	__components = new std::vector<DrumkitComponent*> ();
 }
 
 
@@ -88,6 +89,9 @@ Song::~Song()
 	// delete all patterns
 	delete __pattern_list;
 
+	__components->clear();
+	delete __components;
+
 	if ( __pattern_group_sequence ) {
 		for ( unsigned i = 0; i < __pattern_group_sequence->size(); ++i ) {
 			PatternList* pPatternList = ( *__pattern_group_sequence )[i];
@@ -151,7 +155,7 @@ Song* Song::get_default_song()
 	song->set_instrument_list( pList );
 
 #ifdef H2CORE_HAVE_JACK
-	Hydrogen::get_instance()->renameJackPorts();
+	Hydrogen::get_instance()->renameJackPorts( song );
 #endif
 
 	PatternList* patternList = new PatternList();
@@ -165,7 +169,7 @@ Song* Song::get_default_song()
 	patternSequence->add( emptyPattern );
 	pPatternGroupVector->push_back( patternSequence );
 	song->set_pattern_group_vector( pPatternGroupVector );
-	song->__is_modified = false;
+	song->set_is_modified( false );
 	song->set_filename( "empty_song" );
 
 	return song;
@@ -188,6 +192,15 @@ Song* Song::get_empty_song()
 	return song;
 }
 
+DrumkitComponent* Song::get_component( int ID )
+{
+	for (std::vector<DrumkitComponent*>::iterator it = __components->begin() ; it != __components->end(); ++it) {
+		if( (*it)->get_id() == ID )
+			return *it;
+	}
+
+	return NULL;
+}
 
 
 void Song::set_swing_factor( float factor )
@@ -201,6 +214,19 @@ void Song::set_swing_factor( float factor )
 	__swing_factor = factor;
 }
 
+void Song::set_is_modified(bool is_modified){
+	bool Notify = false;
+
+	if(__is_modified != is_modified){
+		Notify = true;
+	}
+
+	__is_modified = is_modified;
+
+	if(Notify){
+		EventQueue::get_instance()->push_event( EVENT_SONG_MODIFIED, -1 );
+	}
+}
 
 void Song::readTempPatternList( QString filename )
 {
@@ -416,12 +442,25 @@ Song* SongReader::readSong( const QString& filename )
 	song->set_humanize_velocity_value( fHumanizeVelocityValue );
 	song->set_swing_factor( fSwingFactor );
 
-	/*
-	song->m_bDelayFXEnabled = LocalFileMng::readXmlBool( songNode, "delayFXEnabled", false, false );
-	song->m_fDelayFXWetLevel = LocalFileMng::readXmlFloat( songNode, "delayFXWetLevel", 1.0, false, false );
-	song->m_fDelayFXFeedback= LocalFileMng::readXmlFloat( songNode, "delayFXFeedback", 0.4, false, false );
-	song->m_nDelayFXTime = LocalFileMng::readXmlInt( songNode, "delayFXTime", MAX_NOTES / 4, false, false );
-	*/
+	QDomNode componentListNode = songNode.firstChildElement( "componentList" );
+	if ( ( ! componentListNode.isNull()  ) ) {
+		QDomNode componentNode = componentListNode.firstChildElement( "drumkitComponent" );
+		while ( ! componentNode.isNull()  ) {
+			int id = LocalFileMng::readXmlInt( componentNode, "id", -1 );			// instrument id
+			QString sName = LocalFileMng::readXmlString( componentNode, "name", "" );		// name
+			float fVolume = LocalFileMng::readXmlFloat( componentNode, "volume", 1.0 );	// volume
+			DrumkitComponent* pDrumkitComponent = new DrumkitComponent( id, sName );
+			pDrumkitComponent->set_volume( fVolume );
+
+			song->get_components()->push_back(pDrumkitComponent);
+
+			componentNode = ( QDomNode ) componentNode.nextSiblingElement( "drumkitComponent" );
+		}
+	}
+	else {
+		DrumkitComponent* pDrumkitComponent = new DrumkitComponent( 0, "Main" );
+		song->get_components()->push_back(pDrumkitComponent);
+	}
 
 	//  Instrument List
 	InstrumentList* instrumentList = new InstrumentList();
@@ -452,10 +491,11 @@ Song* SongReader::readSong( const QString& filename )
 			int fAttack = LocalFileMng::readXmlInt( instrumentNode, "Attack", 0, false, false );		// Attack
 			int fDecay = LocalFileMng::readXmlInt( instrumentNode, "Decay", 0, false, false );		// Decay
 			float fSustain = LocalFileMng::readXmlFloat( instrumentNode, "Sustain", 1.0, false, false );	// Sustain
-			int fRelease = LocalFileMng::readXmlInt( instrumentNode, "Release", 1000, false, false );	// Release
+			int fRelease = LocalFileMng::readXmlFloat( instrumentNode, "Release", 1000.0, false, false );	// Release
 
 			float fRandomPitchFactor = LocalFileMng::readXmlFloat( instrumentNode, "randomPitchFactor", 0.0f, false, false );
 
+			bool bApplyVelocity = LocalFileMng::readXmlBool( instrumentNode, "applyVelocity", true );
 			bool bFilterActive = LocalFileMng::readXmlBool( instrumentNode, "filterActive", false );
 			float fFilterCutoff = LocalFileMng::readXmlFloat( instrumentNode, "filterCutoff", 1.0f, false );
 			float fFilterResonance = LocalFileMng::readXmlFloat( instrumentNode, "filterResonance", 0.0f, false );
@@ -464,6 +504,8 @@ Song* SongReader::readSong( const QString& filename )
 			QString sMidiOutNote = LocalFileMng::readXmlString( instrumentNode, "midiOutNote", "60", false, false );
 			int nMuteGroup = sMuteGroup.toInt();
 			bool isStopNote = LocalFileMng::readXmlBool( instrumentNode, "isStopNote", false );
+			QString sRead_sample_select_algo = LocalFileMng::readXmlString( instrumentNode, "sampleSelectionAlgo", "VELOCITY" );
+
 			int nMidiOutChannel = sMidiOutChannel.toInt();
 			int nMidiOutNote = sMidiOutNote.toInt();
 
@@ -473,6 +515,9 @@ Song* SongReader::readSong( const QString& filename )
 				continue;
 			}
 
+			int iIsHiHat = LocalFileMng::readXmlInt( instrumentNode, "isHihat", -1, true );
+			int iLowerCC = LocalFileMng::readXmlInt( instrumentNode, "lower_cc", 0, true );
+			int iHigherCC = LocalFileMng::readXmlInt( instrumentNode, "higher_cc", 127, true );
 
 			// create a new instrument
 			Instrument* pInstrument = new Instrument( id, sName, new ADSR( fAttack, fDecay, fSustain, fRelease ) );
@@ -481,6 +526,7 @@ Song* SongReader::readSong( const QString& filename )
 			pInstrument->set_pan_l( fPan_L );
 			pInstrument->set_pan_r( fPan_R );
 			pInstrument->set_drumkit_name( sDrumkit );
+			pInstrument->set_apply_velocity( bApplyVelocity );
 			pInstrument->set_fx_level( fFX1Level, 0 );
 			pInstrument->set_fx_level( fFX2Level, 1 );
 			pInstrument->set_fx_level( fFX3Level, 2 );
@@ -492,6 +538,15 @@ Song* SongReader::readSong( const QString& filename )
 			pInstrument->set_gain( fGain );
 			pInstrument->set_mute_group( nMuteGroup );
 			pInstrument->set_stop_notes( isStopNote );
+			pInstrument->set_hihat_grp( iIsHiHat );
+			pInstrument->set_lower_cc( iLowerCC );
+			pInstrument->set_higher_cc( iHigherCC );
+			if ( sRead_sample_select_algo.compare("VELOCITY") == 0 )
+				pInstrument->set_sample_selection_alg( Instrument::VELOCITY );
+			else if ( sRead_sample_select_algo.compare("ROUND_ROBIN") == 0 )
+				pInstrument->set_sample_selection_alg( Instrument::ROUND_ROBIN );
+			else if ( sRead_sample_select_algo.compare("RANDOM") == 0 )
+				pInstrument->set_sample_selection_alg( Instrument::RANDOM );
 			pInstrument->set_midi_out_channel( nMidiOutChannel );
 			pInstrument->set_midi_out_note( nMidiOutNote );
 
@@ -524,94 +579,193 @@ Song* SongReader::readSong( const QString& filename )
 					ERRORLOG( "Error loading sample: " + sFilename + " not found" );
 					pInstrument->set_muted( true );
 				}
+				InstrumentComponent* pCompo = new InstrumentComponent ( 0 );
 				InstrumentLayer* pLayer = new InstrumentLayer( pSample );
-				pInstrument->set_layer( pLayer, 0 );
+				pCompo->set_layer( pLayer, 0 );
+				pInstrument->get_components()->push_back( pCompo );
 			}
 			//~ back compatibility code
 			else {
-				unsigned nLayer = 0;
-				QDomNode layerNode = instrumentNode.firstChildElement( "layer" );
-				while (  ! layerNode.isNull()  ) {
-					if ( nLayer >= MAX_LAYERS ) {
-						ERRORLOG( "nLayer > MAX_LAYERS" );
-						continue;
-					}
-					//bool sIsModified = false;
-					QString sFilename = LocalFileMng::readXmlString( layerNode, "filename", "" );
-					bool sIsModified = LocalFileMng::readXmlBool( layerNode, "ismodified", false );
-					Sample::Loops lo;
-					lo.mode = Sample::parse_loop_mode( LocalFileMng::readXmlString( layerNode, "smode", "forward" ) );
-					lo.start_frame = LocalFileMng::readXmlInt( layerNode, "startframe", 0 );
-					lo.loop_frame = LocalFileMng::readXmlInt( layerNode, "loopframe", 0 );
-					lo.count = LocalFileMng::readXmlInt( layerNode, "loops", 0 );
-					lo.end_frame = LocalFileMng::readXmlInt( layerNode, "endframe", 0 );
-					Sample::Rubberband ro;
-					ro.use = LocalFileMng::readXmlInt( layerNode, "userubber", 0, false );
-					ro.divider = LocalFileMng::readXmlFloat( layerNode, "rubberdivider", 0.0 );
-					ro.c_settings = LocalFileMng::readXmlInt( layerNode, "rubberCsettings", 1 );
-					ro.pitch = LocalFileMng::readXmlFloat( layerNode, "rubberPitch", 0.0 );
+				bool p_foundAtLeastOneComponent = false;
+				QDomNode componentNode = instrumentNode.firstChildElement( "instrumentComponent" );
+				while (  ! componentNode.isNull()  ) {
+					p_foundAtLeastOneComponent = true;
+					int id = LocalFileMng::readXmlInt( componentNode, "component_id", 0 );
+					InstrumentComponent* pCompo = new InstrumentComponent( id );
+					float fGainCompo = LocalFileMng::readXmlFloat( componentNode, "gain", 1.0 );
+					pCompo->set_gain( fGainCompo );
 
-					float fMin = LocalFileMng::readXmlFloat( layerNode, "min", 0.0 );
-					float fMax = LocalFileMng::readXmlFloat( layerNode, "max", 1.0 );
-					float fGain = LocalFileMng::readXmlFloat( layerNode, "gain", 1.0 );
-					float fPitch = LocalFileMng::readXmlFloat( layerNode, "pitch", 0.0, false, false );
+					unsigned nLayer = 0;
+					QDomNode layerNode = componentNode.firstChildElement( "layer" );
+					while (  ! layerNode.isNull()  ) {
+						if ( nLayer >= MAX_LAYERS ) {
+							ERRORLOG( "nLayer > MAX_LAYERS" );
+							continue;
+						}
+						//bool sIsModified = false;
+						QString sFilename = LocalFileMng::readXmlString( layerNode, "filename", "" );
+						bool sIsModified = LocalFileMng::readXmlBool( layerNode, "ismodified", false );
+						Sample::Loops lo;
+						lo.mode = Sample::parse_loop_mode( LocalFileMng::readXmlString( layerNode, "smode", "forward" ) );
+						lo.start_frame = LocalFileMng::readXmlInt( layerNode, "startframe", 0 );
+						lo.loop_frame = LocalFileMng::readXmlInt( layerNode, "loopframe", 0 );
+						lo.count = LocalFileMng::readXmlInt( layerNode, "loops", 0 );
+						lo.end_frame = LocalFileMng::readXmlInt( layerNode, "endframe", 0 );
+						Sample::Rubberband ro;
+						ro.use = LocalFileMng::readXmlInt( layerNode, "userubber", 0, false );
+						ro.divider = LocalFileMng::readXmlFloat( layerNode, "rubberdivider", 0.0 );
+						ro.c_settings = LocalFileMng::readXmlInt( layerNode, "rubberCsettings", 1 );
+						ro.pitch = LocalFileMng::readXmlFloat( layerNode, "rubberPitch", 0.0 );
 
-					if ( !QFile( sFilename ).exists() && !drumkitPath.isEmpty() ) {
-						sFilename = drumkitPath + "/" + sFilename;
-					}
+						float fMin = LocalFileMng::readXmlFloat( layerNode, "min", 0.0 );
+						float fMax = LocalFileMng::readXmlFloat( layerNode, "max", 1.0 );
+						float fGain = LocalFileMng::readXmlFloat( layerNode, "gain", 1.0 );
+						float fPitch = LocalFileMng::readXmlFloat( layerNode, "pitch", 0.0, false, false );
 
-					QString program = Preferences::get_instance()->m_rubberBandCLIexecutable;
-					//test the path. if test fails, disable rubberband
-					if ( QFile( program ).exists() == false ) {
-						ro.use = false;
-					}
-
-					Sample* pSample = NULL;
-					if ( !sIsModified ) {
-						pSample = Sample::load( sFilename );
-					} else {
-						Sample::EnvelopePoint pt;
-
-						Sample::VelocityEnvelope velocity;
-						QDomNode volumeNode = layerNode.firstChildElement( "volume" );
-						while (  ! volumeNode.isNull()  ) {
-							pt.frame = LocalFileMng::readXmlInt( volumeNode, "volume-position", 0 );
-							pt.value = LocalFileMng::readXmlInt( volumeNode, "volume-value", 0 );
-							velocity.push_back( pt );
-							volumeNode = volumeNode.nextSiblingElement( "volume" );
-							//ERRORLOG( QString("volume-posi %1").arg(LocalFileMng::readXmlInt( volumeNode, "volume-position", 0)) );
+						if ( !QFile( sFilename ).exists() && !drumkitPath.isEmpty() ) {
+							sFilename = drumkitPath + "/" + sFilename;
 						}
 
-						Sample::VelocityEnvelope pan;
-						QDomNode  panNode = layerNode.firstChildElement( "pan" );
-						while (  ! panNode.isNull()  ) {
-							pt.frame = LocalFileMng::readXmlInt( panNode, "pan-position", 0 );
-							pt.value = LocalFileMng::readXmlInt( panNode, "pan-value", 0 );
-							pan.push_back( pt );
-							panNode = panNode.nextSiblingElement( "pan" );
+						QString program = Preferences::get_instance()->m_rubberBandCLIexecutable;
+						//test the path. if test fails, disable rubberband
+						if ( QFile( program ).exists() == false ) {
+							ro.use = false;
 						}
 
-						pSample = Sample::load( sFilename, lo, ro, velocity, pan );
-					}
-					if ( pSample == NULL ) {
-						ERRORLOG( "Error loading sample: " + sFilename + " not found" );
-						pInstrument->set_muted( true );
-					}
-					InstrumentLayer* pLayer = new InstrumentLayer( pSample );
-					pLayer->set_start_velocity( fMin );
-					pLayer->set_end_velocity( fMax );
-					pLayer->set_gain( fGain );
-					pLayer->set_pitch( fPitch );
-					pInstrument->set_layer( pLayer, nLayer );
-					nLayer++;
+						Sample* pSample = NULL;
+						if ( !sIsModified ) {
+							pSample = Sample::load( sFilename );
+						} else {
+							Sample::EnvelopePoint pt;
 
-					layerNode = ( QDomNode ) layerNode.nextSiblingElement( "layer" );
+							Sample::VelocityEnvelope velocity;
+							QDomNode volumeNode = layerNode.firstChildElement( "volume" );
+							while (  ! volumeNode.isNull()  ) {
+								pt.frame = LocalFileMng::readXmlInt( volumeNode, "volume-position", 0 );
+								pt.value = LocalFileMng::readXmlInt( volumeNode, "volume-value", 0 );
+								velocity.push_back( pt );
+								volumeNode = volumeNode.nextSiblingElement( "volume" );
+								//ERRORLOG( QString("volume-posi %1").arg(LocalFileMng::readXmlInt( volumeNode, "volume-position", 0)) );
+							}
+
+							Sample::VelocityEnvelope pan;
+							QDomNode  panNode = layerNode.firstChildElement( "pan" );
+							while (  ! panNode.isNull()  ) {
+								pt.frame = LocalFileMng::readXmlInt( panNode, "pan-position", 0 );
+								pt.value = LocalFileMng::readXmlInt( panNode, "pan-value", 0 );
+								pan.push_back( pt );
+								panNode = panNode.nextSiblingElement( "pan" );
+							}
+
+							pSample = Sample::load( sFilename, lo, ro, velocity, pan );
+						}
+						if ( pSample == NULL ) {
+							ERRORLOG( "Error loading sample: " + sFilename + " not found" );
+							pInstrument->set_muted( true );
+						}
+						InstrumentLayer* pLayer = new InstrumentLayer( pSample );
+						pLayer->set_start_velocity( fMin );
+						pLayer->set_end_velocity( fMax );
+						pLayer->set_gain( fGain );
+						pLayer->set_pitch( fPitch );
+						pCompo->set_layer( pLayer, nLayer );
+						nLayer++;
+
+						layerNode = ( QDomNode ) layerNode.nextSiblingElement( "layer" );
+					}
+
+					pInstrument->get_components()->push_back( pCompo );
+					componentNode = ( QDomNode ) componentNode.nextSiblingElement( "instrumentComponent" );
+				}
+				if(!p_foundAtLeastOneComponent){
+					InstrumentComponent* pCompo = new InstrumentComponent( 0 );
+					float fGainCompo = LocalFileMng::readXmlFloat( componentNode, "gain", 1.0 );
+					pCompo->set_gain( fGainCompo );
+
+					unsigned nLayer = 0;
+					QDomNode layerNode = instrumentNode.firstChildElement( "layer" );
+					while (  ! layerNode.isNull()  ) {
+						if ( nLayer >= MAX_LAYERS ) {
+							ERRORLOG( "nLayer > MAX_LAYERS" );
+							continue;
+						}
+						QString sFilename = LocalFileMng::readXmlString( layerNode, "filename", "" );
+						bool sIsModified = LocalFileMng::readXmlBool( layerNode, "ismodified", false );
+						Sample::Loops lo;
+						lo.mode = Sample::parse_loop_mode( LocalFileMng::readXmlString( layerNode, "smode", "forward" ) );
+						lo.start_frame = LocalFileMng::readXmlInt( layerNode, "startframe", 0 );
+						lo.loop_frame = LocalFileMng::readXmlInt( layerNode, "loopframe", 0 );
+						lo.count = LocalFileMng::readXmlInt( layerNode, "loops", 0 );
+						lo.end_frame = LocalFileMng::readXmlInt( layerNode, "endframe", 0 );
+						Sample::Rubberband ro;
+						ro.use = LocalFileMng::readXmlInt( layerNode, "userubber", 0, false );
+						ro.divider = LocalFileMng::readXmlFloat( layerNode, "rubberdivider", 0.0 );
+						ro.c_settings = LocalFileMng::readXmlInt( layerNode, "rubberCsettings", 1 );
+						ro.pitch = LocalFileMng::readXmlFloat( layerNode, "rubberPitch", 0.0 );
+
+						float fMin = LocalFileMng::readXmlFloat( layerNode, "min", 0.0 );
+						float fMax = LocalFileMng::readXmlFloat( layerNode, "max", 1.0 );
+						float fGain = LocalFileMng::readXmlFloat( layerNode, "gain", 1.0 );
+						float fPitch = LocalFileMng::readXmlFloat( layerNode, "pitch", 0.0, false, false );
+
+						if ( !QFile( sFilename ).exists() && !drumkitPath.isEmpty() ) {
+							sFilename = drumkitPath + "/" + sFilename;
+						}
+
+						QString program = Preferences::get_instance()->m_rubberBandCLIexecutable;
+						//test the path. if test fails, disable rubberband
+						if ( QFile( program ).exists() == false ) {
+							ro.use = false;
+						}
+
+						Sample* pSample = NULL;
+						if ( !sIsModified ) {
+							pSample = Sample::load( sFilename );
+						} else {
+							Sample::EnvelopePoint pt;
+
+							Sample::VelocityEnvelope velocity;
+							QDomNode volumeNode = layerNode.firstChildElement( "volume" );
+							while (  ! volumeNode.isNull()  ) {
+								pt.frame = LocalFileMng::readXmlInt( volumeNode, "volume-position", 0 );
+								pt.value = LocalFileMng::readXmlInt( volumeNode, "volume-value", 0 );
+								velocity.push_back( pt );
+								volumeNode = volumeNode.nextSiblingElement( "volume" );
+								//ERRORLOG( QString("volume-posi %1").arg(LocalFileMng::readXmlInt( volumeNode, "volume-position", 0)) );
+							}
+
+							Sample::VelocityEnvelope pan;
+							QDomNode  panNode = layerNode.firstChildElement( "pan" );
+							while (  ! panNode.isNull()  ) {
+								pt.frame = LocalFileMng::readXmlInt( panNode, "pan-position", 0 );
+								pt.value = LocalFileMng::readXmlInt( panNode, "pan-value", 0 );
+								pan.push_back( pt );
+								panNode = panNode.nextSiblingElement( "pan" );
+							}
+
+							pSample = Sample::load( sFilename, lo, ro, velocity, pan );
+						}
+						if ( pSample == NULL ) {
+							ERRORLOG( "Error loading sample: " + sFilename + " not found" );
+							pInstrument->set_muted( true );
+						}
+						InstrumentLayer* pLayer = new InstrumentLayer( pSample );
+						pLayer->set_start_velocity( fMin );
+						pLayer->set_end_velocity( fMax );
+						pLayer->set_gain( fGain );
+						pLayer->set_pitch( fPitch );
+						pCompo->set_layer( pLayer, nLayer );
+						nLayer++;
+
+						layerNode = ( QDomNode ) layerNode.nextSiblingElement( "layer" );
+					}
+					pInstrument->get_components()->push_back( pCompo );
 				}
 			}
-
 			instrumentList->add( pInstrument );
 			instrumentNode = ( QDomNode ) instrumentNode.nextSiblingElement( "instrument" );
 		}
+
 		if ( instrumentList_count == 0 ) {
 			WARNINGLOG( "0 instruments?" );
 		}
@@ -807,11 +961,6 @@ Song* SongReader::readSong( const QString& filename )
 						}
 						inputControlNode = ( QDomNode ) inputControlNode.nextSiblingElement( "inputControlPort" );
 					}
-
-					/*
-					TiXmlNode* outputControlNode;
-					for ( outputControlNode = fxNode->FirstChild( "outputControlPort" ); outputControlNode; outputControlNode = outputControlNode->NextSibling( "outputControlPort" ) ) {
-					}*/
 				}
 #endif
 			}
@@ -822,39 +971,40 @@ Song* SongReader::readSong( const QString& filename )
 		WARNINGLOG( "ladspa node not found" );
 	}
 
-	Hydrogen::get_instance()->m_timelinevector.clear();
-	Hydrogen::HTimelineVector tlvector;
+	Timeline* pTimeline = Hydrogen::get_instance()->getTimeline();
+	pTimeline->m_timelinevector.clear();
+	Timeline::HTimelineVector tlvector;
 	QDomNode bpmTimeLine = songNode.firstChildElement( "BPMTimeLine" );
 	if ( !bpmTimeLine.isNull() ) {
 		QDomNode newBPMNode = bpmTimeLine.firstChildElement( "newBPM" );
 		while( !newBPMNode.isNull() ) {
 			tlvector.m_htimelinebeat = LocalFileMng::readXmlInt( newBPMNode, "BAR", 0 );
 			tlvector.m_htimelinebpm = LocalFileMng::readXmlFloat( newBPMNode, "BPM", 120.0 );
-			Hydrogen::get_instance()->m_timelinevector.push_back( tlvector );
-			Hydrogen::get_instance()->sortTimelineVector();
+			pTimeline->m_timelinevector.push_back( tlvector );
+			pTimeline->sortTimelineVector();
 			newBPMNode = newBPMNode.nextSiblingElement( "newBPM" );
 		}
 	} else {
 		WARNINGLOG( "bpmTimeLine node not found" );
 	}
 
-	Hydrogen::get_instance()->m_timelinetagvector.clear();
-	Hydrogen::HTimelineTagVector tltagvector;
+	pTimeline->m_timelinetagvector.clear();
+	Timeline::HTimelineTagVector tltagvector;
 	QDomNode timeLineTag = songNode.firstChildElement( "timeLineTag" );
 	if ( !timeLineTag.isNull() ) {
 		QDomNode newTAGNode = timeLineTag.firstChildElement( "newTAG" );
 		while( !newTAGNode.isNull() ) {
 			tltagvector.m_htimelinetagbeat = LocalFileMng::readXmlInt( newTAGNode, "BAR", 0 );
 			tltagvector.m_htimelinetag = LocalFileMng::readXmlString( newTAGNode, "TAG", "" );
-			Hydrogen::get_instance()->m_timelinetagvector.push_back( tltagvector );
-			Hydrogen::get_instance()->sortTimelineTagVector();
+			pTimeline->m_timelinetagvector.push_back( tltagvector );
+			pTimeline->sortTimelineTagVector();
 			newTAGNode = newTAGNode.nextSiblingElement( "newTAG" );
 		}
 	} else {
 		WARNINGLOG( "TagTimeLine node not found" );
 	}
 
-	song->__is_modified = false;
+	song->set_is_modified( false );
 	song->set_filename( FileName );
 
 	return song;
