@@ -24,14 +24,17 @@
 #include <hydrogen/basics/adsr.h>
 #include <hydrogen/hydrogen.h>
 #include <hydrogen/h2_exception.h>
+#include <hydrogen/basics/drumkit_component.h>
 #include <hydrogen/basics/instrument.h>
 #include <hydrogen/basics/instrument_list.h>
+#include <hydrogen/basics/instrument_component.h>
 #include <hydrogen/basics/instrument_layer.h>
 #include <hydrogen/LocalFileMng.h>
 #include <hydrogen/basics/note.h>
 #include <hydrogen/basics/pattern.h>
 #include <hydrogen/basics/pattern_list.h>
 #include <hydrogen/Preferences.h>
+#include <hydrogen/timeline.h>
 #include <hydrogen/basics/song.h>
 #include <hydrogen/basics/drumkit.h>
 #include <hydrogen/basics/sample.h>
@@ -70,7 +73,7 @@ LocalFileMng::~LocalFileMng()
 
 QString LocalFileMng::getDrumkitNameForPattern( const QString& patternDir )
 {
-	QDomDocument doc = LocalFileMng::openXmlDocument( patternDir );
+	QDomDocument doc = openXmlDocument( patternDir );
 
 	QDomNode rootNode = doc.firstChildElement( "drumkit_pattern" );	// root element
 	if (  rootNode.isNull() ) {
@@ -777,11 +780,16 @@ int  LocalFileMng::mergeAllPatternList( std::vector<QString> current )
  * \return Returns an Errorcode.
  */
 
-int LocalFileMng::savePlayList( const std::string& filename)
+int LocalFileMng::savePlayList( const std::string& playListFilename)
 {
+	Preferences *pPref = H2Core::Preferences::get_instance();
+	bool		useRelativePathNames = pPref->isPlaylistUsingRelativeFilenames();
 
-	std::string name = filename.c_str();
+	QString		qFileName = QString::fromStdString( playListFilename );
+	QFileInfo	fileInfo( qFileName );
+	QDir		playlistDir = fileInfo.absoluteDir();
 
+	std::string name = playListFilename.c_str();
 	std::string realname = name.substr(name.rfind("/")+1);
 
 	QDomDocument doc;
@@ -796,7 +804,14 @@ int LocalFileMng::savePlayList( const std::string& filename)
 	for ( uint i = 0; i < Hydrogen::get_instance()->m_PlayList.size(); ++i ){
 		QDomNode nextNode = doc.createElement( "next" );
 
-		LocalFileMng::writeXmlString ( nextNode, "song", Hydrogen::get_instance()->m_PlayList[i].m_hFile );
+		QString playlistItemFilename;
+		if(useRelativePathNames){
+			playlistItemFilename = playlistDir.relativeFilePath(Hydrogen::get_instance()->m_PlayList[i].m_hFile);
+		} else {
+			playlistItemFilename = Hydrogen::get_instance()->m_PlayList[i].m_hFile;
+		}
+
+		LocalFileMng::writeXmlString ( nextNode, "song", playlistItemFilename);
 		LocalFileMng::writeXmlString ( nextNode, "script", Hydrogen::get_instance()->m_PlayList[i].m_hScript );
 		LocalFileMng::writeXmlString ( nextNode, "enabled", Hydrogen::get_instance()->m_PlayList[i].m_hScriptEnabled );
 
@@ -807,7 +822,7 @@ int LocalFileMng::savePlayList( const std::string& filename)
 	doc.appendChild( rootNode );
 
 	int rv = 0;
-	QFile file( filename.c_str() );
+	QFile file( playListFilename.c_str() );
 	if ( !file.open(QIODevice::WriteOnly) )
 		rv = 1;
 
@@ -820,7 +835,6 @@ int LocalFileMng::savePlayList( const std::string& filename)
 	file.close();
 
 	return rv; // ok
-
 }
 
 /**
@@ -831,13 +845,22 @@ int LocalFileMng::savePlayList( const std::string& filename)
 
 int LocalFileMng::loadPlayList( const std::string& filename)
 {
-	std::string playlistInfoFile = filename;
-	std::ifstream verify( playlistInfoFile.c_str() , std::ios::in | std::ios::binary );
-	if ( verify ) {
+	/* openXmlDocument can create new document ( which is bad idea anyway )
+	   We don't want create new playlist here , so we just open it for test ;-)
+	*/
+	QString		Filename = QString( filename.c_str() );
+	QFileInfo	playlistFileInfo( Filename );
+	QDir		playlistDir = playlistFileInfo.absoluteDir();
+
+	QFile file( Filename );
+	if ( file.open(QIODevice::ReadOnly) ) {
+		file.close();
+	} else {
+		ERRORLOG( QString("Error reading playlist: can't open file %1").arg( Filename ) );
 		return 1;
 	}
 
-	QDomDocument doc = LocalFileMng::openXmlDocument( QString( filename.c_str() ) );
+	QDomDocument doc = openXmlDocument( Filename );
 
 	Hydrogen::get_instance()->m_PlayList.clear();
 
@@ -854,11 +877,15 @@ int LocalFileMng::loadPlayList( const std::string& filename)
 		SongReader reader;
 		while (  ! nextNode.isNull() ) {
 			Hydrogen::HPlayListNode playListItem;
-			playListItem.m_hFile = LocalFileMng::readXmlString( nextNode, "song", "" );
+			QString playlistItemPath = readXmlString( nextNode, "song", "" );
+
+			QFileInfo playlistItemInfo (playlistDir, playlistItemPath);
+			playListItem.m_hFile = playlistItemInfo.absoluteFilePath();
+
 			QString FilePath = reader.getPath( playListItem.m_hFile );
 			playListItem.m_hFileExists = Filesystem::file_readable( FilePath );
 			playListItem.m_hScript = LocalFileMng::readXmlString( nextNode, "script", "" );
-			playListItem.m_hScriptEnabled = LocalFileMng::readXmlString( nextNode, "enabled", "" );
+			playListItem.m_hScriptEnabled = readXmlString( nextNode, "enabled", "" );
 
 			Hydrogen::get_instance()->m_PlayList.push_back( playListItem );
 			nextNode = nextNode.nextSiblingElement( "next" );
@@ -867,8 +894,6 @@ int LocalFileMng::loadPlayList( const std::string& filename)
 	return 0; // ok
 }
 
-
-
 /* New QtXml based methods */
 
 QString LocalFileMng::readXmlString( QDomNode node , const QString& nodeName, const QString& defaultValue, bool bCanBeEmpty, bool bShouldExists, bool tinyXmlCompatMode)
@@ -1253,6 +1278,21 @@ int SongWriter::writeSong( Song *song, const QString& filename )
 	LocalFileMng::writeXmlString( songNode, "humanize_velocity", QString("%1").arg( song->get_humanize_velocity_value() ) );
 	LocalFileMng::writeXmlString( songNode, "swing_factor", QString("%1").arg( song->get_swing_factor() ) );
 
+	// component List
+	QDomNode componentListNode = doc.createElement( "componentList" );
+	for (std::vector<DrumkitComponent*>::iterator it = song->get_components()->begin() ; it != song->get_components()->end(); ++it) {
+		DrumkitComponent* pCompo = *it;
+
+		QDomNode componentNode = doc.createElement( "drumkitComponent" );
+
+		LocalFileMng::writeXmlString( componentNode, "id", QString("%1").arg( pCompo->get_id() ) );
+		LocalFileMng::writeXmlString( componentNode, "name", pCompo->get_name() );
+		LocalFileMng::writeXmlString( componentNode, "volume", QString("%1").arg( pCompo->get_volume() ) );
+
+		componentListNode.appendChild( componentNode );
+	}
+	songNode.appendChild( componentListNode );
+
 	// instrument list
 	QDomNode instrumentListNode = doc.createElement( "instrumentList" );
 	unsigned nInstrument = song->get_instrument_list()->size();
@@ -1272,6 +1312,7 @@ int SongWriter::writeSong( Song *song, const QString& filename )
 		LocalFileMng::writeXmlString( instrumentNode, "pan_L", QString("%1").arg( instr->get_pan_l() ) );
 		LocalFileMng::writeXmlString( instrumentNode, "pan_R", QString("%1").arg( instr->get_pan_r() ) );
 		LocalFileMng::writeXmlString( instrumentNode, "gain", QString("%1").arg( instr->get_gain() ) );
+		LocalFileMng::writeXmlBool( instrumentNode, "applyVelocity", instr->get_apply_velocity() );
 
 		LocalFileMng::writeXmlBool( instrumentNode, "filterActive", instr->is_filter_active() );
 		LocalFileMng::writeXmlString( instrumentNode, "filterCutoff", QString("%1").arg( instr->get_filter_cutoff() ) );
@@ -1292,57 +1333,82 @@ int SongWriter::writeSong( Song *song, const QString& filename )
 
 		LocalFileMng::writeXmlString( instrumentNode, "muteGroup", QString("%1").arg( instr->get_mute_group() ) );
 		LocalFileMng::writeXmlBool( instrumentNode, "isStopNote", instr->is_stop_notes() );
+		switch ( instr->sample_selection_alg() ) {
+			case Instrument::VELOCITY:
+				LocalFileMng::writeXmlString( instrumentNode, "sampleSelectionAlgo", "VELOCITY" );
+				break;
+			case Instrument::RANDOM:
+				LocalFileMng::writeXmlString( instrumentNode, "sampleSelectionAlgo", "RANDOM" );
+				break;
+			case Instrument::ROUND_ROBIN:
+				LocalFileMng::writeXmlString( instrumentNode, "sampleSelectionAlgo", "ROUND_ROBIN" );
+				break;
+		}
 
 		LocalFileMng::writeXmlString( instrumentNode, "midiOutChannel", QString("%1").arg( instr->get_midi_out_channel() ) );
 		LocalFileMng::writeXmlString( instrumentNode, "midiOutNote", QString("%1").arg( instr->get_midi_out_note() ) );
+		LocalFileMng::writeXmlString( instrumentNode, "isHihat", QString("%1").arg( instr->get_hihat_grp() ) );
+		LocalFileMng::writeXmlString( instrumentNode, "lower_cc", QString("%1").arg( instr->get_lower_cc() ) );
+		LocalFileMng::writeXmlString( instrumentNode, "higher_cc", QString("%1").arg( instr->get_higher_cc() ) );
 
-		for ( unsigned nLayer = 0; nLayer < MAX_LAYERS; nLayer++ ) {
-			InstrumentLayer *pLayer = instr->get_layer( nLayer );
-			if ( pLayer == NULL ) continue;
-			Sample *pSample = pLayer->get_sample();
-			if ( pSample == NULL ) continue;
+		for (std::vector<InstrumentComponent*>::iterator it = instr->get_components()->begin() ; it != instr->get_components()->end(); ++it) {
+			InstrumentComponent* pComponent = *it;
 
-			bool sIsModified = pSample->get_is_modified();
-			Sample::Loops lo = pSample->get_loops();
-			Sample::Rubberband ro = pSample->get_rubberband();
-			QString sMode = pSample->get_loop_mode_string();
+			QDomNode componentNode = doc.createElement( "instrumentComponent" );
+
+			LocalFileMng::writeXmlString( componentNode, "component_id", QString("%1").arg( pComponent->get_drumkit_componentID() ) );
+			LocalFileMng::writeXmlString( componentNode, "gain", QString("%1").arg( pComponent->get_gain() ) );
+
+			for ( unsigned nLayer = 0; nLayer < MAX_LAYERS; nLayer++ ) {
+				InstrumentLayer *pLayer = pComponent->get_layer( nLayer );
+
+				if ( pLayer == NULL ) continue;
+				Sample *pSample = pLayer->get_sample();
+				if ( pSample == NULL ) continue;
+
+				bool sIsModified = pSample->get_is_modified();
+				Sample::Loops lo = pSample->get_loops();
+				Sample::Rubberband ro = pSample->get_rubberband();
+				QString sMode = pSample->get_loop_mode_string();
 
 
-			QDomNode layerNode = doc.createElement( "layer" );
-			LocalFileMng::writeXmlString( layerNode, "filename", prepare_filename( pSample->get_filepath() ) );
-			LocalFileMng::writeXmlBool( layerNode, "ismodified", sIsModified);
-			LocalFileMng::writeXmlString( layerNode, "smode", pSample->get_loop_mode_string() );
-			LocalFileMng::writeXmlString( layerNode, "startframe", QString("%1").arg( lo.start_frame ) );
-			LocalFileMng::writeXmlString( layerNode, "loopframe", QString("%1").arg( lo.loop_frame ) );
-			LocalFileMng::writeXmlString( layerNode, "loops", QString("%1").arg( lo.count ) );
-			LocalFileMng::writeXmlString( layerNode, "endframe", QString("%1").arg( lo.end_frame ) );
-			LocalFileMng::writeXmlString( layerNode, "userubber", QString("%1").arg( ro.use ) );
-			LocalFileMng::writeXmlString( layerNode, "rubberdivider", QString("%1").arg( ro.divider ) );
-			LocalFileMng::writeXmlString( layerNode, "rubberCsettings", QString("%1").arg( ro.c_settings ) );
-			LocalFileMng::writeXmlString( layerNode, "rubberPitch", QString("%1").arg( ro.pitch ) );
-			LocalFileMng::writeXmlString( layerNode, "min", QString("%1").arg( pLayer->get_start_velocity() ) );
-			LocalFileMng::writeXmlString( layerNode, "max", QString("%1").arg( pLayer->get_end_velocity() ) );
-			LocalFileMng::writeXmlString( layerNode, "gain", QString("%1").arg( pLayer->get_gain() ) );
-			LocalFileMng::writeXmlString( layerNode, "pitch", QString("%1").arg( pLayer->get_pitch() ) );
+				QDomNode layerNode = doc.createElement( "layer" );
+				LocalFileMng::writeXmlString( layerNode, "filename", prepare_filename( pSample->get_filepath() ) );
+				LocalFileMng::writeXmlBool( layerNode, "ismodified", sIsModified);
+				LocalFileMng::writeXmlString( layerNode, "smode", pSample->get_loop_mode_string() );
+				LocalFileMng::writeXmlString( layerNode, "startframe", QString("%1").arg( lo.start_frame ) );
+				LocalFileMng::writeXmlString( layerNode, "loopframe", QString("%1").arg( lo.loop_frame ) );
+				LocalFileMng::writeXmlString( layerNode, "loops", QString("%1").arg( lo.count ) );
+				LocalFileMng::writeXmlString( layerNode, "endframe", QString("%1").arg( lo.end_frame ) );
+				LocalFileMng::writeXmlString( layerNode, "userubber", QString("%1").arg( ro.use ) );
+				LocalFileMng::writeXmlString( layerNode, "rubberdivider", QString("%1").arg( ro.divider ) );
+				LocalFileMng::writeXmlString( layerNode, "rubberCsettings", QString("%1").arg( ro.c_settings ) );
+				LocalFileMng::writeXmlString( layerNode, "rubberPitch", QString("%1").arg( ro.pitch ) );
+				LocalFileMng::writeXmlString( layerNode, "min", QString("%1").arg( pLayer->get_start_velocity() ) );
+				LocalFileMng::writeXmlString( layerNode, "max", QString("%1").arg( pLayer->get_end_velocity() ) );
+				LocalFileMng::writeXmlString( layerNode, "gain", QString("%1").arg( pLayer->get_gain() ) );
+				LocalFileMng::writeXmlString( layerNode, "pitch", QString("%1").arg( pLayer->get_pitch() ) );
 
 
-			Sample::VelocityEnvelope* velocity = pSample->get_velocity_envelope();
-			for (int y = 0; y < velocity->size(); y++){
-				QDomNode volumeNode = doc.createElement( "volume" );
-				LocalFileMng::writeXmlString( volumeNode, "volume-position", QString("%1").arg( velocity->at(y).frame ) );
-				LocalFileMng::writeXmlString( volumeNode, "volume-value", QString("%1").arg( velocity->at(y).value ) );
-				layerNode.appendChild( volumeNode );
+				Sample::VelocityEnvelope* velocity = pSample->get_velocity_envelope();
+				for (int y = 0; y < velocity->size(); y++){
+					QDomNode volumeNode = doc.createElement( "volume" );
+					LocalFileMng::writeXmlString( volumeNode, "volume-position", QString("%1").arg( velocity->at(y).frame ) );
+					LocalFileMng::writeXmlString( volumeNode, "volume-value", QString("%1").arg( velocity->at(y).value ) );
+					layerNode.appendChild( volumeNode );
+				}
+
+				Sample::PanEnvelope* pan = pSample->get_pan_envelope();
+				for (int y = 0; y < pan->size(); y++){
+					QDomNode panNode = doc.createElement( "pan" );
+					LocalFileMng::writeXmlString( panNode, "pan-position", QString("%1").arg( pan->at(y).frame ) );
+					LocalFileMng::writeXmlString( panNode, "pan-value", QString("%1").arg( pan->at(y).value ) );
+					layerNode.appendChild( panNode );
+				}
+
+				componentNode.appendChild( layerNode );
 			}
-
-			Sample::PanEnvelope* pan = pSample->get_pan_envelope();
-			for (int y = 0; y < pan->size(); y++){
-				QDomNode panNode = doc.createElement( "pan" );
-				LocalFileMng::writeXmlString( panNode, "pan-position", QString("%1").arg( pan->at(y).frame ) );
-				LocalFileMng::writeXmlString( panNode, "pan-value", QString("%1").arg( pan->at(y).value ) );
-				layerNode.appendChild( panNode );
-			}
-
-			instrumentNode.appendChild( layerNode );
+			instrumentNode.appendChild( componentNode );
 		}
 
 		instrumentListNode.appendChild( instrumentNode );
@@ -1362,6 +1428,7 @@ int SongWriter::writeSong( Song *song, const QString& filename )
 		LocalFileMng::writeXmlString( patternNode, "name", pat->get_name() );
 		LocalFileMng::writeXmlString( patternNode, "category", pat->get_category() );
 		LocalFileMng::writeXmlString( patternNode, "size", QString("%1").arg( pat->get_length() ) );
+		LocalFileMng::writeXmlString( patternNode, "info", pat->get_info() );
 
 		QDomNode noteListNode = doc.createElement( "noteList" );
 		const Pattern::notes_t* notes = pat->get_notes();
@@ -1476,12 +1543,15 @@ int SongWriter::writeSong( Song *song, const QString& filename )
 
 
 	//bpm time line
+	Timeline * pTimeline = Hydrogen::get_instance()->getTimeline();
+
 	QDomNode bpmTimeLine = doc.createElement( "BPMTimeLine" );
-	if(Hydrogen::get_instance()->m_timelinevector.size() >= 1 ){
-		for ( int t = 0; t < static_cast<int>(Hydrogen::get_instance()->m_timelinevector.size()); t++){
+
+	if(pTimeline->m_timelinevector.size() >= 1 ){
+		for ( int t = 0; t < static_cast<int>(pTimeline->m_timelinevector.size()); t++){
 			QDomNode newBPMNode = doc.createElement( "newBPM" );
-			LocalFileMng::writeXmlString( newBPMNode, "BAR",QString("%1").arg( Hydrogen::get_instance()->m_timelinevector[t].m_htimelinebeat ));
-			LocalFileMng::writeXmlString( newBPMNode, "BPM", QString("%1").arg( Hydrogen::get_instance()->m_timelinevector[t].m_htimelinebpm  ) );
+			LocalFileMng::writeXmlString( newBPMNode, "BAR",QString("%1").arg( pTimeline->m_timelinevector[t].m_htimelinebeat ));
+			LocalFileMng::writeXmlString( newBPMNode, "BPM", QString("%1").arg( pTimeline->m_timelinevector[t].m_htimelinebpm  ) );
 			bpmTimeLine.appendChild( newBPMNode );
 		}
 	}
@@ -1489,11 +1559,11 @@ int SongWriter::writeSong( Song *song, const QString& filename )
 
 	//time line tag
 	QDomNode timeLineTag = doc.createElement( "timeLineTag" );
-	if(Hydrogen::get_instance()->m_timelinetagvector.size() >= 1 ){
-		for ( int t = 0; t < static_cast<int>(Hydrogen::get_instance()->m_timelinetagvector.size()); t++){
+	if(pTimeline->m_timelinetagvector.size() >= 1 ){
+		for ( int t = 0; t < static_cast<int>(pTimeline->m_timelinetagvector.size()); t++){
 			QDomNode newTAGNode = doc.createElement( "newTAG" );
-			LocalFileMng::writeXmlString( newTAGNode, "BAR",QString("%1").arg( Hydrogen::get_instance()->m_timelinetagvector[t].m_htimelinetagbeat ));
-			LocalFileMng::writeXmlString( newTAGNode, "TAG", QString("%1").arg( Hydrogen::get_instance()->m_timelinetagvector[t].m_htimelinetag  ) );
+			LocalFileMng::writeXmlString( newTAGNode, "BAR",QString("%1").arg( pTimeline->m_timelinetagvector[t].m_htimelinetagbeat ));
+			LocalFileMng::writeXmlString( newTAGNode, "TAG", QString("%1").arg( pTimeline->m_timelinetagvector[t].m_htimelinetag  ) );
 			timeLineTag.appendChild( newTAGNode );
 		}
 	}
@@ -1514,7 +1584,7 @@ int SongWriter::writeSong( Song *song, const QString& filename )
 	if( rv ) {
 		WARNINGLOG("File save reported an error.");
 	} else {
-		song->__is_modified = false;
+		song->set_is_modified( false );
 		INFOLOG("Save was successful.");
 	}
 
