@@ -61,7 +61,6 @@ const char* Preferences::__class_name = "Preferences";
 Preferences::Preferences()
 		: Object( __class_name )
 		, demoPath( Filesystem::demos_dir()+"/")
-		, m_sLastNews( "" )
 {
 	__instance = this;
 	INFOLOG( "INIT" );
@@ -80,12 +79,9 @@ Preferences::Preferences()
 	//server list
 	std::list<QString> sServerList;
 
-//	//musicCategories
-//	std::list<QString> m_musicCategories;
-
 	//rubberband bpm change queue
 	m_useTheRubberbandBpmChangeEvent = false;
-		__rubberBandCalcTime = 5;
+	__rubberBandCalcTime = 5;
 
 	QString rubberBandCLIPath = getenv( "PATH" );
 	QStringList rubberBandCLIPathList = rubberBandCLIPath.split(":");//linx use ":" as seperator. maybe windows and osx use other seperators
@@ -130,10 +126,8 @@ Preferences::Preferences()
 	}
 
 
-
-
 	m_pDefaultUIStyle = new UIStyle();
-		m_nDefaultUILayout = UI_LAYOUT_SINGLE_PANE;
+	m_nDefaultUILayout = UI_LAYOUT_SINGLE_PANE;
 
 #ifdef Q_OS_MACX
 	m_sPreferencesFilename = QDir::homePath().append( "/Library/Application Support/Hydrogen/hydrogen.conf" );
@@ -161,7 +155,7 @@ Preferences::Preferences()
 
 	__expandSongItem = true; //SoundLibraryPanel
 	__expandPatternItem = true; //SoundLibraryPanel
-		__useTimelineBpm = false;		// use timeline
+	__useTimelineBpm = false;		// use timeline
 
 
 	/////////////////////////////////////////////////////////////////////////
@@ -196,6 +190,7 @@ Preferences::Preferences()
 	m_sMidiPortName = QString("None");
 	m_nMidiChannelFilter = -1;
 	m_bMidiNoteOffIgnore = false;
+	m_bMidiFixedMapping = false;
 	m_bMidiDiscardNoteAfterAction = false;
 
 	//___  alsa audio driver properties ___
@@ -228,7 +223,7 @@ Preferences::Preferences()
 	// NONE: m_ladspaPathVect;
 	quantizeEvents = true;
 	recordEvents = false;
-	m_sLastNews = QString("-");
+	m_bUseRelativeFilenamesForPlaylists = false;
 
 	//___ GUI properties ___
 	m_sQTStyle = "Plastique";
@@ -254,6 +249,10 @@ Preferences::Preferences()
 	m_ladspaProperties[2].set(2, 20, 0, 0, false);
 	m_ladspaProperties[3].set(2, 20, 0, 0, false);
 
+	m_nColoringMethod = 2;
+	m_nColoringMethodAuxValue = 213;
+
+
 	UIStyle* uis = m_pDefaultUIStyle;
 	uis->m_songEditor_backgroundColor = H2RGBColor(95, 101, 117);
 	uis->m_songEditor_alternateRowColor = H2RGBColor(128, 134, 152);
@@ -316,8 +315,6 @@ void Preferences::loadPreferences( bool bGlobal )
 		sPreferencesDirectory = m_sPreferencesDirectory;
 		sDataDirectory = QDir::homePath().append( "/.hydrogen/data" );
 		INFOLOG( "Loading preferences file (USER) [" + sPreferencesFilename + "]" );
-
-
 	}
 
 	// preferences directory exists?
@@ -377,19 +374,21 @@ void Preferences::loadPreferences( bool bGlobal )
 
 			//////// GENERAL ///////////
 			//m_sLadspaPath = LocalFileMng::readXmlString( this, rootNode, "ladspaPath", m_sLadspaPath );
+			__playselectedinstrument = LocalFileMng::readXmlBool( rootNode, "instrumentInputMode", __playselectedinstrument );
 			m_bShowDevelWarning = LocalFileMng::readXmlBool( rootNode, "showDevelWarning", m_bShowDevelWarning );
 			m_brestoreLastSong = LocalFileMng::readXmlBool( rootNode, "restoreLastSong", m_brestoreLastSong );
 			m_brestoreLastPlaylist = LocalFileMng::readXmlBool( rootNode, "restoreLastPlaylist", m_brestoreLastPlaylist );
-			m_bPatternModePlaysSelected = LocalFileMng::readXmlBool( rootNode, "patternModePlaysSelected", TRUE );
-			m_bUseLash = LocalFileMng::readXmlBool( rootNode, "useLash", FALSE );
-						__useTimelineBpm = LocalFileMng::readXmlBool( rootNode, "useTimeLine", __useTimelineBpm );
+			m_bPatternModePlaysSelected = LocalFileMng::readXmlBool( rootNode, "patternModePlaysSelected", true );
+			m_bUseLash = LocalFileMng::readXmlBool( rootNode, "useLash", false );
+			__useTimelineBpm = LocalFileMng::readXmlBool( rootNode, "useTimeLine", __useTimelineBpm );
 			maxBars = LocalFileMng::readXmlInt( rootNode, "maxBars", 400 );
-						m_nDefaultUILayout =  LocalFileMng::readXmlInt( rootNode, "defaultUILayout", UI_LAYOUT_SINGLE_PANE );
-						m_nLastOpenTab =  LocalFileMng::readXmlInt( rootNode, "lastOpenTab", 0 );
+			m_nDefaultUILayout =  LocalFileMng::readXmlInt( rootNode, "defaultUILayout", UI_LAYOUT_SINGLE_PANE );
+			m_nLastOpenTab =  LocalFileMng::readXmlInt( rootNode, "lastOpenTab", 0 );
+			m_bUseRelativeFilenamesForPlaylists = LocalFileMng::readXmlBool( rootNode, "useRelativeFilenamesForPlaylists", false );
 
 			//restore the right m_bsetlash value
 			m_bsetLash = m_bUseLash;
-					   m_useTheRubberbandBpmChangeEvent = LocalFileMng::readXmlBool( rootNode, "useTheRubberbandBpmChangeEvent", m_useTheRubberbandBpmChangeEvent );
+			m_useTheRubberbandBpmChangeEvent = LocalFileMng::readXmlBool( rootNode, "useTheRubberbandBpmChangeEvent", m_useTheRubberbandBpmChangeEvent );
 			m_nRecPreDelete = LocalFileMng::readXmlInt( rootNode, "preDelete", 0 );
 			m_nRecPostDelete = LocalFileMng::readXmlInt( rootNode, "postDelete", 0 );
 
@@ -456,9 +455,6 @@ void Preferences::loadPreferences( bool bGlobal )
 			}
 
 
-
-			m_sLastNews = LocalFileMng::readXmlString( rootNode, "lastNews", "-", true );
-
 			/////////////// AUDIO ENGINE //////////////
 			QDomNode audioEngineNode = rootNode.firstChildElement( "audio_engine" );
 			if ( audioEngineNode.isNull() ) {
@@ -531,6 +527,8 @@ void Preferences::loadPreferences( bool bGlobal )
 					m_sMidiPortName = LocalFileMng::readXmlString( midiDriverNode, "port_name", "None" );
 					m_nMidiChannelFilter = LocalFileMng::readXmlInt( midiDriverNode, "channel_filter", -1 );
 					m_bMidiNoteOffIgnore = LocalFileMng::readXmlBool( midiDriverNode, "ignore_note_off", true );
+					m_bMidiDiscardNoteAfterAction = LocalFileMng::readXmlBool( midiDriverNode, "discard_note_after_action", true);
+					m_bMidiFixedMapping = LocalFileMng::readXmlBool( midiDriverNode, "fixed_mapping", false, true );
 				}
 
 
@@ -623,6 +621,11 @@ void Preferences::loadPreferences( bool bGlobal )
 					WARNINGLOG( "UI_Style node not found" );
 					recreate = true;
 				}
+
+				//SongEditor coloring
+				m_nColoringMethod = LocalFileMng::readXmlInt( guiNode, "SongEditor_ColoringMethod", 2 );
+				m_nColoringMethodAuxValue = LocalFileMng::readXmlInt( guiNode, "SongEditor_ColoringMethodAuxValue", 213 );
+
 			}
 
 			/////////////// FILES //////////////
@@ -719,7 +722,6 @@ void Preferences::loadPreferences( bool bGlobal )
 ///
 void Preferences::savePreferences()
 {
-	//string prefDir = QDir::homePath().append("/.hydrogen").toLocal8Bit().constData();
 	QString filename = m_sPreferencesFilename;
 
 	INFOLOG( "Saving preferences file: " + filename );
@@ -740,18 +742,22 @@ void Preferences::savePreferences()
 	LocalFileMng::writeXmlString( rootNode, "patternModePlaysSelected", m_bPatternModePlaysSelected ? "true": "false" );
 
 	LocalFileMng::writeXmlString( rootNode, "useLash", m_bsetLash ? "true": "false" );
-		LocalFileMng::writeXmlString( rootNode, "useTimeLine", __useTimelineBpm ? "true": "false" );
+	LocalFileMng::writeXmlString( rootNode, "useTimeLine", __useTimelineBpm ? "true": "false" );
 
 	LocalFileMng::writeXmlString( rootNode, "maxBars", QString::number( maxBars ) );
 
-		LocalFileMng::writeXmlString( rootNode, "defaultUILayout", QString::number( m_nDefaultUILayout ) );
-		LocalFileMng::writeXmlString( rootNode, "lastOpenTab", QString::number( m_nLastOpenTab ) );
+	LocalFileMng::writeXmlString( rootNode, "defaultUILayout", QString::number( m_nDefaultUILayout ) );
+	LocalFileMng::writeXmlString( rootNode, "lastOpenTab", QString::number( m_nLastOpenTab ) );
 
-		LocalFileMng::writeXmlString( rootNode, "useTheRubberbandBpmChangeEvent", m_useTheRubberbandBpmChangeEvent ? "true": "false" );
+	LocalFileMng::writeXmlString( rootNode, "useTheRubberbandBpmChangeEvent", m_useTheRubberbandBpmChangeEvent ? "true": "false" );
 
 	LocalFileMng::writeXmlString( rootNode, "preDelete", QString("%1").arg(m_nRecPreDelete) );
 	LocalFileMng::writeXmlString( rootNode, "postDelete", QString("%1").arg(m_nRecPostDelete) );
-
+	LocalFileMng::writeXmlString( rootNode, "useRelativeFilenamesForPlaylists", m_bUseRelativeFilenamesForPlaylists ? "true": "false" );
+	
+	// instrument input mode
+	LocalFileMng::writeXmlString( rootNode, "instrumentInputMode", __playselectedinstrument ? "true": "false" );
+	
 	//show development version warning
 	LocalFileMng::writeXmlString( rootNode, "showDevelWarning", m_bShowDevelWarning ? "true": "false" );
 
@@ -812,10 +818,6 @@ void Preferences::savePreferences()
 
 
 
-
-	LocalFileMng::writeXmlString( rootNode, "lastNews", m_sLastNews );
-
-
 	//---- AUDIO ENGINE ----
 	QDomNode audioEngineNode = doc.createElement( "audio_engine" );
 	{
@@ -905,6 +907,14 @@ void Preferences::savePreferences()
 			} else {
 				LocalFileMng::writeXmlString( midiDriverNode, "discard_note_after_action", "false" );
 			}
+
+			if ( m_bMidiFixedMapping ) {
+				LocalFileMng::writeXmlString( midiDriverNode, "fixed_mapping", "true" );
+				INFOLOG("Saving fixed mapping\n");
+			} else {
+				LocalFileMng::writeXmlString( midiDriverNode, "fixed_mapping", "false" );
+				INFOLOG("Saving fixed mapping false\n");
+			}
 		}
 		audioEngineNode.appendChild( midiDriverNode );
 
@@ -947,25 +957,26 @@ void Preferences::savePreferences()
 
 		//beatcounter
 		QString bcMode;
-			if ( m_bbc == BC_OFF ) {
-				bcMode = "BC_OFF";
-			} else if ( m_bbc  == BC_ON ) {
-				bcMode = "BC_ON";
-			}
-			LocalFileMng::writeXmlString( guiNode, "bc", bcMode );
+
+		if ( m_bbc == BC_OFF ) {
+			bcMode = "BC_OFF";
+		} else if ( m_bbc  == BC_ON ) {
+			bcMode = "BC_ON";
+		}
+		LocalFileMng::writeXmlString( guiNode, "bc", bcMode );
 
 
 
 		QString setPlay;
-			if ( m_mmcsetplay == SET_PLAY_OFF ) {
-				setPlay = "SET_PLAY_OFF";
-			} else if ( m_mmcsetplay == SET_PLAY_ON ) {
-				setPlay = "SET_PLAY_ON";
-			}
-			LocalFileMng::writeXmlString( guiNode, "setplay", setPlay );
+		if ( m_mmcsetplay == SET_PLAY_OFF ) {
+			setPlay = "SET_PLAY_OFF";
+		} else if ( m_mmcsetplay == SET_PLAY_ON ) {
+			setPlay = "SET_PLAY_ON";
+		}
+		LocalFileMng::writeXmlString( guiNode, "setplay", setPlay );
 
-			LocalFileMng::writeXmlString( guiNode, "countoffset", QString("%1").arg(m_countOffset) );
-			LocalFileMng::writeXmlString( guiNode, "playoffset", QString("%1").arg(m_startOffset) );
+		LocalFileMng::writeXmlString( guiNode, "countoffset", QString("%1").arg(m_countOffset) );
+		LocalFileMng::writeXmlString( guiNode, "playoffset", QString("%1").arg(m_startOffset) );
 		//~ beatcounter
 
 
@@ -975,6 +986,11 @@ void Preferences::savePreferences()
 
 		// User interface style
 		writeUIStyle( guiNode );
+
+		//SongEditor coloring method
+		LocalFileMng::writeXmlString( guiNode, "SongEditor_ColoringMethod", QString::number( m_nColoringMethod ) );
+		LocalFileMng::writeXmlString( guiNode, "SongEditor_ColoringMethodAuxValue", QString::number( m_nColoringMethodAuxValue ) );
+
 	}
 	rootNode.appendChild( guiNode );
 
@@ -989,7 +1005,7 @@ void Preferences::savePreferences()
 	rootNode.appendChild( filesNode );
 
 	MidiMap * mM = MidiMap::get_instance();
-		std::map< QString, MidiAction* > mmcMap = mM->getMMCMap();
+	std::map< QString, MidiAction* > mmcMap = mM->getMMCMap();
 
 	//---- MidiMap ----
 	QDomNode midiEventMapNode = doc.createElement( "midiEventMap" );
@@ -1344,23 +1360,6 @@ H2RGBColor::H2RGBColor( const QString& sColor )
 	m_red %= 256;
 	m_green %= 256;
 	m_blue %= 256;
-
-/*
-	int nPos = temp.indexOf( ',' );
-	QString sRed = temp.substr( 0, nPos );
-	temp.erase( 0, nPos + 1 );
-
-	nPos = temp.find( ',' );
-	QString sGreen = temp.substr( 0, nPos );
-	temp.erase( 0, nPos + 1 );
-
-	nPos = temp.find( ',' );
-	QString sBlue = temp.substr( 0, nPos );
-
-	m_red = atoi( sRed.c_str() );
-	m_green = atoi( sGreen.c_str() );
-	m_blue = atoi( sBlue.c_str() );
-*/
 }
 
 
@@ -1370,9 +1369,6 @@ QString H2RGBColor::toStringFmt()
 	char tmp[255];
 	sprintf( tmp, "%d,%d,%d", m_red, m_green, m_blue );
 
-	//string sRes = to_string( m_red ) + "," + to_string( m_green ) + "," + to_string( m_blue );
-//	return sRes;
-
 	return QString( tmp );
 }
 
