@@ -22,6 +22,7 @@
 
 #include <cassert>
 #include <cmath>
+#include <cstdlib>
 
 #include <hydrogen/IO/AudioOutput.h>
 #include <hydrogen/IO/JackOutput.h>
@@ -30,7 +31,9 @@
 #include <hydrogen/audio_engine.h>
 #include <hydrogen/globals.h>
 #include <hydrogen/hydrogen.h>
+#include <hydrogen/basics/drumkit_component.h>
 #include <hydrogen/basics/instrument.h>
+#include <hydrogen/basics/instrument_component.h>
 #include <hydrogen/basics/instrument_list.h>
 #include <hydrogen/basics/instrument_layer.h>
 #include <hydrogen/basics/note.h>
@@ -67,9 +70,12 @@ Sampler::Sampler()
 	// instrument used in file preview
 	QString sEmptySampleFilename = Filesystem::empty_sample();
 	__preview_instrument = new Instrument( EMPTY_INSTR_ID, sEmptySampleFilename );
+	__preview_instrument->set_is_preview_instrument(true);
 	__preview_instrument->set_volume( 0.8 );
-	__preview_instrument->set_layer( new InstrumentLayer( Sample::load( sEmptySampleFilename ) ), 0 );
-
+	InstrumentLayer* pLayer = new InstrumentLayer( Sample::load( sEmptySampleFilename ) );
+	InstrumentComponent* pComponent = new InstrumentComponent( 0 );
+	pComponent->set_layer( pLayer, 0 );
+	__preview_instrument->get_components()->push_back( pComponent );
 }
 
 
@@ -106,14 +112,18 @@ void Sampler::process( uint32_t nFrames, Song* pSong )
 		delete oldNote;	// FIXME: send note-off instead of removing the note from the list?
 	}
 
+	for (std::vector<DrumkitComponent*>::iterator it = pSong->get_components()->begin() ; it != pSong->get_components()->end(); ++it) {
+		DrumkitComponent* component = *it;
+		component->reset_outs(nFrames);
+	}
+
 
 	// eseguo tutte le note nella lista di note in esecuzione
 	unsigned i = 0;
 	Note* pNote;
 	while ( i < __playing_notes_queue.size() ) {
 		pNote = __playing_notes_queue[ i ];		// recupero una nuova nota
-		unsigned res = __render_note( pNote, nFrames, pSong );
-		if ( res == 1 ) {	// la nota e' finita
+		if ( __render_note( pNote, nFrames, pSong ) ) {	// la nota e' finita
 			__playing_notes_queue.erase( __playing_notes_queue.begin() + i );
 			pNote->get_instrument()->dequeue();
 			__queuedNoteOffs.push_back( pNote );
@@ -213,9 +223,9 @@ void Sampler::note_off( Note* note )
 
 
 /// Render a note
-/// Return 0: the note is not ended
-/// Return 1: the note is ended
-unsigned Sampler::__render_note( Note* pNote, unsigned nBufferSize, Song* pSong )
+/// Return false: the note is not ended
+/// Return true: the note is ended
+bool Sampler::__render_note( Note* pNote, unsigned nBufferSize, Song* pSong )
 {
 	//infoLog( "[renderNote] instr: " + pNote->getInstrument()->m_sName );
 	assert( pSong );
@@ -236,130 +246,278 @@ unsigned Sampler::__render_note( Note* pNote, unsigned nBufferSize, Song* pSong
 		return 1;
 	}
 
-	float fLayerGain = 1.0;
-	float fLayerPitch = 0.0;
+	bool nReturnValues [pInstr->get_components()->size()];
+	int nReturnValueIndex = 0;
+	int nAlreadySelectedLayer = -1;
 
-	// scelgo il sample da usare in base alla velocity
-	Sample *pSample = NULL;
-	for ( unsigned nLayer = 0; nLayer < MAX_LAYERS; ++nLayer ) {
-		InstrumentLayer *pLayer = pInstr->get_layer( nLayer );
-		if ( pLayer == NULL ) continue;
+	for (std::vector<InstrumentComponent*>::iterator it = pInstr->get_components()->begin() ; it !=pInstr->get_components()->end(); ++it) {
+		nReturnValues[nReturnValueIndex] = false;
+		InstrumentComponent *pCompo = *it;
+		DrumkitComponent* pMainCompo = pEngine->getSong()->get_component( pCompo->get_drumkit_componentID() );
+
+		if( pNote->get_specific_compo_id() != -1 && pNote->get_specific_compo_id() != pCompo->get_drumkit_componentID() )
+			continue;
+
+		if(		pInstr->is_preview_instrument()
+			||	pInstr->is_metronome_instrument()){
+			pMainCompo = pEngine->getSong()->get_components()->front();
+		} else {
+			pMainCompo = pEngine->getSong()->get_component( pCompo->get_drumkit_componentID() );
+		}
+
+		assert(pMainCompo);
+
+		float fLayerGain = 1.0;
+		float fLayerPitch = 0.0;
+
+		// scelgo il sample da usare in base alla velocity
+		Sample *pSample = NULL;
+		SelectedLayerInfo *pSelectedLayer = pNote->get_layer_selected( pCompo->get_drumkit_componentID() );
+
+		if ( !pSelectedLayer ) {
+			QString dummy = QString( "NULL Layer Informationfor instrument %1. Component: %2" ).arg( pInstr->get_name() ).arg( pCompo->get_drumkit_componentID() );
+			WARNINGLOG( dummy );
+			nReturnValues[nReturnValueIndex] = true;
+			continue;
+		}
+
+		if( pSelectedLayer->SelectedLayer != -1 ) {
+			InstrumentLayer *pLayer = pCompo->get_layer( pSelectedLayer->SelectedLayer );
 
-		if ( ( pNote->get_velocity() >= pLayer->get_start_velocity() ) && ( pNote->get_velocity() <= pLayer->get_end_velocity() ) ) {
 			pSample = pLayer->get_sample();
 			fLayerGain = pLayer->get_gain();
 			fLayerPitch = pLayer->get_pitch();
-			break;
 		}
-	}
-	if ( !pSample ) {
-		QString dummy = QString( "NULL sample for instrument %1. Note velocity: %2" ).arg( pInstr->get_name() ).arg( pNote->get_velocity() );
-		WARNINGLOG( dummy );
-		return 1;
-	}
+		else {
+			switch ( pInstr->sample_selection_alg() ) {
+				case Instrument::VELOCITY:
+					for ( unsigned nLayer = 0; nLayer < MAX_LAYERS; ++nLayer ) {
+						InstrumentLayer *pLayer = pCompo->get_layer( nLayer );
+						if ( pLayer == NULL ) continue;
 
-	if ( pNote->get_sample_position() >= pSample->get_frames() ) {
-		WARNINGLOG( "sample position out of bounds. The layer has been resized during note play?" );
-		return 1;
-	}
+						if ( ( pNote->get_velocity() >= pLayer->get_start_velocity() ) && ( pNote->get_velocity() <= pLayer->get_end_velocity() ) ) {
+							pSelectedLayer->SelectedLayer = nLayer;
 
-	int noteStartInFrames = ( int ) ( pNote->get_position() * audio_output->m_transport.m_nTickSize ) + pNote->get_humanize_delay();
+							pSample = pLayer->get_sample();
+							fLayerGain = pLayer->get_gain();
+							fLayerPitch = pLayer->get_pitch();
+							break;
+						}
+					}
+					break;
 
-	int nInitialSilence = 0;
-	if ( noteStartInFrames > ( int ) nFramepos ) {	// scrivo silenzio prima dell'inizio della nota
-		nInitialSilence = noteStartInFrames - nFramepos;
-		int nFrames = nBufferSize - nInitialSilence;
-		if ( nFrames < 0 ) {
-			int noteStartInFramesNoHumanize = ( int )pNote->get_position() * audio_output->m_transport.m_nTickSize;
-			if ( noteStartInFramesNoHumanize > ( int )( nFramepos + nBufferSize ) ) {
-				// this note is not valid. it's in the future...let's skip it....
-				ERRORLOG( QString( "Note pos in the future?? Current frames: %1, note frame pos: %2" ).arg( nFramepos ).arg(noteStartInFramesNoHumanize ) );
-				//pNote->dumpInfo();
-				return 1;
+				case Instrument::RANDOM:
+					if( nAlreadySelectedLayer != -1 ) {
+						InstrumentLayer *pLayer = pCompo->get_layer( nAlreadySelectedLayer );
+						if ( pLayer != NULL ) {
+							pSelectedLayer->SelectedLayer = nAlreadySelectedLayer;
+
+							pSample = pLayer->get_sample();
+							fLayerGain = pLayer->get_gain();
+							fLayerPitch = pLayer->get_pitch();
+						}
+					}
+					if( pSample == NULL ) {
+						int __possibleIndex[MAX_LAYERS];
+						int __poundSamples = 0;
+						for ( unsigned nLayer = 0; nLayer < MAX_LAYERS; ++nLayer ) {
+							InstrumentLayer *pLayer = pCompo->get_layer( nLayer );
+							if ( pLayer == NULL ) continue;
+
+							if ( ( pNote->get_velocity() >= pLayer->get_start_velocity() ) && ( pNote->get_velocity() <= pLayer->get_end_velocity() ) ) {
+								__possibleIndex[__poundSamples] = nLayer;
+								__poundSamples++;
+							}
+						}
+
+						if( __poundSamples > 0 ) {
+							nAlreadySelectedLayer = __possibleIndex[rand() % __poundSamples];
+							pSelectedLayer->SelectedLayer = nAlreadySelectedLayer;
+
+							InstrumentLayer *pLayer = pCompo->get_layer( nAlreadySelectedLayer );
+
+							pSample = pLayer->get_sample();
+							fLayerGain = pLayer->get_gain();
+							fLayerPitch = pLayer->get_pitch();
+						}
+					}
+					break;
+
+				case Instrument::ROUND_ROBIN:
+					if( nAlreadySelectedLayer != -1 ) {
+						InstrumentLayer *pLayer = pCompo->get_layer( nAlreadySelectedLayer );
+						if ( pLayer != NULL ) {
+							pSelectedLayer->SelectedLayer = nAlreadySelectedLayer;
+
+							pSample = pLayer->get_sample();
+							fLayerGain = pLayer->get_gain();
+							fLayerPitch = pLayer->get_pitch();
+						}
+					}
+					if( !pSample ) {
+						int __possibleIndex[MAX_LAYERS];
+						int __foundSamples = 0;
+						float __roundRobinID;
+						for ( unsigned nLayer = 0; nLayer < MAX_LAYERS; ++nLayer ) {
+							InstrumentLayer *pLayer = pCompo->get_layer( nLayer );
+							if ( pLayer == NULL ) continue;
+
+							if ( ( pNote->get_velocity() >= pLayer->get_start_velocity() ) && ( pNote->get_velocity() <= pLayer->get_end_velocity() ) ) {
+								__possibleIndex[__foundSamples] = nLayer;
+								__roundRobinID = pLayer->get_start_velocity();
+								__foundSamples++;
+							}
+						}
+
+						if( __foundSamples > 0 ) {
+							__roundRobinID = pInstr->get_id() * 10 + __roundRobinID;
+							int p_indexToUse = pSong->get_latest_round_robin(__roundRobinID)+1;
+							if( p_indexToUse > __foundSamples - 1)
+								p_indexToUse = 0;
+
+							pSong->set_latest_round_robin(__roundRobinID, p_indexToUse);
+							nAlreadySelectedLayer = __possibleIndex[p_indexToUse];
+
+							pSelectedLayer->SelectedLayer = nAlreadySelectedLayer;
+
+							InstrumentLayer *pLayer = pCompo->get_layer( nAlreadySelectedLayer );
+							pSample = pLayer->get_sample();
+							fLayerGain = pLayer->get_gain();
+							fLayerPitch = pLayer->get_pitch();
+						}
+					}
+					break;
 			}
-			// delay note execution
-			//INFOLOG( "Delaying note execution. noteStartInFrames: " + to_string( noteStartInFrames ) + ", nFramePos: " + to_string( nFramepos ) );
-			return 0;
 		}
-	}
-
-	float cost_L = 1.0f;
-	float cost_R = 1.0f;
-	float cost_track_L = 1.0f;
-	float cost_track_R = 1.0f;
-
-	if ( pInstr->is_muted() || pSong->__is_muted ) {	// is instrument muted?
-		cost_L = 0.0;
-		cost_R = 0.0;
-		if ( Preferences::get_instance()->m_nJackTrackOutputMode == 0 ) {
-		// Post-Fader
-		cost_track_L = 0.0;
-		cost_track_R = 0.0;
+		if ( !pSample ) {
+			QString dummy = QString( "NULL sample for instrument %1. Note velocity: %2" ).arg( pInstr->get_name() ).arg( pNote->get_velocity() );
+			WARNINGLOG( dummy );
+			nReturnValues[nReturnValueIndex] = true;
+			continue;
 		}
 
-	} else {	// Precompute some values...
-		cost_L = cost_L * pNote->get_velocity();		// note velocity
-		cost_L = cost_L * pNote->get_pan_l();		// note pan
-		cost_L = cost_L * fLayerGain;				// layer gain
-		cost_L = cost_L * pInstr->get_pan_l();		// instrument pan
-		cost_L = cost_L * pInstr->get_gain();		// instrument gain
+		if ( pSelectedLayer->SamplePosition >= pSample->get_frames() ) {
+			WARNINGLOG( "sample position out of bounds. The layer has been resized during note play?" );
+			nReturnValues[nReturnValueIndex] = true;
+			continue;
+		}
 
-		cost_L = cost_L * pInstr->get_volume();		// instrument volume
-		if ( Preferences::get_instance()->m_nJackTrackOutputMode == 0 ) {
+		int noteStartInFrames = ( int ) ( pNote->get_position() * audio_output->m_transport.m_nTickSize ) + pNote->get_humanize_delay();
+
+		int nInitialSilence = 0;
+		if ( noteStartInFrames > ( int ) nFramepos ) {	// scrivo silenzio prima dell'inizio della nota
+			nInitialSilence = noteStartInFrames - nFramepos;
+			int nFrames = nBufferSize - nInitialSilence;
+			if ( nFrames < 0 ) {
+				int noteStartInFramesNoHumanize = ( int )pNote->get_position() * audio_output->m_transport.m_nTickSize;
+				if ( noteStartInFramesNoHumanize > ( int )( nFramepos + nBufferSize ) ) {
+					// this note is not valid. it's in the future...let's skip it....
+					ERRORLOG( QString( "Note pos in the future?? Current frames: %1, note frame pos: %2" ).arg( nFramepos ).arg(noteStartInFramesNoHumanize ) );
+					//pNote->dumpInfo();
+					nReturnValues[nReturnValueIndex] = true;
+					continue;
+				}
+				// delay note execution
+				//INFOLOG( "Delaying note execution. noteStartInFrames: " + to_string( noteStartInFrames ) + ", nFramePos: " + to_string( nFramepos ) );
+				//return 0;
+				continue;
+			}
+		}
+
+		float cost_L = 1.0f;
+		float cost_R = 1.0f;
+		float cost_track_L = 1.0f;
+		float cost_track_R = 1.0f;
+
+		assert(pMainCompo);
+
+		if ( pInstr->is_muted() || pSong->__is_muted || pMainCompo->is_muted() ) {	// is instrument muted?
+			cost_L = 0.0;
+			cost_R = 0.0;
+			if ( Preferences::get_instance()->m_nJackTrackOutputMode == 0 ) {
+				// Post-Fader
+				cost_track_L = 0.0;
+				cost_track_R = 0.0;
+			}
+
+		} else {	// Precompute some values...
+			if ( pInstr->get_apply_velocity() ) {
+				cost_L = cost_L * pNote->get_velocity();		// note velocity
+				cost_R = cost_R * pNote->get_velocity();		// note velocity
+			}
+			cost_L = cost_L * pNote->get_pan_l();		// note pan
+			cost_L = cost_L * fLayerGain;				// layer gain
+			cost_L = cost_L * pInstr->get_pan_l();		// instrument pan
+			cost_L = cost_L * pInstr->get_gain();		// instrument gain
+
+			cost_L = cost_L * pCompo->get_gain();       // Component gain
+			cost_L = cost_L * pMainCompo->get_volume(); // Component volument
+
+			cost_L = cost_L * pInstr->get_volume();		// instrument volume
+			if ( Preferences::get_instance()->m_nJackTrackOutputMode == 0 ) {
 			// Post-Fader
 			cost_track_L = cost_L * 2;
-		}
-		cost_L = cost_L * pSong->get_volume();	// song volume
-		cost_L = cost_L * 2; // max pan is 0.5
+			}
+			cost_L = cost_L * pSong->get_volume();	// song volume
+			cost_L = cost_L * 2; // max pan is 0.5
 
+			cost_R = cost_R * pNote->get_pan_r();		// note pan
+			cost_R = cost_R * fLayerGain;				// layer gain
+			cost_R = cost_R * pInstr->get_pan_r();		// instrument pan
+			cost_R = cost_R * pInstr->get_gain();		// instrument gain
 
-		cost_R = cost_R * pNote->get_velocity();		// note velocity
-		cost_R = cost_R * pNote->get_pan_r();		// note pan
-		cost_R = cost_R * fLayerGain;				// layer gain
-		cost_R = cost_R * pInstr->get_pan_r();		// instrument pan
-		cost_R = cost_R * pInstr->get_gain();		// instrument gain
+			cost_R = cost_R * pCompo->get_gain();       // Component gain
+			cost_R = cost_R * pMainCompo->get_volume(); // Component volument
 
-		cost_R = cost_R * pInstr->get_volume();		// instrument volume
-		if ( Preferences::get_instance()->m_nJackTrackOutputMode == 0 ) {
-		// Post-Fader
+			cost_R = cost_R * pInstr->get_volume();		// instrument volume
+			if ( Preferences::get_instance()->m_nJackTrackOutputMode == 0 ) {
+			// Post-Fader
 			cost_track_R = cost_R * 2;
+			}
+			cost_R = cost_R * pSong->get_volume();	// song pan
+			cost_R = cost_R * 2; // max pan is 0.5
 		}
-		cost_R = cost_R * pSong->get_volume();	// song pan
-		cost_R = cost_R * 2; // max pan is 0.5
-	}
 
-	// direct track outputs only use velocity
-	if ( Preferences::get_instance()->m_nJackTrackOutputMode == 1 ) {
-		cost_track_L = cost_track_L * pNote->get_velocity();
-		cost_track_L = cost_track_L * fLayerGain;
-		cost_track_R = cost_track_L;
-	}
+		// direct track outputs only use velocity
+		if ( Preferences::get_instance()->m_nJackTrackOutputMode == 1 ) {
+			cost_track_L = cost_track_L * pNote->get_velocity();
+			cost_track_L = cost_track_L * fLayerGain;
+			cost_track_R = cost_track_L;
+		}
 
-	// Se non devo fare resample (drumkit) posso evitare di utilizzare i float e gestire il tutto in
-	// maniera ottimizzata
-	//	constant^12 = 2, so constant = 2^(1/12) = 1.059463.
-	//	float nStep = 1.0;1.0594630943593
+		// Se non devo fare resample (drumkit) posso evitare di utilizzare i float e gestire il tutto in
+		// maniera ottimizzata
+		//	constant^12 = 2, so constant = 2^(1/12) = 1.059463.
+		//	float nStep = 1.0;1.0594630943593
 
-	float fTotalPitch = pNote->get_total_pitch() + fLayerPitch;
+		float fTotalPitch = pNote->get_total_pitch() + fLayerPitch;
 
-	//_INFOLOG( "total pitch: " + to_string( fTotalPitch ) );
-	if( ( int )pNote->get_sample_position() == 0 )
-	{
-		if( Hydrogen::get_instance()->getMidiOutput() != NULL ){
+		//_INFOLOG( "total pitch: " + to_string( fTotalPitch ) );
+		if( ( int )pSelectedLayer->SamplePosition == 0 )
+		{
+			if( Hydrogen::get_instance()->getMidiOutput() != NULL ){
 			Hydrogen::get_instance()->getMidiOutput()->handleQueueNote( pNote );
+			}
 		}
-	}
 
-	if ( fTotalPitch == 0.0 && pSample->get_sample_rate() == audio_output->getSampleRate() ) {	// NO RESAMPLE
-				return __render_note_no_resample( pSample, pNote, nBufferSize, nInitialSilence, cost_L, cost_R, cost_track_L, cost_track_R, pSong );
-	} else {	// RESAMPLE
-				return __render_note_resample( pSample, pNote, nBufferSize, nInitialSilence, cost_L, cost_R, cost_track_L, cost_track_R, fLayerPitch, pSong );
+		if ( fTotalPitch == 0.0 && pSample->get_sample_rate() == audio_output->getSampleRate() ) // NO RESAMPLE
+			nReturnValues[nReturnValueIndex] = __render_note_no_resample( pSample, pNote, pSelectedLayer, pCompo, pMainCompo, nBufferSize, nInitialSilence, cost_L, cost_R, cost_track_L, cost_track_R, pSong );
+		else // RESAMPLE
+			nReturnValues[nReturnValueIndex] = __render_note_resample( pSample, pNote, pSelectedLayer, pCompo, pMainCompo, nBufferSize, nInitialSilence, cost_L, cost_R, cost_track_L, cost_track_R, fLayerPitch, pSong );
+
+		nReturnValueIndex++;
 	}
+	for ( unsigned i = 0 ; i < pInstr->get_components()->size() ; i++ )
+		if ( !nReturnValues[i] ) return false;
+	return true;
 }
 
-int Sampler::__render_note_no_resample(
+bool Sampler::__render_note_no_resample(
 	Sample *pSample,
 	Note *pNote,
+	SelectedLayerInfo *pSelectedLayerInfo,
+	InstrumentComponent *pCompo,
+	DrumkitComponent *pDrumCompo,
 	int nBufferSize,
 	int nInitialSilence,
 	float cost_L,
@@ -369,30 +527,29 @@ int Sampler::__render_note_no_resample(
 	Song* pSong
 )
 {
-	AudioOutput* audio_output = Hydrogen::get_instance()->getAudioOutput();
-	int retValue = 1; // the note is ended
+	AudioOutput* pAudioOutput = Hydrogen::get_instance()->getAudioOutput();
+	bool retValue = true; // the note is ended
 
 	int nNoteLength = -1;
 	if ( pNote->get_length() != -1 ) {
-		nNoteLength = ( int )( pNote->get_length() * audio_output->m_transport.m_nTickSize );
+		nNoteLength = ( int )( pNote->get_length() * pAudioOutput->m_transport.m_nTickSize );
 	}
 
-	int nAvail_bytes = pSample->get_frames() - ( int )pNote->get_sample_position();	// verifico il numero di frame disponibili ancora da eseguire
+	int nAvail_bytes = pSample->get_frames() - ( int )pSelectedLayerInfo->SamplePosition;	// verifico il numero di frame disponibili ancora da eseguire
 
 	if ( nAvail_bytes > nBufferSize - nInitialSilence ) {	// il sample e' piu' grande del buffersize
 		// imposto il numero dei bytes disponibili uguale al buffersize
 		nAvail_bytes = nBufferSize - nInitialSilence;
-		retValue = 0; // the note is not ended yet
+		retValue = false; // the note is not ended yet
 	}
 
 
 	//ADSR *pADSR = pNote->m_pADSR;
 
 	int nInitialBufferPos = nInitialSilence;
-	int nInitialSamplePos = ( int )pNote->get_sample_position();
+	int nInitialSamplePos = ( int )pSelectedLayerInfo->SamplePosition;
 	int nSamplePos = nInitialSamplePos;
 	int nTimes = nInitialBufferPos + nAvail_bytes;
-	int nInstrument = pSong->get_instrument_list()->index( pNote->get_instrument() );
 
 	float *pSample_data_L = pSample->get_data_l();
 	float *pSample_data_R = pSample->get_data_r();
@@ -404,30 +561,23 @@ int Sampler::__render_note_no_resample(
 	float fVal_L;
 	float fVal_R;
 
-	/*
-	 * nInstrument could be -1 if the instrument is not found in the current drumset.
-	 * This happens when someone is using the prelistening function of the soundlibrary.
-	 */
-
-	if( nInstrument < 0 ) {
-		nInstrument = 0;
-	}
 
 #ifdef H2CORE_HAVE_JACK
-	JackOutput* jao = 0;
-	float *track_out_L = 0;
-	float *track_out_R = 0;
-	if( audio_output->has_track_outs()
-	&& (jao = dynamic_cast<JackOutput*>(audio_output)) ) {
-		track_out_L = jao->getTrackOut_L( nInstrument );
-		track_out_R = jao->getTrackOut_R( nInstrument );
+	JackOutput* pJackOutput = 0;
+	float *		pTrackOutL = 0;
+	float *		pTrackOutR = 0;
+
+	if( pAudioOutput->has_track_outs()
+	&& (pJackOutput = dynamic_cast<JackOutput*>(pAudioOutput)) ) {
+		 pTrackOutL = pJackOutput->getTrackOut_L( pNote->get_instrument(), pCompo );
+		pTrackOutR = pJackOutput->getTrackOut_R( pNote->get_instrument(), pCompo );
 	}
 #endif
 
 	for ( int nBufferPos = nInitialBufferPos; nBufferPos < nTimes; ++nBufferPos ) {
-		if ( ( nNoteLength != -1 ) && ( nNoteLength <= pNote->get_sample_position() )  ) {
+		if ( ( nNoteLength != -1 ) && ( nNoteLength <= pSelectedLayerInfo->SamplePosition ) ) {
 						if ( pNote->get_adsr()->release() == 0 ) {
-				retValue = 1;	// the note is ended
+				retValue = true;	// the note is ended
 			}
 		}
 
@@ -441,11 +591,11 @@ int Sampler::__render_note_no_resample(
 		}
 
 #ifdef H2CORE_HAVE_JACK
-		if( track_out_L ) {
-			track_out_L[nBufferPos] += fVal_L * cost_track_L;
+		if(  pTrackOutL ) {
+			 pTrackOutL[nBufferPos] += fVal_L * cost_track_L;
 		}
-		if( track_out_R ) {
-			track_out_R[nBufferPos] += fVal_R * cost_track_R;
+		if( pTrackOutR ) {
+			pTrackOutR[nBufferPos] += fVal_R * cost_track_R;
 		}
 #endif
 
@@ -460,13 +610,15 @@ int Sampler::__render_note_no_resample(
 			fInstrPeak_R = fVal_R;
 		}
 
+		pDrumCompo->set_outs( nBufferPos, fVal_L, fVal_R );
+
 		// to main mix
 		__main_out_L[nBufferPos] += fVal_L;
 		__main_out_R[nBufferPos] += fVal_R;
 
 		++nSamplePos;
 	}
-	pNote->update_sample_position( nAvail_bytes );
+	pSelectedLayerInfo->SamplePosition += nAvail_bytes;
 	pNote->get_instrument()->set_peak_l( fInstrPeak_L );
 	pNote->get_instrument()->set_peak_r( fInstrPeak_R );
 
@@ -484,8 +636,6 @@ int Sampler::__render_note_no_resample(
 			float *pBuf_L = pFX->m_pBuffer_L;
 			float *pBuf_R = pFX->m_pBuffer_R;
 
-//			float fFXCost_L = cost_L * fLevel;
-//			float fFXCost_R = cost_R * fLevel;
 			float fFXCost_L = fLevel * masterVol;
 			float fFXCost_R = fLevel * masterVol;
 
@@ -507,9 +657,12 @@ int Sampler::__render_note_no_resample(
 
 
 
-int Sampler::__render_note_resample(
+bool Sampler::__render_note_resample(
 	Sample *pSample,
 	Note *pNote,
+	SelectedLayerInfo *pSelectedLayerInfo,
+	InstrumentComponent *pCompo,
+	DrumkitComponent *pDrumCompo,
 	int nBufferSize,
 	int nInitialSilence,
 	float cost_L,
@@ -520,35 +673,35 @@ int Sampler::__render_note_resample(
 	Song* pSong
 )
 {
-	AudioOutput* audio_output = Hydrogen::get_instance()->getAudioOutput();
+	AudioOutput* pAudioOutput = Hydrogen::get_instance()->getAudioOutput();
+
 	int nNoteLength = -1;
 	if ( pNote->get_length() != -1 ) {
-		nNoteLength = ( int )( pNote->get_length() * audio_output->m_transport.m_nTickSize );
+		nNoteLength = ( int )( pNote->get_length() * pAudioOutput->m_transport.m_nTickSize );
 	}
 	float fNotePitch = pNote->get_total_pitch() + fLayerPitch;
 
 	float fStep = pow( 1.0594630943593, ( double )fNotePitch );
 //	_ERRORLOG( QString("pitch: %1, step: %2" ).arg(fNotePitch).arg( fStep) );
-	fStep *= ( float )pSample->get_sample_rate() / audio_output->getSampleRate(); // Adjust for audio driver sample rate
+	fStep *= ( float )pSample->get_sample_rate() / pAudioOutput->getSampleRate(); // Adjust for audio driver sample rate
 
 	// verifico il numero di frame disponibili ancora da eseguire
-	int nAvail_bytes = ( int )( ( float )( pSample->get_frames() - pNote->get_sample_position() ) / fStep );
+	int nAvail_bytes = ( int )( ( float )( pSample->get_frames() - pSelectedLayerInfo->SamplePosition ) / fStep );
 
 
-	int retValue = 1; // the note is ended
+	bool retValue = true; // the note is ended
 	if ( nAvail_bytes > nBufferSize - nInitialSilence ) {	// il sample e' piu' grande del buffersize
 		// imposto il numero dei bytes disponibili uguale al buffersize
 		nAvail_bytes = nBufferSize - nInitialSilence;
-		retValue = 0; // the note is not ended yet
+		retValue = false; // the note is not ended yet
 	}
 
 	//	ADSR *pADSR = pNote->m_pADSR;
 
 	int nInitialBufferPos = nInitialSilence;
-	float fInitialSamplePos = pNote->get_sample_position();
-	double fSamplePos = pNote->get_sample_position();
+	//float fInitialSamplePos = pNote->get_sample_position( pCompo->get_drumkit_componentID() );
+	double fSamplePos = pSelectedLayerInfo->SamplePosition;
 	int nTimes = nInitialBufferPos + nAvail_bytes;
-	int nInstrument = pSong->get_instrument_list()->index( pNote->get_instrument() );
 
 	float *pSample_data_L = pSample->get_data_l();
 	float *pSample_data_R = pSample->get_data_r();
@@ -561,28 +714,21 @@ int Sampler::__render_note_resample(
 	float fVal_R;
 	int nSampleFrames = pSample->get_frames();
 
-	/*
-	 * nInstrument could be -1 if the instrument is not found in the current drumset.
-	 * This happens when someone is using the prelistening function of the soundlibrary.
-	 */
-
-	if( nInstrument < 0 ) {
-		nInstrument = 0;
-	}
 
 #ifdef H2CORE_HAVE_JACK
-	JackOutput* jao = 0;
-	float *track_out_L = 0;
-	float *track_out_R = 0;
-	if( audio_output->has_track_outs()
-	&& (jao = dynamic_cast<JackOutput*>(audio_output)) ) {
-		track_out_L = jao->getTrackOut_L( nInstrument );
-		track_out_R = jao->getTrackOut_R( nInstrument );
+	JackOutput* pJackOutput = 0;
+	float *		pTrackOutL = 0;
+	float *		pTrackOutR = 0;
+
+	if( pAudioOutput->has_track_outs()
+	&& (pJackOutput = dynamic_cast<JackOutput*>(pAudioOutput)) ) {
+				pTrackOutL = pJackOutput->getTrackOut_L( pNote->get_instrument(), pCompo );
+				pTrackOutR = pJackOutput->getTrackOut_R( pNote->get_instrument(), pCompo );
 	}
 #endif
 
 	for ( int nBufferPos = nInitialBufferPos; nBufferPos < nTimes; ++nBufferPos ) {
-		if ( ( nNoteLength != -1 ) && ( nNoteLength <= pNote->get_sample_position() )  ) {
+		if ( ( nNoteLength != -1 ) && ( nNoteLength <= pSelectedLayerInfo->SamplePosition ) ) {
 						if ( pNote->get_adsr()->release() == 0 ) {
 				retValue = 1;	// the note is ended
 			}
@@ -646,11 +792,11 @@ int Sampler::__render_note_resample(
 
 
 #ifdef H2CORE_HAVE_JACK
-		if( track_out_L ) {
-			track_out_L[nBufferPos] += fVal_L * cost_track_L;
+		if( 		pTrackOutL ) {
+					pTrackOutL[nBufferPos] += fVal_L * cost_track_L;
 		}
-		if( track_out_R ) {
-			track_out_R[nBufferPos] += fVal_R * cost_track_R;
+		if( 		pTrackOutR ) {
+					pTrackOutR[nBufferPos] += fVal_R * cost_track_R;
 		}
 #endif
 
@@ -665,13 +811,15 @@ int Sampler::__render_note_resample(
 			fInstrPeak_R = fVal_R;
 		}
 
+		pDrumCompo->set_outs( nBufferPos, fVal_L, fVal_R );
+
 		// to main mix
 		__main_out_L[nBufferPos] += fVal_L;
 		__main_out_R[nBufferPos] += fVal_R;
 
 		fSamplePos += fStep;
 	}
-	pNote->update_sample_position( nAvail_bytes * fStep );
+	pSelectedLayerInfo->SamplePosition += nAvail_bytes * fStep;
 	pNote->get_instrument()->set_peak_l( fInstrPeak_L );
 	pNote->get_instrument()->set_peak_r( fInstrPeak_R );
 
@@ -695,7 +843,7 @@ int Sampler::__render_note_resample(
 			float fFXCost_R = fLevel * masterVol;
 
 			int nBufferPos = nInitialBufferPos;
-			float fSamplePos = fInitialSamplePos;
+			float fSamplePos = pSelectedLayerInfo->SamplePosition;
 			for ( int i = 0; i < nAvail_bytes; ++i ) {
 				int nSamplePos = ( int )fSamplePos;
 				double fDiff = fSamplePos - nSamplePos;
@@ -723,8 +871,6 @@ int Sampler::__render_note_resample(
 					case LINEAR:
 						fVal_L = pSample_data_L[nSamplePos] * (1 - fDiff ) + pSample_data_L[nSamplePos + 1] * fDiff;
 						fVal_R = pSample_data_R[nSamplePos] * (1 - fDiff ) + pSample_data_R[nSamplePos + 1] * fDiff;
-						//fVal_L = linear_Interpolate( pSample_data_L[nSamplePos], pSample_data_L[nSamplePos + 1], fDiff);
-						//fVal_R = linear_Interpolate( pSample_data_R[nSamplePos], pSample_data_R[nSamplePos + 1], fDiff);
 						break;
 					case COSINE:
 						fVal_L = cosine_Interpolate( pSample_data_L[nSamplePos], pSample_data_L[nSamplePos + 1], fDiff);
@@ -743,10 +889,6 @@ int Sampler::__render_note_resample(
 						fVal_R = hermite_Interpolate( pSample_data_R[ nSamplePos -1], pSample_data_R[nSamplePos], pSample_data_R[nSamplePos + 1], last_r, fDiff);
 						break;
 					}
-					// methode Interpolate produce an extra function call and eat much more time here.
-					// so i deside to code the switch direct in the resampler methode
-					//fVal_L = Interpolate( pSample_data_L[ nSamplePos -1], pSample_data_L[nSamplePos], pSample_data_L[nSamplePos + 1], pSample_data_L[nSamplePos + 2] ,fDiff);
-					//fVal_L = Interpolate( pSample_data_R[ nSamplePos -1], pSample_data_R[nSamplePos], pSample_data_R[nSamplePos + 1], pSample_data_R[nSamplePos + 2] ,fDiff);
 				}
 
 				pBuf_L[ nBufferPos ] += fVal_L * fFXCost_L;
@@ -764,14 +906,6 @@ int Sampler::__render_note_resample(
 
 void Sampler::stop_playing_notes( Instrument* instrument )
 {
-	/*
-	// send a note-off event to all notes present in the playing note queue
-	for ( int i = 0; i < __playing_notes_queue.size(); ++i ) {
-		Note *pNote = __playing_notes_queue[ i ];
-		pNote->m_pADSR->release();
-	}
-	*/
-
 	if ( instrument ) { // stop all notes using this instrument
 		for ( unsigned i = 0; i < __playing_notes_queue.size(); ) {
 			Note *pNote = __playing_notes_queue[ i ];
@@ -801,16 +935,20 @@ void Sampler::preview_sample( Sample* sample, int length )
 {
 	AudioEngine::get_instance()->lock( RIGHT_HERE );
 
-	InstrumentLayer *pLayer = __preview_instrument->get_layer( 0 );
+	for (std::vector<InstrumentComponent*>::iterator it = __preview_instrument->get_components()->begin() ; it != __preview_instrument->get_components()->end(); ++it) {
+		InstrumentComponent* pComponent = *it;
+		InstrumentLayer *pLayer = pComponent->get_layer( 0 );
 
-	Sample *pOldSample = pLayer->get_sample();
-	pLayer->set_sample( sample );
 
-	Note *previewNote = new Note( __preview_instrument, 0, 1.0, 0.5, 0.5, length, 0 );
+		Sample *pOldSample = pLayer->get_sample();
+		pLayer->set_sample( sample );
 
-	stop_playing_notes( __preview_instrument );
-	note_on( previewNote );
-	delete pOldSample;
+		Note *pPreviewNote = new Note( __preview_instrument, 0, 1.0, 0.5, 0.5, length, 0 );
+
+		stop_playing_notes( __preview_instrument );
+		note_on( pPreviewNote );
+		delete pOldSample;
+	}
 
 	AudioEngine::get_instance()->unlock();
 }
@@ -819,19 +957,20 @@ void Sampler::preview_sample( Sample* sample, int length )
 
 void Sampler::preview_instrument( Instrument* instr )
 {
-	Instrument * old_preview;
+	Instrument * pOldPreview;
 	AudioEngine::get_instance()->lock( RIGHT_HERE );
 
 	stop_playing_notes( __preview_instrument );
 
-	old_preview = __preview_instrument;
+	pOldPreview = __preview_instrument;
 	__preview_instrument = instr;
+	instr->set_is_preview_instrument(true);
 
-	Note *previewNote = new Note( __preview_instrument, 0, 1.0, 0.5, 0.5, MAX_NOTES, 0 );
+	Note *pPreviewNote = new Note( __preview_instrument, 0, 1.0, 0.5, 0.5, MAX_NOTES, 0 );
 
-	note_on( previewNote );	// exclusive note
+	note_on( pPreviewNote );	// exclusive note
 	AudioEngine::get_instance()->unlock();
-	delete old_preview;
+	delete pOldPreview;
 }
 
 
@@ -840,35 +979,35 @@ void Sampler::setPlayingNotelength( Instrument* instrument, unsigned long ticks,
 {
 	if ( instrument ) { // stop all notes using this instrument
 		Hydrogen *pEngine = Hydrogen::get_instance();
-		Song* mSong = pEngine->getSong();
+		Song* pSong = pEngine->getSong();
 		int selectedpattern = pEngine->__get_selected_PatterNumber();
-		Pattern* currentPattern = NULL;
+		Pattern* pCurrentPattern = NULL;
 
 
-		if ( mSong->get_mode() == Song::PATTERN_MODE ||
+		if ( pSong->get_mode() == Song::PATTERN_MODE ||
 		( pEngine->getState() != STATE_PLAYING )){
-			PatternList *pPatternList = mSong->get_pattern_list();
+			PatternList *pPatternList = pSong->get_pattern_list();
 			if ( ( selectedpattern != -1 )
 			&& ( selectedpattern < ( int )pPatternList->size() ) ) {
-				currentPattern = pPatternList->get( selectedpattern );
+				pCurrentPattern = pPatternList->get( selectedpattern );
 			}
 		}else
 		{
-			std::vector<PatternList*> *pColumns = mSong->get_pattern_group_vector();
+			std::vector<PatternList*> *pColumns = pSong->get_pattern_group_vector();
 //			Pattern *pPattern = NULL;
 			int pos = pEngine->getPatternPos() +1;
 			for ( int i = 0; i < pos; ++i ) {
 				PatternList *pColumn = ( *pColumns )[i];
-				currentPattern = pColumn->get( 0 );
+				pCurrentPattern = pColumn->get( 0 );
 			}
 		}
 
 
-		if ( currentPattern ) {
-				int patternsize = currentPattern->get_length();
+		if ( pCurrentPattern ) {
+				int patternsize = pCurrentPattern->get_length();
 
-				for ( unsigned nNote = 0; nNote < currentPattern->get_length(); nNote++ ) {
-					const Pattern::notes_t* notes = currentPattern->get_notes();
+				for ( unsigned nNote = 0; nNote < pCurrentPattern->get_length(); nNote++ ) {
+					const Pattern::notes_t* notes = pCurrentPattern->get_notes();
 					FOREACH_NOTE_CST_IT_BOUND(notes,it,nNote) {
 						Note *pNote = it->second;
 						if ( pNote!=NULL ) {
@@ -880,7 +1019,7 @@ void Sampler::setPlayingNotelength( Instrument* instrument, unsigned long ticks,
 									if ( ticks >  patternsize )
 										ticks = patternsize - noteOnTick;
 									pNote->set_length( ticks );
-									Hydrogen::get_instance()->getSong()->__is_modified = true;
+									Hydrogen::get_instance()->getSong()->set_is_modified( true );
 									AudioEngine::get_instance()->unlock(); // unlock the audio engine
 								}
 							}else
@@ -891,7 +1030,7 @@ void Sampler::setPlayingNotelength( Instrument* instrument, unsigned long ticks,
 									if ( ticks >  patternsize )
 										ticks = patternsize - noteOnTick;
 									pNote->set_length( ticks );
-									Hydrogen::get_instance()->getSong()->__is_modified = true;
+									Hydrogen::get_instance()->getSong()->set_is_modified( true );
 									AudioEngine::get_instance()->unlock(); // unlock the audio engine
 								}
 							}
@@ -900,6 +1039,7 @@ void Sampler::setPlayingNotelength( Instrument* instrument, unsigned long ticks,
 				}
 			}
 		}
+
 	EventQueue::get_instance()->push_event( EVENT_PATTERN_MODIFIED, -1 );
 }
 
