@@ -20,21 +20,25 @@
  *
  */
 
+#include <hydrogen/event_queue.h>
 #include <hydrogen/version.h>
 #include <hydrogen/hydrogen.h>
 #include <hydrogen/playlist.h>
 #include <hydrogen/audio_engine.h>
 #include <hydrogen/smf/SMF.h>
 #include <hydrogen/Preferences.h>
+#include <hydrogen/timeline.h>
 #include <hydrogen/LocalFileMng.h>
 #include <hydrogen/basics/pattern.h>
 #include <hydrogen/basics/pattern_list.h>
 #include <hydrogen/basics/instrument_list.h>
 #include <hydrogen/basics/instrument_layer.h>
-#include <hydrogen/event_queue.h>
+
+#include <hydrogen/lilypond/lilypond.h>
 
 #include "AboutDialog.h"
 #include "AudioEngineInfoForm.h"
+#include "DonationDialog.h"
 #include "ExportSongDialog.h"
 #include "HydrogenApp.h"
 #include "InstrumentRack.h"
@@ -66,7 +70,7 @@
 #endif
 
 #ifdef H2CORE_HAVE_LASH
-#include <lash-1.0/lash/lash.h>
+#include <lash/lash.h>
 #include <hydrogen/LashClient.h>
 #endif
 
@@ -100,13 +104,18 @@ MainForm::MainForm( QApplication *app, const QString& songFilename )
 	m_pQApp->processEvents();
 
 	// Load default song
-	Song *song = NULL;
+	Song *pSong = NULL;
+
 	if ( !songFilename.isEmpty() ) {
-		song = Song::load( songFilename );
-		if (song == NULL) {
-			//QMessageBox::warning( this, "Hydrogen", trUtf8("Error loading song.") );
-			song = Song::get_empty_song();
-			song->set_filename( "" );
+		pSong = Song::load( songFilename );
+
+		/*
+		 * If the song could not be loaded, create
+		 * a new one with the specified filename
+		 */
+		if (pSong == NULL) {
+			pSong = Song::get_empty_song();
+			pSong->set_filename( songFilename );
 		}
 	}
 	else {
@@ -114,20 +123,20 @@ MainForm::MainForm( QApplication *app, const QString& songFilename )
 		bool restoreLastSong = pref->isRestoreLastSongEnabled();
 		QString filename = pref->getLastSongFilename();
 		if ( restoreLastSong && ( !filename.isEmpty() )) {
-			song = Song::load( filename );
-			if (song == NULL) {
+			pSong = Song::load( filename );
+			if (pSong == NULL) {
 				//QMessageBox::warning( this, "Hydrogen", trUtf8("Error restoring last song.") );
-				song = Song::get_empty_song();
-				song->set_filename( "" );
+				pSong = Song::get_empty_song();
+				pSong->set_filename( "" );
 			}
 		}
 		else {
-			song = Song::get_empty_song();
-			song->set_filename( "" );
+			pSong = Song::get_empty_song();
+			pSong->set_filename( "" );
 		}
 	}
 
-	h2app = new HydrogenApp( this, song );
+	h2app = new HydrogenApp( this, pSong );
 	h2app->addEventListener( this );
 	createMenuBar();
 
@@ -148,7 +157,7 @@ MainForm::MainForm( QApplication *app, const QString& songFilename )
 	//	h2app->getPlayListDialog()->installEventFilter(this);
 	installEventFilter( this );
 
-	showDevelWarning();
+	//showDevelWarning();
 
 	connect( &m_autosaveTimer, SIGNAL(timeout()), this, SLOT(onAutoSaveTimer()));
 	m_autosaveTimer.start( 60 * 1000 );
@@ -241,7 +250,7 @@ void MainForm::createMenuBar()
 	setMenuBar( m_pMenubar );
 
 	// FILE menu
-	QMenu *m_pFileMenu = m_pMenubar->addMenu( trUtf8( "&Project" ) );
+	QMenu *m_pFileMenu = m_pMenubar->addMenu( trUtf8( "Pro&ject" ) );
 
 	m_pFileMenu->addAction( trUtf8( "&New" ), this, SLOT( action_file_new() ), QKeySequence( "Ctrl+N" ) );
 	m_pFileMenu->addAction( trUtf8( "Show &info" ), this, SLOT( action_file_songProperties() ), QKeySequence( "" ) );
@@ -267,6 +276,7 @@ void MainForm::createMenuBar()
 
 	m_pFileMenu->addAction( trUtf8( "Export &MIDI file" ), this, SLOT( action_file_export_midi() ), QKeySequence( "Ctrl+M" ) );
 	m_pFileMenu->addAction( trUtf8( "&Export song" ), this, SLOT( action_file_export() ), QKeySequence( "Ctrl+E" ) );
+	m_pFileMenu->addAction( trUtf8( "Export &LilyPond file" ), this, SLOT( action_file_export_lilypond() ), QKeySequence( "Ctrl+L" ) );
 
 
 #ifndef Q_OS_MACX
@@ -329,12 +339,12 @@ void MainForm::createMenuBar()
 	//~ Tools menu
 
 
-	Logger *l = Logger::get_instance();
-	if ( l->bit_mask() >= 1 ) {
+	Logger *pLogger = Logger::get_instance();
+	if ( pLogger->bit_mask() >= 1 ) {
 		// DEBUG menu
 		QMenu *m_pDebugMenu = m_pMenubar->addMenu( trUtf8("De&bug") );
 		m_pDebugMenu->addAction( trUtf8( "Show &audio engine info" ), this, SLOT( action_debug_showAudioEngineInfo() ) );
-		if(l->bit_mask() == 8) // hydrogen -V8 list object map in console
+		if(pLogger->bit_mask() == 8) // hydrogen -V8 list object map in console
 			m_pDebugMenu->addAction( trUtf8( "Print Objects" ), this, SLOT( action_debug_printObjects() ) );
 		//~ DEBUG menu
 	}
@@ -345,6 +355,7 @@ void MainForm::createMenuBar()
 	m_pInfoMenu->addSeparator();
 	m_pInfoMenu->addAction( trUtf8("&About"), this, SLOT( action_help_about() ), QKeySequence( trUtf8("", "Info|About") ) );
 	m_pInfoMenu->addAction( trUtf8("Report bug"), this, SLOT( action_report_bug() ));
+	m_pInfoMenu->addAction( trUtf8("Donate"), this, SLOT( action_donate() ));
 	//~ INFO menu
 }
 
@@ -432,6 +443,11 @@ void MainForm::onLashPollTimer()
 #endif
 }
 
+void MainForm::action_donate()
+{
+	DonationDialog *dialog = new DonationDialog( NULL );
+	dialog->exec();
+}
 
 /// return true if the app needs to be closed.
 bool MainForm::action_file_exit()
@@ -448,8 +464,9 @@ bool MainForm::action_file_exit()
 
 void MainForm::action_file_new()
 {
-	if ( (Hydrogen::get_instance()->getState() == STATE_PLAYING) ) {
-		Hydrogen::get_instance()->sequencer_stop();
+	Hydrogen * pEngine = Hydrogen::get_instance();
+	if ( (pEngine->getState() == STATE_PLAYING) ) {
+		pEngine->sequencer_stop();
 	}
 
 	bool proceed = handleUnsavedChanges();
@@ -458,14 +475,14 @@ void MainForm::action_file_new()
 	}
 
 	h2app->m_undoStack->clear();
-	Hydrogen::get_instance()->m_timelinevector.clear();
-	Song * song = Song::get_empty_song();
-	song->set_filename( "" );
-	h2app->setSong(song);
-	Hydrogen::get_instance()->setSelectedPatternNumber( 0 );
-	HydrogenApp::get_instance()->getInstrumentRack()->getSoundLibraryPanel()->update_background_color();
-	HydrogenApp::get_instance()->getSongEditorPanel()->updatePositionRuler();
-	Hydrogen::get_instance()->m_timelinetagvector.clear();
+	pEngine->getTimeline()->m_timelinevector.clear();
+	Song * pSong = Song::get_empty_song();
+	pSong->set_filename( "" );
+	h2app->setSong(pSong);
+	pEngine->setSelectedPatternNumber( 0 );
+	h2app->getInstrumentRack()->getSoundLibraryPanel()->update_background_color();
+	h2app->getSongEditorPanel()->updatePositionRuler();
+	pEngine->getTimeline()->m_timelinetagvector.clear();
 
 	// update director tags
 	EventQueue::get_instance()->push_event( EVENT_METRONOME, 2 );
@@ -477,8 +494,10 @@ void MainForm::action_file_new()
 
 void MainForm::action_file_save_as()
 {
-	if ( (Hydrogen::get_instance()->getState() == STATE_PLAYING) ) {
-		Hydrogen::get_instance()->sequencer_stop();
+	Hydrogen* pEngine = Hydrogen::get_instance();
+
+	if ( pEngine->getState() == STATE_PLAYING ) {
+		  pEngine->sequencer_stop();
 	}
 
 	//std::auto_ptr<QFileDialog> fd( new QFileDialog );
@@ -489,12 +508,12 @@ void MainForm::action_file_save_as()
 	fd.setWindowTitle( trUtf8( "Save song" ) );
 	fd.setSidebarUrls( fd.sidebarUrls() << QUrl::fromLocalFile( Filesystem::songs_dir() ) );
 
-	Song *song = Hydrogen::get_instance()->getSong();
+	Song *song = pEngine->getSong();
 	QString defaultFilename;
 	QString lastFilename = song->get_filename();
 
 	if ( lastFilename.isEmpty() ) {
-		defaultFilename = Hydrogen::get_instance()->getSong()->__name;
+		defaultFilename = pEngine->getSong()->__name;
 		defaultFilename += ".h2song";
 	}
 	else {
@@ -518,15 +537,15 @@ void MainForm::action_file_save_as()
 		action_file_save();
 	}
 	h2app->setScrollStatusBarMessage( trUtf8("Song saved as.") + QString(" Into: ") + defaultFilename, 2000 );
-	h2app->setWindowTitle( filename );
+	h2app->updateWindowTitle();
 }
 
 
 
 void MainForm::action_file_save()
 {
-	Song *song = Hydrogen::get_instance()->getSong();
-	QString filename = song->get_filename();
+	Song *pSong = Hydrogen::get_instance()->getSong();
+	QString filename = pSong->get_filename();
 
 	if ( filename.isEmpty() ) {
 		// just in case!
@@ -534,13 +553,13 @@ void MainForm::action_file_save()
 	}
 
 	bool saved = false;
-	saved = song->save( filename );
+	saved = pSong->save( filename );
 
 
 	if(! saved) {
 		QMessageBox::warning( this, "Hydrogen", trUtf8("Could not save song.") );
 	} else {
-		Preferences::get_instance()->setLastSongFilename( song->get_filename() );
+		Preferences::get_instance()->setLastSongFilename( pSong->get_filename() );
 
 		// add the new loaded song in the "last used song" vector
 		Preferences *pPref = Preferences::get_instance();
@@ -699,11 +718,11 @@ void MainForm::action_file_open() {
 void MainForm::action_file_openPattern()
 {
 
-	Hydrogen *engine = Hydrogen::get_instance();
-	Song *song = engine->getSong();
-	PatternList *pPatternList = song->get_pattern_list();
+	Hydrogen *pEngine = Hydrogen::get_instance();
+	Song *pSong = pEngine->getSong();
+	PatternList *pPatternList = pSong->get_pattern_list();
 
-	Instrument *instr = song->get_instrument_list()->get ( 0 );
+	Instrument *instr = pSong->get_instrument_list()->get ( 0 );
 	assert ( instr );
 
 	QDir dirPattern( Preferences::get_instance()->getDataDirectory() + "/patterns" );
@@ -722,8 +741,6 @@ void MainForm::action_file_openPattern()
 	}
 	QString patternname = filename;
 
-
-	LocalFileMng mng;
 	LocalFileMng fileMng;
 	Pattern* err = fileMng.loadPattern ( patternname );
 	if ( err == 0 )
@@ -735,7 +752,8 @@ void MainForm::action_file_openPattern()
 	{
 		H2Core::Pattern *pNewPattern = err;
 		pPatternList->add ( pNewPattern );
-		song->__is_modified = true;
+		pSong->set_is_modified( true );
+		EventQueue::get_instance()->push_event( EVENT_SONG_MODIFIED, -1 );
 	}
 
 	HydrogenApp::get_instance()->getSongEditorPanel()->updateAll();
@@ -849,23 +867,13 @@ void MainForm::action_instruments_clearAll()
 	}
 
 	// Remove all layers
-	//	AudioEngine::get_instance()->lock( RIGHT_HERE );
+
 	Song *pSong = Hydrogen::get_instance()->getSong();
 	InstrumentList* pList = pSong->get_instrument_list();
 	for (uint i = pList->size(); i > 0; i--) {
 		functionDeleteInstrument(i - 1);
-		/*
-		Instrument* pInstr = pList->get( i );
-		pInstr->set_name( (QString( trUtf8( "Instrument %1" ) ).arg( i + 1 )) );
-		// remove all layers
-		for ( int nLayer = 0; nLayer < MAX_LAYERS; nLayer++ ) {
-			InstrumentLayer* pLayer = pInstr->get_layer( nLayer );
-			delete pLayer;
-			pInstr->set_layer( NULL, nLayer );
-				}
-			 */
 	}
-	//	AudioEngine::get_instance()->unlock();
+
 	EventQueue::get_instance()->push_event( EVENT_SELECTED_INSTRUMENT_CHANGED, -1 );
 }
 
@@ -1147,15 +1155,16 @@ void MainForm::openSongFile( const QString& sFilename )
 		engine->sequencer_stop();
 	}
 
-	engine->m_timelinetagvector.clear();
+	engine->getTimeline()->m_timelinetagvector.clear();
 
 	h2app->closeFXProperties();
-	LocalFileMng mng;
+
 	Song *pSong = Song::load( sFilename );
 	if ( pSong == NULL ) {
 		QMessageBox::information( this, "Hydrogen", trUtf8("Error loading song.") );
 		return;
 	}
+
 	h2app->m_undoStack->clear();
 
 	// add the new loaded song in the "last used song" vector
@@ -1169,7 +1178,6 @@ void MainForm::openSongFile( const QString& sFilename )
 	updateRecentUsedSongList();
 	engine->setSelectedPatternNumber( 0 );
 	HydrogenApp::get_instance()->getSongEditorPanel()->updatePositionRuler();
-	//	EventQueue::get_instance()->push_event( EVENT_METRONOME, 1 );
 	EventQueue::get_instance()->push_event( EVENT_METRONOME, 3 );
 }
 
@@ -1323,17 +1331,7 @@ bool MainForm::eventFilter( QObject *o, QEvent *e )
 			onSaveAccelEvent();
 			return TRUE;
 			break;
-			/*
-				case  Qt::Key_T:
-					   if( engine->getSong()->get_mode() == Song::PATTERN_MODE ){
-						   engine->getSong()->set_mode( Song::SONG_MODE);
-					   }
-					   else if ( engine->getSong()->get_mode() == Song::SONG_MODE ) {
-						   engine->getSong()->set_mode( Song::PATTERN_MODE );
-					   }
-						return TRUE;
-						break;
-*/
+
 		case  Qt::Key_F5 :
 			if( engine->m_PlayList.size() == 0)
 				break;
@@ -1370,11 +1368,6 @@ bool MainForm::eventFilter( QObject *o, QEvent *e )
 			app->getSongEditorPanel()->updateAll();
 			return TRUE;
 			break;
-
-			// 	QAccel *a = new QAccel( this );
-			// 	a->connectItem( a->insertItem(Key_S + CTRL), this, SLOT( onSaveAccelEvent() ) );
-			// 	a->connectItem( a->insertItem(Key_O + CTRL), this, SLOT( onOpenAccelEvent() ) );
-
 		}
 
 		// virtual keyboard handling
@@ -1384,13 +1377,12 @@ bool MainForm::eventFilter( QObject *o, QEvent *e )
 			// insert note at the current column in time
 			// if event recording enabled
 			int row = (*found).second;
-			Hydrogen* engine = Hydrogen::get_instance();
 
 			float velocity = 0.8;
 			float pan_L = 0.5f;
 			float pan_R = 0.5f;
 
-			engine->addRealtimeNote (row, velocity, pan_L, pan_R, 0, NULL, NULL , row + 36);
+			engine->addRealtimeNote (row, velocity, pan_L, pan_R, 0, false, false , row + 36);
 
 			return TRUE; // eat event
 		}
@@ -1453,7 +1445,50 @@ void MainForm::action_file_export_midi()
 	}
 }
 
+void MainForm::action_file_export_lilypond()
+{
+	if ( ( ( Hydrogen::get_instance() )->getState() == STATE_PLAYING ) ) {
+		Hydrogen::get_instance()->sequencer_stop();
+	}
+	switch ( QMessageBox::information(
+	        this,
+	        "Hydrogen",
+	        trUtf8( "\nThe LilyPond export is an experimental feature.\n"
+	                "It should work like a charm provided that you use the "
+	                "GM-kit, and that you do not use triplet\n" ),
+	        trUtf8( "Ok" ),
+	        trUtf8( "&Cancel" ),
+	        0,
+	        2 ) ) {
+	case 1:
+	case 2: return;
+	}
 
+	QFileDialog fd( this );
+	fd.setFileMode( QFileDialog::AnyFile );
+	fd.setFilter( trUtf8( "LilyPond file (*.ly)" ) );
+	fd.setDirectory( QDir::homePath() );
+	fd.setWindowTitle( trUtf8( "Export LilyPond file" ) );
+	fd.setAcceptMode( QFileDialog::AcceptSave );
+	fd.setWindowIcon( QPixmap( Skin::getImagePath() + "/icon16.png" ) );
+
+	QString sFilename;
+	if ( fd.exec() == QDialog::Accepted ) {
+		sFilename = fd.selectedFiles().first();
+	}
+
+	if ( !sFilename.isEmpty() ) {
+		if ( sFilename.endsWith( ".ly" ) == false ) {
+			sFilename += ".ly";
+		}
+
+		Song *pSong = Hydrogen::get_instance()->getSong();
+
+		LilyPond ly;
+		ly.extractData( *pSong );
+		ly.write( sFilename );
+	}
+}
 
 void MainForm::errorEvent( int nErrorCode )
 {
@@ -1500,17 +1535,17 @@ void MainForm::playlistLoadSongEvent (int nIndex)
 
 	Song* pSong = Hydrogen::get_instance()->getSong();
 
-        h2app->getSongEditorPanel()->updateAll();
-        h2app->getPatternEditorPanel()->updateSLnameLabel();
+	h2app->getSongEditorPanel()->updateAll();
+	h2app->getPatternEditorPanel()->updateSLnameLabel();
 
-        QString songName( pSong->__name );
-        if( songName == "Untitled Song" && !pSong->get_filename().isEmpty() ){
-                songName = pSong->get_filename();
-                songName = songName.section( '/', -1 );
-        }
-        setWindowTitle( songName  );
+	QString songName( pSong->__name );
+	if( songName == "Untitled Song" && !pSong->get_filename().isEmpty() ){
+		songName = pSong->get_filename();
+		songName = songName.section( '/', -1 );
+	}
+	setWindowTitle( songName  );
 
-        h2app->getMainForm()->updateRecentUsedSongList();
+	h2app->getMainForm()->updateRecentUsedSongList();
 	h2app->closeFXProperties();
 	h2app->m_undoStack->clear();
 
@@ -1535,7 +1570,7 @@ void MainForm::action_file_songProperties()
 {
 	SongPropertiesDialog *pDialog = new SongPropertiesDialog( this );
 	if ( pDialog->exec() == QDialog::Accepted ) {
-		Hydrogen::get_instance()->getSong()->__is_modified = true;
+		Hydrogen::get_instance()->getSong()->set_is_modified( true );
 	}
 	delete pDialog;
 }
@@ -1552,7 +1587,7 @@ void MainForm::showDevelWarning()
 {
 
 	//set this to 'false' for the case that you want to make a release..
-	if ( false ) {
+	if ( true ) {
 		Preferences *pref = Preferences::get_instance();
 		bool isDevelWarningEnabled = pref->getShowDevelWarning();
 		if(isDevelWarningEnabled) {
@@ -1601,16 +1636,6 @@ void MainForm::onAutoSaveTimer()
 	pSong->save( getAutoSaveFilename() );
 
 	pSong->set_filename(sOldFilename);
-
-	/*
-	Song *pSong = h2app->getSong();
-	if (pSong->getFilename() == "") {
-		pSong->save( "autosave.h2song" );
-		return;
-	}
-
-	action_file_save();
-*/
 }
 
 
@@ -1639,7 +1664,7 @@ bool MainForm::handleUnsavedChanges()
 {
 	bool done = false;
 	bool rv = true;
-	while ( !done && Hydrogen::get_instance()->getSong()->__is_modified ) {
+	while ( !done && Hydrogen::get_instance()->getSong()->get_is_modified() ) {
 		switch(
 			   QMessageBox::information( this, "Hydrogen",
 										 trUtf8("\nThe document contains unsaved changes.\n"
@@ -1668,6 +1693,31 @@ bool MainForm::handleUnsavedChanges()
 			break;
 		}
 	}
+
+	if(rv != false)
+	{
+		while ( !done && Playlist::get_instance()->getIsModified() ) {
+			switch(
+					QMessageBox::information( this, "Hydrogen",
+											 trUtf8("\nThe current playlist contains unsaved changes.\n"
+													"Do you want to discard the changes?\n"),
+											trUtf8("&Discard"), trUtf8("&Cancel"),
+											 0,      // Enter == button 0
+											 2 ) ) { // Escape == button 1
+			case 0: // Discard clicked or Alt+D pressed
+				// don't save but exit
+				done = true;
+				break;
+			case 1: // Cancel clicked or Alt+C pressed or Escape pressed
+				// don't exit
+				done = true;
+				rv = false;
+				break;
+			}
+		}
+	}
+
+
 	return rv;
 }
 
@@ -1725,4 +1775,3 @@ bool MainForm::handleSelectNextPrevSongOnPlaylist( int step )
 }
 
 
-
