@@ -33,6 +33,8 @@
 #include <hydrogen/audio_engine.h>
 #include <hydrogen/hydrogen.h>
 #include <hydrogen/basics/instrument.h>
+#include <hydrogen/basics/drumkit_component.h>
+#include <hydrogen/basics/instrument_component.h>
 #include <hydrogen/basics/instrument_list.h>
 #include <hydrogen/basics/song.h>
 #include <hydrogen/Preferences.h>
@@ -49,8 +51,6 @@ const char* Mixer::__class_name = "Mixer";
 
 Mixer::Mixer( QWidget* pParent )
  : QWidget( pParent )
-// : QWidget( pParent, Qt::WindowStaysOnTopHint )
-// : QWidget( pParent, Qt::Tool )
  , Object( __class_name )
 {
 	setWindowTitle( trUtf8( __class_name ) );
@@ -155,24 +155,18 @@ Mixer::Mixer( QWidget* pParent )
 	this->setLayout( pLayout );
 
 
-
 	m_pUpdateTimer = new QTimer( this );
 	connect( m_pUpdateTimer, SIGNAL( timeout() ), this, SLOT( updateMixer() ) );
 	m_pUpdateTimer->start(50);
 
-        HydrogenApp::get_instance()->addEventListener( this );
-
+	HydrogenApp::get_instance()->addEventListener( this );
 }
 
-
-
 Mixer::~Mixer()
 {
 	m_pUpdateTimer->stop();
 }
 
-
-
 MixerLine* Mixer::createMixerLine( int nInstr )
 {
 	MixerLine *pMixerLine = new MixerLine( 0 , nInstr);
@@ -194,6 +188,21 @@ MixerLine* Mixer::createMixerLine( int nInstr )
 }
 
 
+ComponentMixerLine* Mixer::createComponentMixerLine( int theCompoID )
+{
+	ComponentMixerLine *pMixerLine = new ComponentMixerLine( 0 , theCompoID);
+	pMixerLine->setVolume( 0.2 );
+	pMixerLine->setMuteClicked( false );
+	pMixerLine->setSoloClicked( false );
+
+	connect( pMixerLine, SIGNAL( muteBtnClicked(ComponentMixerLine*) ), this, SLOT( muteClicked(ComponentMixerLine*) ) );
+	connect( pMixerLine, SIGNAL( soloBtnClicked(ComponentMixerLine*) ), this, SLOT( soloClicked(ComponentMixerLine*) ) );
+	connect( pMixerLine, SIGNAL( volumeChanged(ComponentMixerLine*) ), this, SLOT( volumeChanged(ComponentMixerLine*) ) );
+
+	return pMixerLine;
+}
+
+
 void Mixer::muteClicked(MixerLine* ref)
 {
 	int nLine = findMixerLineByRef(ref);
@@ -206,31 +215,83 @@ void Mixer::muteClicked(MixerLine* ref)
 
 	Instrument *pInstr = instrList->get(nLine);
 	pInstr->set_muted( isMuteClicked);
-	//(HydrogenApp::get_instance())->setSelectedInstrument(nLine);
 	Hydrogen::get_instance()->setSelectedInstrumentNumber(nLine);
 }
 
+void Mixer::muteClicked(ComponentMixerLine* ref)
+{
+	bool isMuteClicked = ref->isMuteClicked();
+
+	DrumkitComponent *pCompo = Hydrogen::get_instance()->getSong()->get_component( ref->getCompoID() );
+
+	pCompo->set_muted( isMuteClicked );
+}
+
+void Mixer::soloClicked(ComponentMixerLine* ref)
+{
+    Hydrogen *pEngine = Hydrogen::get_instance();
+	Song *pSong = pEngine->getSong();
+	std::vector<DrumkitComponent*> pCompoList = *(pSong->get_components());
+	int nComponents = pCompoList.size();
+
+	bool isSoloClicked = ref->isSoloClicked();
+	int nLine = findCompoMixerLineByRef(ref);
+
+	if ( isSoloClicked ) {
+		for ( int i = 0; i < nComponents ; ++i ) {
+			ComponentMixerLine* p_tmpCompoMixer = m_pComponentMixerLine[i];
+			p_tmpCompoMixer->setSoloClicked( false );
+			p_tmpCompoMixer->setMuteClicked( true );
+			DrumkitComponent* p_tmpCompo = pCompoList[i];
+			p_tmpCompo->set_muted( true );
+		}
+		ComponentMixerLine* p_tmpCompoMixer = m_pComponentMixerLine[nLine];
+		p_tmpCompoMixer->setSoloClicked( true );
+		p_tmpCompoMixer->setMuteClicked( false );
+		DrumkitComponent* p_tmpCompo = pCompoList[nLine];
+		p_tmpCompo->set_muted( false );
+	}
+	else {
+		for ( int i = 0; i < nComponents ; ++i ) {
+			ComponentMixerLine* p_tmpCompoMixer = m_pComponentMixerLine[i];
+			p_tmpCompoMixer->setSoloClicked( false );
+			p_tmpCompoMixer->setMuteClicked( false );
+			DrumkitComponent* p_tmpCompo = pCompoList[i];
+			p_tmpCompo->set_muted( false );
+		}
+	}
+}
+
+void Mixer::volumeChanged(ComponentMixerLine* ref)
+{
+	float newVolume = ref->getVolume();
+
+	DrumkitComponent *pCompo = Hydrogen::get_instance()->getSong()->get_component( ref->getCompoID() );
+
+	pCompo->set_volume( newVolume );
+}
+
 void Mixer::unmuteAll( bool findSelectedInstr )
 {
-        if(findSelectedInstr)
-                unmuteAll( Hydrogen::get_instance()->getSelectedInstrumentNumber() );
-        else
-                unmuteAll( 0 );
+	if(findSelectedInstr)
+		unmuteAll( Hydrogen::get_instance()->getSelectedInstrumentNumber() );
+	else
+		unmuteAll( 0 );
 }
 
 void Mixer::unmuteAll( int selectedInstrument )
 {
-        Hydrogen *pEngine = Hydrogen::get_instance();
-        Song *pSong = pEngine->getSong();
-        InstrumentList *pInstrList = pSong->get_instrument_list();
-        int nInstruments = pInstrList->size();
-        for ( int i = 0; i < nInstruments; ++i ) {
-                m_pMixerLine[i]->setMuteClicked( false );
-                m_pMixerLine[i]->setSoloClicked( false );
-                pInstrList->get( i )->set_muted( false );
-        }
-        // select first instrument after unmute all
-        Hydrogen::get_instance()->setSelectedInstrumentNumber(selectedInstrument);
+	Hydrogen *pEngine = Hydrogen::get_instance();
+	Song *pSong = pEngine->getSong();
+	InstrumentList *pInstrList = pSong->get_instrument_list();
+	int nInstruments = pInstrList->size();
+	for ( int i = 0; i < nInstruments; ++i ) {
+		m_pMixerLine[i]->setMuteClicked( false );
+		m_pMixerLine[i]->setSoloClicked( false );
+		pInstrList->get( i )->set_muted( false );
+	}
+	// select first instrument after unmute all
+	Hydrogen::get_instance()->setSelectedInstrumentNumber(selectedInstrument);
 }
 
 void Mixer::soloClicked(MixerLine* ref)
@@ -273,31 +334,6 @@ void Mixer::soloClicked(uint nLine)
 	MixerLine * L = m_pMixerLine[ nLine ];
 	L->setSoloClicked( !L->isSoloClicked() );
 	soloClicked( L );
-// 	Hydrogen *pEngine = Hydrogen::get_instance();
-// 	Song *pSong = pEngine->getSong();
-// 	InstrumentList *pInstrList = pSong->get_instrument_list();
-// 	int nInstruments = pInstrList->size();
-// 
-// 	bool isSoloClicked = m_pMixerLine[ nLine ]->isSoloClicked();
-// 
-// 	if (!isSoloClicked) {
-// 		for ( int i = 0; i < nInstruments; i++ ) {
-// 			m_pMixerLine[i]->setSoloClicked( false );
-// 			m_pMixerLine[i]->setMuteClicked( true );
-// 			pInstrList->get( i )->set_muted( true );
-// 		}
-// 		m_pMixerLine[nLine]->setSoloClicked( true );
-// 		m_pMixerLine[nLine]->setMuteClicked( false );
-// 		pInstrList->get( nLine )->set_muted( false );
-// 	}
-// 	else {
-// 		for ( int i = 0; i < nInstruments; i++ ) {
-// 			m_pMixerLine[i]->setMuteClicked( false );
-// 			m_pMixerLine[i]->setSoloClicked( false );
-// 			pInstrList->get( i )->set_muted( false );
-// 		}
-// 	}
-
 }
 
 bool Mixer::isSoloClicked( uint n )
@@ -313,13 +349,13 @@ void Mixer::noteOnClicked( MixerLine* ref )
 	int nLine = findMixerLineByRef( ref );
 	Hydrogen::get_instance()->setSelectedInstrumentNumber( nLine );
 
-	Hydrogen *engine = Hydrogen::get_instance();
-	Song *song = engine->getSong();
-	InstrumentList *instrList = song->get_instrument_list();
+	Hydrogen *pEngine = Hydrogen::get_instance();
+	Song *song = pEngine->getSong();
+	InstrumentList *pInstrList = song->get_instrument_list();
 
 	const float fPitch = 0.0f;
-	Note *note = new Note( instrList->get(nLine), 0, 1.0, 0.5f, 0.5f, -1, fPitch );
-	AudioEngine::get_instance()->get_sampler()->note_on(note);
+	Note *pNote = new Note( pInstrList->get(nLine), 0, 1.0, 0.5f, 0.5f, -1, fPitch );
+	AudioEngine::get_instance()->get_sampler()->note_on(pNote);
 
 	Hydrogen::get_instance()->setSelectedInstrumentNumber(nLine);
 }
@@ -332,13 +368,13 @@ void Mixer::noteOnClicked( MixerLine* ref )
 	int nLine = findMixerLineByRef( ref );
 	Hydrogen::get_instance()->setSelectedInstrumentNumber( nLine );
 
-	Hydrogen *engine = Hydrogen::get_instance();
-	Song *song = engine->getSong();
-	InstrumentList *instrList = song->get_instrument_list();
+	Hydrogen *pEngine = Hydrogen::get_instance();
+	Song *pSong = pEngine->getSong();
+	InstrumentList *instrList = pSong->get_instrument_list();
 
 	const float fPitch = 0.0f;
-	Note *note = new Note( instrList->get( nLine ), 0, 1.0, 0.5, 0.5, -1, fPitch );
-	AudioEngine::get_instance()->get_sampler()->note_off(note);
+	Note *pNote = new Note( instrList->get( nLine ), 0, 1.0, 0.5, 0.5, -1, fPitch );
+	AudioEngine::get_instance()->get_sampler()->note_off(pNote);
 
 	Hydrogen::get_instance()->setSelectedInstrumentNumber(nLine);
 }
@@ -356,19 +392,31 @@ uint Mixer::findMixerLineByRef(MixerLine* ref)
 }
 
 
+uint Mixer::findCompoMixerLineByRef(ComponentMixerLine* ref)
+{
+	for (std::map<int, ComponentMixerLine*>::iterator it=m_pComponentMixerLine.begin(); it!=m_pComponentMixerLine.end(); ++it) {
+		if(it->second == ref)
+			return it->first;
+	}
+
+	return 0;
+}
+
+
+
 
 void Mixer::volumeChanged(MixerLine* ref)
 {
 	int nLine = findMixerLineByRef(ref);
 	Hydrogen::get_instance()->setSelectedInstrumentNumber( nLine );
 
-	Hydrogen *engine = Hydrogen::get_instance();
-	Song *song = engine->getSong();
-	InstrumentList *instrList = song->get_instrument_list();
+	Hydrogen *pEngine = Hydrogen::get_instance();
+	Song *pSong = pEngine->getSong();
+	InstrumentList *instrList = pSong->get_instrument_list();
 
-	Instrument *instr = instrList->get(nLine);
+	Instrument *pInstr = instrList->get(nLine);
 
-	instr->set_volume( ref->getVolume() );
+	pInstr->set_volume( ref->getVolume() );
 
 	Hydrogen::get_instance()->setSelectedInstrumentNumber(nLine);
 }
@@ -378,9 +426,9 @@ void Mixer::volumeChanged(MixerLine* ref)
 
 void Mixer::masterVolumeChanged(MasterMixerLine* ref)
 {
-	float volume = ref->getVolume();
-	Song *song = Hydrogen::get_instance()->getSong();
-	song->set_volume(volume);
+	float Volume = ref->getVolume();
+	Song *pSong = Hydrogen::get_instance()->getSong();
+	pSong->set_volume(Volume);
 }
 
 
@@ -393,13 +441,15 @@ void Mixer::updateMixer()
 	Hydrogen *pEngine = Hydrogen::get_instance();
 	Song *pSong = pEngine->getSong();
 	InstrumentList *pInstrList = pSong->get_instrument_list();
+	std::vector<DrumkitComponent*>* compoList = pSong->get_components();
 
 	uint nSelectedInstr = pEngine->getSelectedInstrumentNumber();
 
 	float fallOff = pPref->getMixerFalloffSpeed();
 
 	uint nMuteClicked = 0;
-	uint nInstruments = pInstrList->size();
+	int nInstruments = pInstrList->size();
+	int nCompo = compoList->size();
 	for ( unsigned nInstr = 0; nInstr < MAX_INSTRUMENTS; ++nInstr ) {
 
 		if ( nInstr >= nInstruments ) {	// unused instrument! let's hide and destroy the mixerline!
@@ -407,7 +457,7 @@ void Mixer::updateMixer()
 				delete m_pMixerLine[ nInstr ];
 				m_pMixerLine[ nInstr ] = NULL;
 
-				int newWidth = MIXER_STRIP_WIDTH * nInstruments;
+				int newWidth = MIXER_STRIP_WIDTH * ( nInstruments + nCompo );
 				if ( m_pFaderPanel->width() != newWidth ) {
 					m_pFaderPanel->resize( newWidth, height() );
 				}
@@ -418,9 +468,9 @@ void Mixer::updateMixer()
 			if ( m_pMixerLine[ nInstr ] == NULL ) {
 				// the mixerline doesn't exists..I'll create a new one!
 				m_pMixerLine[ nInstr ] = createMixerLine( nInstr );
-				m_pFaderHBox->addWidget( m_pMixerLine[ nInstr ] );
+				m_pFaderHBox->insertWidget( nInstr, m_pMixerLine[ nInstr ] );
 
-				int newWidth = MIXER_STRIP_WIDTH * nInstruments;
+				int newWidth = MIXER_STRIP_WIDTH * ( nInstruments + nCompo );
 				if ( m_pFaderPanel->width() != newWidth ) {
 					m_pFaderPanel->resize( newWidth, height() );
 				}
@@ -508,6 +558,97 @@ void Mixer::updateMixer()
 		}
 	}
 
+	for (std::vector<DrumkitComponent*>::iterator it = compoList->begin() ; it != compoList->end(); ++it) {
+		DrumkitComponent* p_compo = *it;
+
+		if( m_pComponentMixerLine.find(p_compo->get_id()) == m_pComponentMixerLine.end() ) {
+			// the mixerline doesn't exists..I'll create a new one!
+			m_pComponentMixerLine[ p_compo->get_id() ] = createComponentMixerLine( p_compo->get_id() );
+			m_pFaderHBox->addWidget( m_pComponentMixerLine[ p_compo->get_id() ] );
+
+			int newWidth = MIXER_STRIP_WIDTH * ( nInstruments + nCompo );
+			if ( m_pFaderPanel->width() != newWidth ) {
+				m_pFaderPanel->resize( newWidth, height() );
+			}
+		}
+
+		ComponentMixerLine *pLine = m_pComponentMixerLine[ p_compo->get_id() ];
+
+		float fNewPeak_L = p_compo->get_peak_l();
+		p_compo->set_peak_l( 0.0f );	// reset instrument peak
+
+		float fNewPeak_R = p_compo->get_peak_r();
+		p_compo->set_peak_r( 0.0f );	// reset instrument peak
+
+		float fNewVolume = p_compo->get_volume();
+		bool bMuted = p_compo->is_muted();
+
+		QString sName = p_compo->get_name();
+
+		float fOldPeak_L = pLine->getPeak_L();
+		float fOldPeak_R = pLine->getPeak_R();
+
+		if (!bShowPeaks) {
+			fNewPeak_L = 0.0f;
+			fNewPeak_R = 0.0f;
+		}
+
+		if ( fNewPeak_L >= fOldPeak_L) {	// LEFT peak
+			pLine->setPeak_L( fNewPeak_L );
+		}
+		else {
+			pLine->setPeak_L( fOldPeak_L / fallOff );
+		}
+		if ( fNewPeak_R >= fOldPeak_R) {	// Right peak
+			pLine->setPeak_R( fNewPeak_R );
+		}
+		else {
+			pLine->setPeak_R( fOldPeak_R / fallOff );
+		}
+
+		// fader position
+		pLine->setVolume( fNewVolume );
+
+		// mute
+		if ( bMuted ) {
+			nMuteClicked++;
+		}
+		pLine->setMuteClicked( bMuted );
+
+		// instr name
+		pLine->setName( sName );
+
+		pLine->updateMixerLine();
+	}
+
+	if( compoList->size() < m_pComponentMixerLine.size() ) {
+		std::vector<int>* p_ids_to_delete = new std::vector<int>();
+		for (std::map<int, ComponentMixerLine*>::iterator it=m_pComponentMixerLine.begin(); it!=m_pComponentMixerLine.end(); ++it) {
+
+			bool p_foundExistingRelatedComponent = false;
+			for ( std::vector<DrumkitComponent*>::iterator it2 = compoList->begin() ; it2 != compoList->end(); ++it2 ) {
+				DrumkitComponent* p_compo = *it2;
+				if( p_compo->get_id() == it->first ) {
+					p_foundExistingRelatedComponent = true;
+					break;
+				}
+			}
+			if( !p_foundExistingRelatedComponent )
+				p_ids_to_delete->push_back( it->first ) ;
+		}
+
+		for ( std::vector<int>::iterator it = p_ids_to_delete->begin() ; it != p_ids_to_delete->end(); ++it ) {
+			int p_compoID = *it;
+			delete m_pComponentMixerLine[p_compoID];
+			m_pComponentMixerLine.erase( p_compoID );
+
+			int newWidth = MIXER_STRIP_WIDTH * ( nInstruments + nCompo );
+			if ( m_pFaderPanel->width() != newWidth ) {
+				m_pFaderPanel->resize( newWidth, height() );
+			}
+		}
+	}
+
 	if (nMuteClicked == nInstruments - 1) {
 		// find the not muted button
 		for (uint i = 0; i < nInstruments; i++) {
@@ -647,13 +788,13 @@ void Mixer::panChanged(MixerLine* ref) {
 	int nLine = findMixerLineByRef(ref);
 	Hydrogen::get_instance()->setSelectedInstrumentNumber( nLine );
 
-	Hydrogen *engine = Hydrogen::get_instance();
-	Song *song = engine->getSong();
-	InstrumentList *instrList = song->get_instrument_list();
+	Hydrogen *pEngine = Hydrogen::get_instance();
+	Song *pSong = pEngine->getSong();
+	InstrumentList *pInstrList = pSong->get_instrument_list();
 
-	Instrument *instr = instrList->get(nLine);
-	instr->set_pan_l( pan_L );
-	instr->set_pan_r( pan_R );
+	Instrument *pInstr = pInstrList->get(nLine);
+	pInstr->set_pan_l( pan_L );
+	pInstr->set_pan_r( pan_R );
 
 
 	Hydrogen::get_instance()->setSelectedInstrumentNumber(nLine);
@@ -665,10 +806,10 @@ void Mixer::knobChanged(MixerLine* ref, int nKnob) {
 	int nLine = findMixerLineByRef(ref);
 	Hydrogen::get_instance()->setSelectedInstrumentNumber( nLine );
 
-	Hydrogen *engine = Hydrogen::get_instance();
-	Song *song = engine->getSong();
-	InstrumentList *instrList = song->get_instrument_list();
-	Instrument *pInstr = instrList->get(nLine);
+	Hydrogen *pEngine = Hydrogen::get_instance();
+	Song *pSong = pEngine->getSong();
+	InstrumentList *pInstrList = pSong->get_instrument_list();
+	Instrument *pInstr = pInstrList->get(nLine);
 	pInstr->set_fx_level( ref->getFXLevel(nKnob), nKnob );
 	QString sInfo = trUtf8( "Set FX %1 level ").arg( nKnob + 1 );
 	( HydrogenApp::get_instance() )->setStatusBarMessage( sInfo+ QString( "[%1]" ).arg( ref->getFXLevel(nKnob), 0, 'f', 2 ), 2000 );
@@ -690,27 +831,6 @@ void Mixer::noteOnEvent( int nInstrument )
 void Mixer::resizeEvent ( QResizeEvent *ev )
 {
 	UNUSED( ev );
-/*
-	uint nMaster_X = width() - MASTERMIXER_STRIP_WIDTH;
-	int nFXFrameWidth = 213;
-
-	if ( m_pFXFrame->isVisible() ) {
-		int w = width() - MASTERMIXER_STRIP_WIDTH - nFXFrameWidth;
-		m_pFaderScrollArea->resize( w, m_nMixerHeight );
-	}
-	else {
-		int w = width() - MASTERMIXER_STRIP_WIDTH;
-		m_pFaderScrollArea->resize( w, m_nMixerHeight );
-	}
-
-	m_pFXFrame->move( nMaster_X - nFXFrameWidth, 0 );
-
-	m_pMasterLine->move( nMaster_X, 0);
-	m_pShowPeaksBtn->move( nMaster_X + 67, 242 );
-	m_pShowFXPanelBtn->move( nMaster_X + 67, 258 );
-
-	resize(width(), m_nMixerHeight);	// qt bug workaround
-*/
 }
 
 
@@ -752,9 +872,6 @@ void Mixer::ladspaActiveBtnClicked( LadspaFXMixerLine* ref )
 #ifdef H2CORE_HAVE_LADSPA
 	bool bActive = ref->isFxActive();
 
-	//Hydrogen *engine = Hydrogen::get_instance();
-	//Song *song = engine->getSong();
-
 	for (uint nFX = 0; nFX < MAX_FX; nFX++) {
 		if (ref == m_pLadspaFXLine[ nFX ] ) {
 			LadspaFX *pFX = Effects::get_instance()->getLadspaFX(nFX);
@@ -779,7 +896,7 @@ void Mixer::ladspaEditBtnClicked( LadspaFXMixerLine *ref )
 			HydrogenApp::get_instance()->getLadspaFXProperties(nFX)->show();
 		}
 	}
-	Hydrogen::get_instance()->getSong()->__is_modified = true;
+	Hydrogen::get_instance()->getSong()->set_is_modified( true );
 #endif
 }
 
@@ -789,7 +906,7 @@ void Mixer::ladspaVolumeChanged( LadspaFXMixerLine* ref)
 {
 #ifdef H2CORE_HAVE_LADSPA
 	Song *pSong = (Hydrogen::get_instance() )->getSong();
-	pSong->__is_modified = true;
+	pSong->set_is_modified( true );
 
 	for (uint nFX = 0; nFX < MAX_FX; nFX++) {
 		if (ref == m_pLadspaFXLine[ nFX ] ) {
