@@ -29,8 +29,10 @@
 #include <hydrogen/hydrogen.h>
 #include <hydrogen/Preferences.h>
 #include <hydrogen/event_queue.h>
+#include <hydrogen/basics/drumkit_component.h>
 #include <hydrogen/basics/instrument.h>
 #include <hydrogen/basics/instrument_list.h>
+#include <hydrogen/basics/instrument_component.h>
 #include <hydrogen/basics/pattern.h>
 #include <hydrogen/basics/pattern_list.h>
 #include <hydrogen/basics/adsr.h>
@@ -72,7 +74,7 @@ DrumPatternEditor::DrumPatternEditor(QWidget* parent, PatternEditorPanel *panel)
 	resize( nEditorWidth, m_nEditorHeight );
 
 	HydrogenApp::get_instance()->addEventListener( this );
-	
+
 }
 
 
@@ -156,7 +158,7 @@ void DrumPatternEditor::mousePressEvent(QMouseEvent *ev)
 	int nColumn = getColumn( ev );
 	int nRealColumn = 0;
 	if( ev->x() > 20 ) {
-		nRealColumn = (ev->x() - 20) / static_cast<float>(m_nGridWidth);
+		nRealColumn = ev->x() / static_cast<float>(m_nGridWidth) - 20;
 	}
 	if ( nColumn >= (int)m_pPattern->get_length() ) {
 		update( 0, 0, width(), height() );
@@ -326,7 +328,7 @@ void DrumPatternEditor::addOrDeleteNoteAction(	int nColumn,
 			AudioEngine::get_instance()->get_sampler()->note_on(pNote2);
 		}
 	}
-	pSong->__is_modified = true;
+	pSong->set_is_modified( true );
 	AudioEngine::get_instance()->unlock(); // unlock the audio engine
 
 	// update the selected line
@@ -428,11 +430,11 @@ void DrumPatternEditor::mouseMoveEvent(QMouseEvent *ev)
 			fStep = pow( 1.0594630943593, ( double )fNotePitch );
 		}else
 		{
-			fStep = 1.0; 
+			fStep = 1.0;
 		}
 		m_pDraggedNote->set_length( nLen * fStep);
 
-		Hydrogen::get_instance()->getSong()->__is_modified = true;
+		Hydrogen::get_instance()->getSong()->set_is_modified( true );
 		AudioEngine::get_instance()->unlock(); // unlock the audio engine
 
 		//__draw_pattern();
@@ -474,7 +476,7 @@ void DrumPatternEditor::__draw_pattern(QPainter& painter)
 
 	InstrumentList * pInstrList = pSong->get_instrument_list();
 
-	
+
 	if ( m_nEditorHeight != (int)( m_nGridHeight * pInstrList->size() ) ) {
 		// the number of instruments is changed...recreate all
 		m_nEditorHeight = m_nGridHeight * pInstrList->size();
@@ -491,13 +493,13 @@ void DrumPatternEditor::__draw_pattern(QPainter& painter)
 
 	// draw the grid
 	__draw_grid( painter );
-	
+
 
 	/*
 		BUGFIX
-		
-		if m_pPattern is not renewed every time we draw a note, 
-		hydrogen will crash after you save a song and create a new one. 
+
+		if m_pPattern is not renewed every time we draw a note,
+		hydrogen will crash after you save a song and create a new one.
 		-smoors
 	*/
 	Hydrogen *pEngine = Hydrogen::get_instance();
@@ -608,7 +610,7 @@ void DrumPatternEditor::__draw_note( Note *note, QPainter& p )
 
 
 
-	}		
+	}
 	else {
 		float fNotePitch = note->get_octave() * 12 + note->get_key();
 		float fStep = pow( 1.0594630943593, ( double )fNotePitch );
@@ -779,7 +781,7 @@ void DrumPatternEditor::paintEvent( QPaintEvent* /*ev*/ )
 {
 	//INFOLOG( "paint" );
 	//QWidget::paintEvent(ev);
-	
+
 	QPainter painter( this );
 	__draw_pattern( painter );
 }
@@ -914,9 +916,10 @@ void DrumPatternEditor::undoRedoAction( int column,
 			pNote->set_lead_lag( leadLag );
 		}
 		else if ( mode == "NOTEKEY" ){
-            pNote->set_key_octave( (Note::Key)noteKeyVal, (Note::Octave)octaveKeyVal );
+			pNote->set_key_octave( (Note::Key)noteKeyVal, (Note::Octave)octaveKeyVal );
 		}
-		pSong->__is_modified = true;
+
+		pSong->set_is_modified( true );
 		break;
 	}
 	updateEditor();
@@ -974,7 +977,7 @@ void DrumPatternEditor::functionPasteNotesUndoAction(std::list<H2Core::Pattern*>
 	// Get song's pattern list
 	Hydrogen * H = Hydrogen::get_instance();
 	PatternList *patternList = H->getSong()->get_pattern_list();
-	
+
 	AudioEngine::get_instance()->lock( RIGHT_HERE );	// lock the audio engine
 
 	while (appliedList.size() > 0)
@@ -982,7 +985,7 @@ void DrumPatternEditor::functionPasteNotesUndoAction(std::list<H2Core::Pattern*>
 		// Get next applied pattern
 		Pattern *pApplied = appliedList.front();
 		assert(pApplied);
-		
+
 		// Find destination pattern to perform undo
 		Pattern *pat = patternList->find(pApplied->get_name());
 
@@ -995,7 +998,7 @@ void DrumPatternEditor::functionPasteNotesUndoAction(std::list<H2Core::Pattern*>
 				// Get note to remove
 				Note *pNote = it->second;
 				assert(pNote);
-				
+
 				// Check if note is not present
 				Pattern::notes_t* notes = (Pattern::notes_t *)pat->get_notes();
 				FOREACH_NOTE_IT_BOUND(notes, it, pNote->get_position())
@@ -1010,8 +1013,8 @@ void DrumPatternEditor::functionPasteNotesUndoAction(std::list<H2Core::Pattern*>
 				}
 			}
 		}
-		
-		
+
+
 		// Remove applied pattern;
 		delete pApplied;
 		appliedList.pop_front();
@@ -1033,7 +1036,7 @@ void DrumPatternEditor::functionPasteNotesRedoAction(std::list<H2Core::Pattern*>
 {
 	Hydrogen * H = Hydrogen::get_instance();
 	PatternList *patternList = H->getSong()->get_pattern_list();
-	
+
 	AudioEngine::get_instance()->lock( RIGHT_HERE );	// lock the audio engine
 
 	// Add notes to pattern
@@ -1053,7 +1056,7 @@ void DrumPatternEditor::functionPasteNotesRedoAction(std::list<H2Core::Pattern*>
 					pat->get_info(),
 					pat->get_category(),
 					pat->get_length());
-			
+
 			// Add all notes of source pattern to destination pattern
 			// and store all applied notes in applied pattern
 			const Pattern::notes_t* notes = pPattern->get_notes();
@@ -1061,7 +1064,7 @@ void DrumPatternEditor::functionPasteNotesRedoAction(std::list<H2Core::Pattern*>
 			{
 				Note *pNote = it->second;
 				assert(pNote);
-				
+
 				// Check if note is not present
 				bool noteExists = false;
 				const Pattern::notes_t* notes = pat->get_notes();
@@ -1075,7 +1078,7 @@ void DrumPatternEditor::functionPasteNotesRedoAction(std::list<H2Core::Pattern*>
 						break;
 					}
 				}
-				
+
 				// Apply note and store it as applied
 				if (!noteExists)
 				{
@@ -1083,7 +1086,7 @@ void DrumPatternEditor::functionPasteNotesRedoAction(std::list<H2Core::Pattern*>
 					pApplied->insert_note(new Note(pNote));
 				}
 			}
-			
+
 			// Add applied pattern to applied list
 			appliedList.push_back(pApplied);
 		}
@@ -1226,40 +1229,95 @@ void DrumPatternEditor::functionMoveInstrumentAction( int nSourceInstrument,  in
 			return;
 		}
 
-        pInstrumentList->move( nSourceInstrument, nTargetInstrument );
+		pInstrumentList->move( nSourceInstrument, nTargetInstrument );
 
 		#ifdef H2CORE_HAVE_JACK
-		engine->renameJackPorts();
+		engine->renameJackPorts( pSong );
 		#endif
 
 		AudioEngine::get_instance()->unlock();
 		engine->setSelectedInstrumentNumber( nTargetInstrument );
 
-		pSong->__is_modified = true;
+		pSong->set_is_modified( true );
 }
 
 
-void  DrumPatternEditor::functionDropInstrumentUndoAction( int nTargetInstrument )
+void  DrumPatternEditor::functionDropInstrumentUndoAction( int nTargetInstrument, std::vector<int>* AddedComponents )
 {
 	Hydrogen *pEngine = Hydrogen::get_instance();
 	pEngine->removeInstrument( nTargetInstrument, false );
-	
+
+	std::vector<DrumkitComponent*>* pDrumkitComponents = pEngine->getSong()->get_components();
+
+	for (std::vector<int>::iterator it = AddedComponents->begin() ; it != AddedComponents->end(); ++it) {
+		int p_compoID = *it;
+
+		for ( int n = 0 ; n < pDrumkitComponents->size() ; n++ ) {
+			DrumkitComponent* pTmpDrumkitComponent = pDrumkitComponents->at( n );
+			if( pTmpDrumkitComponent->get_id() == p_compoID ) {
+				pDrumkitComponents->erase( pDrumkitComponents->begin() + n );
+				break;
+			}
+		}
+	}
+
 	AudioEngine::get_instance()->lock( RIGHT_HERE );
 #ifdef H2CORE_HAVE_JACK
-	pEngine->renameJackPorts();
+	Song *pSong = pEngine->getSong();
+	pEngine->renameJackPorts(pSong);
 #endif
 	AudioEngine::get_instance()->unlock();
 	updateEditor();
 }
 
 
-void  DrumPatternEditor::functionDropInstrumentRedoAction( QString sDrumkitName, QString sInstrumentName, int nTargetInstrument )
+void  DrumPatternEditor::functionDropInstrumentRedoAction( QString sDrumkitName, QString sInstrumentName, int nTargetInstrument, std::vector<int>* AddedComponents)
 {
 		Instrument *pNewInstrument = Instrument::load_instrument( sDrumkitName, sInstrumentName );
-		if( pNewInstrument == NULL ) return;		
+		if( pNewInstrument == NULL ){
+			return;
+		}
+
+		Drumkit *pNewDrumkit = Drumkit::load_by_name( sDrumkitName, false );
+		if( pNewDrumkit == NULL ){
+			return;
+		}
 
 		Hydrogen *pEngine = Hydrogen::get_instance();
 
+		AudioEngine::get_instance()->lock( RIGHT_HERE );
+
+		std::vector<InstrumentComponent*>* pOldInstrumentComponents = new std::vector<InstrumentComponent*> ( pNewInstrument->get_components()->begin(), pNewInstrument->get_components()->end() );
+		pNewInstrument->get_components()->clear();
+
+		for (std::vector<DrumkitComponent*>::iterator it = pNewDrumkit->get_components()->begin() ; it != pNewDrumkit->get_components()->end(); ++it) {
+			DrumkitComponent* pComponent = *it;
+			int OldID = pComponent->get_id();
+			int NewID = -1;
+
+			NewID = findExistingCompo( pComponent->get_name() );
+
+			if ( NewID == -1 ) {
+				NewID = findFreeCompoID();
+
+				AddedComponents->push_back( NewID );
+
+				pComponent->set_id( NewID );
+				pComponent->set_name( renameCompo( pComponent->get_name() ) );
+				Hydrogen::get_instance()->getSong()->get_components()->push_back( pComponent );
+			}
+
+			for ( std::vector<InstrumentComponent*>::iterator it2 = pOldInstrumentComponents->begin() ; it2 != pOldInstrumentComponents->end(); ++it2 ) {
+				InstrumentComponent* pOldInstrCompo = *it2;
+				if( pOldInstrCompo->get_drumkit_componentID() == OldID ) {
+					InstrumentComponent* pNewInstrCompo = new InstrumentComponent( pOldInstrCompo );
+					pNewInstrCompo->set_drumkit_componentID( NewID );
+
+					pNewInstrument->get_components()->push_back( pNewInstrCompo );
+				}
+			}
+		}
+
 		// create a new valid ID for this instrument
 		int nID = -1;
 		for ( uint i = 0; i < pEngine->getSong()->get_instrument_list()->size(); ++i ) {
@@ -1272,11 +1330,10 @@ void  DrumPatternEditor::functionDropInstrumentRedoAction( QString sDrumkitName,
 
 		pNewInstrument->set_id( nID );
 
-		AudioEngine::get_instance()->lock( RIGHT_HERE );
 		pEngine->getSong()->get_instrument_list()->add( pNewInstrument );
 
 		#ifdef H2CORE_HAVE_JACK
-		pEngine->renameJackPorts();
+		pEngine->renameJackPorts( pEngine->getSong() );
 		#endif
 
 		AudioEngine::get_instance()->unlock();
@@ -1285,9 +1342,52 @@ void  DrumPatternEditor::functionDropInstrumentRedoAction( QString sDrumkitName,
 
 		// select the new instrument
 		pEngine->setSelectedInstrumentNumber(nTargetInstrument);
+		EventQueue::get_instance()->push_event( EVENT_SELECTED_INSTRUMENT_CHANGED, -1 );
 		updateEditor();
 }
 
+QString DrumPatternEditor::renameCompo( QString OriginalName )
+{
+	std::vector<DrumkitComponent*>* pComponentList = Hydrogen::get_instance()->getSong()->get_components();
+	for (std::vector<DrumkitComponent*>::iterator it = pComponentList->begin() ; it != pComponentList->end(); ++it) {
+		DrumkitComponent* pComponent = *it;
+		if( pComponent->get_name().compare( OriginalName ) == 0 ){
+			return renameCompo( OriginalName + "_new" );
+		}
+	}
+	return OriginalName;
+}
+
+int DrumPatternEditor::findFreeCompoID( int startingPoint )
+{
+	bool FoundFreeSlot = true;
+	std::vector<DrumkitComponent*>* pComponentList = Hydrogen::get_instance()->getSong()->get_components();
+	for (std::vector<DrumkitComponent*>::iterator it = pComponentList->begin() ; it != pComponentList->end(); ++it) {
+		DrumkitComponent* pComponent = *it;
+		if( pComponent->get_id() == startingPoint ) {
+			FoundFreeSlot = false;
+			break;
+		}
+	}
+
+	if(FoundFreeSlot){
+		return startingPoint;
+	} else {
+		return findFreeCompoID( startingPoint + 1 );
+	}
+}
+
+int DrumPatternEditor::findExistingCompo( QString SourceName )
+{
+	std::vector<DrumkitComponent*>* pComponentList = Hydrogen::get_instance()->getSong()->get_components();
+	for (std::vector<DrumkitComponent*>::iterator it = pComponentList->begin() ; it != pComponentList->end(); ++it) {
+		DrumkitComponent* pComponent = *it;
+		if ( pComponent->get_name().compare( SourceName ) == 0 ){
+			return pComponent->get_id();
+		}
+	}
+	return -1;
+}
 
 
 
@@ -1301,7 +1401,7 @@ void DrumPatternEditor::functionDeleteInstrumentUndoAction( std::list< H2Core::N
 	{
 		pNewInstrument = Instrument::load_instrument( drumkitName, instrumentName );
 	}
-	if( pNewInstrument == NULL ) return;		
+	if( pNewInstrument == NULL ) return;
 
 	// create a new valid ID for this instrument
 	int nID = -1;
@@ -1320,7 +1420,7 @@ void DrumPatternEditor::functionDeleteInstrumentUndoAction( std::list< H2Core::N
 	pEngine->getSong()->get_instrument_list()->add( pNewInstrument );
 
 	#ifdef H2CORE_HAVE_JACK
-	pEngine->renameJackPorts();
+	pEngine->renameJackPorts( pEngine->getSong() );
 	#endif
 
 	AudioEngine::get_instance()->unlock();	// unlock the audio engine
@@ -1358,10 +1458,10 @@ void DrumPatternEditor::functionAddEmptyInstrumentUndo()
 
 	Hydrogen *pEngine = Hydrogen::get_instance();
 	pEngine->removeInstrument( pEngine->getSong()->get_instrument_list()->size() -1 , false );
-	
+
 	AudioEngine::get_instance()->lock( RIGHT_HERE );
 #ifdef H2CORE_HAVE_JACK
-	pEngine->renameJackPorts();
+	pEngine->renameJackPorts( pEngine->getSong() );
 #endif
 	AudioEngine::get_instance()->unlock();
 	updateEditor();
@@ -1371,7 +1471,8 @@ void DrumPatternEditor::functionAddEmptyInstrumentUndo()
 void DrumPatternEditor::functionAddEmptyInstrumentRedo()
 {
 	AudioEngine::get_instance()->lock( RIGHT_HERE );
-	InstrumentList* pList = Hydrogen::get_instance()->getSong()->get_instrument_list();
+	Song* pSong = Hydrogen::get_instance()->getSong();
+	InstrumentList* pList = pSong->get_instrument_list();
 
 	// create a new valid ID for this instrument
 	int nID = -1;
@@ -1385,11 +1486,11 @@ void DrumPatternEditor::functionAddEmptyInstrumentRedo()
 
 	Instrument *pNewInstr = new Instrument( nID, "New instrument");
 	pList->add( pNewInstr );
-	
+
 	#ifdef H2CORE_HAVE_JACK
-	Hydrogen::get_instance()->renameJackPorts();
+	Hydrogen::get_instance()->renameJackPorts( pSong );
 	#endif
-	
+
 	AudioEngine::get_instance()->unlock();
 
 	Hydrogen::get_instance()->setSelectedInstrumentNumber( pList->size() - 1 );
