@@ -74,7 +74,7 @@ InstrumentLine::InstrumentLine(QWidget* pParent)
 	);
 	m_pMuteBtn->move( 145, 5 );
 	m_pMuteBtn->setPressed(false);
-    m_pMuteBtn->setToolTip( trUtf8("Mute instrument") );
+	m_pMuteBtn->setToolTip( trUtf8("Mute instrument") );
 	connect(m_pMuteBtn, SIGNAL(clicked(Button*)), this, SLOT(muteClicked()));
 
 	m_pSoloBtn = new ToggleButton(
@@ -86,7 +86,7 @@ InstrumentLine::InstrumentLine(QWidget* pParent)
 	);
 	m_pSoloBtn->move( 163, 5 );
 	m_pSoloBtn->setPressed(false);
-    m_pSoloBtn->setToolTip( trUtf8("Solo") );
+	m_pSoloBtn->setToolTip( trUtf8("Solo") );
 	connect(m_pSoloBtn, SIGNAL(clicked(Button*)), this, SLOT(soloClicked()));
 
 
@@ -111,7 +111,7 @@ InstrumentLine::InstrumentLine(QWidget* pParent)
 	m_pCopyPopupSub->addAction( trUtf8( "Only for this pattern" ), this, SLOT( functionCopyInstrumentPattern() ) );
 	m_pCopyPopupSub->addAction( trUtf8( "For all patterns" ), this, SLOT( functionCopyAllInstrumentPatterns() ) );
 	m_pFunctionPopup->addMenu( m_pCopyPopupSub );
-	
+
 	m_pPastePopupSub = new QMenu( trUtf8( "Paste notes ..." ), m_pFunctionPopup );
 	m_pPastePopupSub->addAction( trUtf8( "Only for this pattern" ), this, SLOT( functionPasteInstrumentPattern() ) );
 	m_pPastePopupSub->addAction( trUtf8( "For all patterns" ), this, SLOT( functionPasteAllInstrumentPatterns() ) );
@@ -200,9 +200,9 @@ void InstrumentLine::mousePressEvent(QMouseEvent *ev)
 		const int nLength = -1;
 		const float fPitch = 0.0f;
 		Song *pSong = Hydrogen::get_instance()->getSong();
-		
+
 		Instrument *pInstr = pSong->get_instrument_list()->get( m_nInstrumentNumber );
-		
+
 		Note *pNote = new Note( pInstr, 0, velocity, pan_L, pan_R, nLength, fPitch);
 		AudioEngine::get_instance()->get_sampler()->note_on(pNote);
 	}
@@ -241,8 +241,8 @@ void InstrumentLine::functionClearNotes()
 	Instrument *pSelectedInstrument = pEngine->getSong()->get_instrument_list()->get( m_nInstrumentNumber );
 
 	std::list< Note* > noteList;
-    const Pattern::notes_t* notes = pPattern->get_notes();
-    FOREACH_NOTE_CST_IT_BEGIN_END(notes,it) {
+	const Pattern::notes_t* notes = pPattern->get_notes();
+	FOREACH_NOTE_CST_IT_BEGIN_END(notes,it) {
 		Note *pNote = it->second;
 		assert( pNote );
 		if ( pNote->get_instrument() == pSelectedInstrument ) {
@@ -261,7 +261,7 @@ void InstrumentLine::functionCopyInstrumentPattern()
 	int selectedPatternNr = pEngine->getSelectedPatternNumber();
 	Song *song = pEngine->getSong();
 	assert(song);
-	
+
 	// Serialize & put to clipboard
 	QString serialized = LocalFileMng::copyInstrumentLineToString(song, selectedPatternNr, m_nInstrumentNumber);
 	QClipboard *clipboard = QApplication::clipboard();
@@ -273,7 +273,7 @@ void InstrumentLine::functionCopyAllInstrumentPatterns()
 	Hydrogen * pEngine = Hydrogen::get_instance();
 	Song *song = pEngine->getSong();
 	assert(song);
-	
+
 	// Serialize & put to clipboard
 	QString serialized = LocalFileMng::copyInstrumentLineToString(song, -1, m_nInstrumentNumber);
 	QClipboard *clipboard = QApplication::clipboard();
@@ -284,7 +284,7 @@ void InstrumentLine::functionPasteInstrumentPattern()
 {
 	Hydrogen * pEngine = Hydrogen::get_instance();
 	int selectedPatternNr = pEngine->getSelectedPatternNumber();
-	
+
 	functionPasteInstrumentPatternExec(selectedPatternNr);
 }
 
@@ -298,20 +298,20 @@ void InstrumentLine::functionPasteInstrumentPatternExec(int patternID)
 	Hydrogen * pEngine = Hydrogen::get_instance();
 	Song *song = pEngine->getSong();
 	assert(song);
-	
+
 	// This is a note list for pasted notes collection
 	std::list< Pattern* > patternList;
-	
+
 	// Get from clipboard & deserialize
 	QClipboard *clipboard = QApplication::clipboard();
 	QString serialized = clipboard->text();
 	if (!LocalFileMng::pasteInstrumentLineFromString(song, serialized, patternID, m_nInstrumentNumber, patternList))
 		return;
-	
+
 	// Ignore empty result
 	if (patternList.size() <= 0)
 		return;
-	
+
 	// Create action
 	SE_pasteNotesPatternEditorAction *action = new SE_pasteNotesPatternEditorAction(patternList);
 	HydrogenApp::get_instance()->m_undoStack->push(action);
@@ -355,8 +355,8 @@ void InstrumentLine::functionFillNotes( int every )
 
 			for (int i = 0; i < nPatternSize; i += nResolution) {
 				bool noteAlreadyPresent = false;
-                const Pattern::notes_t* notes = pCurrentPattern->get_notes();
-                FOREACH_NOTE_CST_IT_BOUND(notes,it,i) {
+				const Pattern::notes_t* notes = pCurrentPattern->get_notes();
+				FOREACH_NOTE_CST_IT_BOUND(notes,it,i) {
 					Note *pNote = it->second;
 					if ( pNote->get_instrument() == instrRef ) {
 						// note already exists
@@ -396,7 +396,7 @@ void InstrumentLine::functionRandomizeVelocity()
 	int nResolution = 4 * MAX_NOTES / ( nBase * pPatternEditor->getResolution() );
 
 	Song *pSong = pEngine->getSong();
-	
+
 	QStringList noteVeloValue;
  	QStringList oldNoteVeloValue;
 
@@ -409,8 +409,8 @@ void InstrumentLine::functionRandomizeVelocity()
 			Instrument *instrRef = (pSong->get_instrument_list())->get( nSelectedInstrument );
 
 			for (int i = 0; i < nPatternSize; i += nResolution) {
-                const Pattern::notes_t* notes = pCurrentPattern->get_notes();
-                FOREACH_NOTE_CST_IT_BOUND(notes,it,i) {
+				const Pattern::notes_t* notes = pCurrentPattern->get_notes();
+				FOREACH_NOTE_CST_IT_BOUND(notes,it,i) {
 					Note *pNote = it->second;
 					if ( pNote->get_instrument() == instrRef ) {
 						float fVal = ( rand() % 100 ) / 100.0;
@@ -597,7 +597,7 @@ void PatternEditorInstrumentList::dropEvent(QDropEvent *event)
 {
 	//WARNINGLOG("Drop!");
 	QString sText = event->mimeData()->text();
-	
+
 
 	if(sText.startsWith("Songs:") || sText.startsWith("Patterns:") || sText.startsWith("move pattern:") || sText.startsWith("drag pattern:")) return;
 
@@ -608,6 +608,10 @@ void PatternEditorInstrumentList::dropEvent(QDropEvent *event)
 
 		int nTargetInstrument = event->pos().y() / m_nGridHeight;
 
+		if( nTargetInstrument > engine->getSong()->get_instrument_list()->size() ){
+			nTargetInstrument = engine->getSong()->get_instrument_list()->size() - 1;
+		}
+
 		if ( nSourceInstrument == nTargetInstrument ) {
 			event->acceptProposedAction();
 			return;
@@ -631,11 +635,16 @@ void PatternEditorInstrumentList::dropEvent(QDropEvent *event)
 
 		/*
 		    "X > 181": border between the instrument names on the left and the grid
-		    Because the right part of the grid starts above the name column, we have to subtract the difference 
+		    Because the right part of the grid starts above the name column, we have to subtract the difference
 		*/
 		if (  event->pos().x() > 181 ) nTargetInstrument = ( event->pos().y() - 90 )  / m_nGridHeight ;
 
-		SE_dragInstrumentAction *action = new SE_dragInstrumentAction( sDrumkitName, sInstrumentName, nTargetInstrument );
+		Hydrogen *engine = Hydrogen::get_instance();
+		if( nTargetInstrument > engine->getSong()->get_instrument_list()->size() ){
+			nTargetInstrument = engine->getSong()->get_instrument_list()->size();
+		}
+
+		SE_dragInstrumentAction *action = new SE_dragInstrumentAction( sDrumkitName, sInstrumentName, nTargetInstrument);
 		HydrogenApp::get_instance()->m_undoStack->push( action );
 
 		event->acceptProposedAction();
