@@ -878,7 +878,7 @@ void PatternEditorPanel::moveUpBtnClicked(Button *)
 		AudioEngine::get_instance()->unlock();
 		engine->setSelectedInstrumentNumber( nSelectedInstrument - 1 );
 
-		pSong->__is_modified = true;
+		pSong->set_is_modified( true );
 	}
 	else {
 		AudioEngine::get_instance()->unlock();
@@ -903,7 +903,7 @@ void PatternEditorPanel::moveDownBtnClicked(Button *)
 		AudioEngine::get_instance()->unlock();
 		engine->setSelectedInstrumentNumber( nSelectedInstrument + 1 );
 
-		pSong->__is_modified = true;
+		pSong->set_is_modified( true );
 	}
 	else {
 		AudioEngine::get_instance()->unlock();
