@@ -335,22 +335,24 @@ PlayerControl::PlayerControl(QWidget *parent)
 			"/lcd/LCDSpinBox_up_on.png",
 			"/lcd/LCDSpinBox_up_off.png",
 			"/lcd/LCDSpinBox_up_over.png",
-			QSize(16, 8)
+			QSize(16, 8),
+			false,
+			true
 	);
 	m_pBPMUpBtn->move( 12, 5 );
 	connect( m_pBPMUpBtn, SIGNAL( clicked( Button* ) ), this, SLOT(bpmButtonClicked( Button* ) ) );
-	connect( m_pBPMUpBtn, SIGNAL( mousePress( Button* ) ), this, SLOT(bpmButtonPressed( Button* ) ) );
 
 	m_pBPMDownBtn = new Button(
 			pBPMPanel,
 			"/lcd/LCDSpinBox_down_on.png",
 			"/lcd/LCDSpinBox_down_off.png",
 			"/lcd/LCDSpinBox_down_over.png",
-			QSize(16, 8)
+			QSize(16, 8),
+			false,
+			true
 	);
 	m_pBPMDownBtn->move( 12, 14 );
 	connect( m_pBPMDownBtn, SIGNAL( clicked( Button* ) ), this, SLOT(bpmButtonClicked( Button* ) ) );
-	connect( m_pBPMDownBtn, SIGNAL( mousePress( Button* ) ), this, SLOT(bpmButtonPressed( Button* ) ) );
 
 	m_pRubberBPMChange = new ToggleButton(
 			pBPMPanel,
@@ -484,9 +486,6 @@ PlayerControl::PlayerControl(QWidget *parent)
 	connect(timer, SIGNAL(timeout()), this, SLOT(updatePlayerControl()));
 	timer->start(100);	// update player control at 10 fps
 
-	m_pBPMTimer = new QTimer( this );
-	connect(m_pBPMTimer, SIGNAL(timeout()), this, SLOT(onBpmTimerEvent()));
-
 	m_pStatusTimer = new QTimer( this );
 	connect( m_pStatusTimer, SIGNAL( timeout() ), this, SLOT( onStatusTimerEvent() ) );
 
@@ -509,6 +508,7 @@ void PlayerControl::updatePlayerControl()
 {
 	Preferences *pPref = Preferences::get_instance();
 	HydrogenApp *pH2App = HydrogenApp::get_instance();
+
 	m_pShowMixerBtn->setPressed( pH2App->getMixer()->isVisible() );
 	m_pShowInstrumentRackBtn->setPressed( pH2App->getInstrumentRack()->isVisible() );
 
@@ -577,7 +577,9 @@ void PlayerControl::updatePlayerControl()
 
 
 #ifdef H2CORE_HAVE_JACK
-	if ( pPref->m_sAudioDriver == "Jack" ) {
+	AudioOutput *p_Driver = m_pEngine->getAudioOutput();
+
+	if ( p_Driver && strncmp(p_Driver->class_name(), "JackOutput", 10) == 0){
 		m_pJackTransportBtn->show();
 		switch ( pPref->m_bJackTransportMode ) {
 			case Preferences::NO_JACK_TRANSPORT:
@@ -678,12 +680,6 @@ void PlayerControl::updatePlayerControl()
 
 	}
 	//~ beatcounter
-
-	//timeline check
-//	if( Preferences::get_instance()->__usetimeline ){
-//		m_pRubberBPMChange->setPressed( false );
-//		Preferences::get_instance()->m_useTheRubberbandBpmChangeEvent = false;
-//	}
 }
 
 
@@ -770,7 +766,6 @@ void PlayerControl::liveModeBtnClicked(Button* ref)
 
 	m_pEngine->sequencer_stop();
 	m_pEngine->getSong()->set_mode( Song::PATTERN_MODE );
-	//m_pEngine->sequencer_setNextPattern( m_pEngine->getSelectedPatternNumber() );	// imposto il pattern correntemente selezionato come il prossimo da suonare
 	m_pSongModeBtn->setPressed(false);
 	m_pLiveModeBtn->setPressed(true);
 	(HydrogenApp::get_instance())->setStatusBarMessage(trUtf8("Pattern mode selected."), 5000);
@@ -787,7 +782,7 @@ void PlayerControl::bpmChanged() {
 		fNewBpmValue = 400;
 	}
 
-	m_pEngine->getSong()->__is_modified = true;
+	m_pEngine->getSong()->set_is_modified( true );
 
 	AudioEngine::get_instance()->lock( RIGHT_HERE );
 	m_pEngine->setBPM( fNewBpmValue );
@@ -907,8 +902,9 @@ void PlayerControl::bctButtonClicked( Button* tBtn)
 void PlayerControl::jackTransportBtnClicked( Button* )
 {
 	Preferences *pPref = Preferences::get_instance();
+	AudioOutput *p_Driver = m_pEngine->getAudioOutput();
 
-	if (pPref->m_sAudioDriver != "Jack") {
+	if ( ! ( p_Driver && strncmp(p_Driver->class_name(), "JackOutput", 10) == 0 ) ){
 		QMessageBox::warning( this, "Hydrogen", trUtf8( "JACK-transport will work only with JACK driver." ) );
 		return;
 	}
@@ -935,8 +931,9 @@ void PlayerControl::jackMasterBtnClicked( Button* )
 {
 #ifdef H2CORE_HAVE_JACK
 	Preferences *pPref = Preferences::get_instance();
+	AudioOutput *p_Driver = m_pEngine->getAudioOutput();
 
-	if (pPref->m_sAudioDriver != "Jack") {
+	if ( ! ( p_Driver && strncmp(p_Driver->class_name(), "JackOutput", 10) == 0 ) ){
 		QMessageBox::warning( this, "Hydrogen", trUtf8( "JACK-transport will work only with JACK driver." ) );
 		return;
 	}
@@ -954,7 +951,6 @@ void PlayerControl::jackMasterBtnClicked( Button* )
 		pPref->m_bJackMasterMode = Preferences::NO_JACK_TIME_MASTER;
 		AudioEngine::get_instance()->unlock();
 		(HydrogenApp::get_instance())->setStatusBarMessage(trUtf8(" Jack-Time-Master mode = Off"), 5000);
-		//m_pControlsBBTPanel->hide();
 		Hydrogen::get_instance()->offJackMaster();
 	}
 #endif
@@ -970,7 +966,7 @@ void PlayerControl::bpmClicked()
 			return;
 		}
 
-		m_pEngine->getSong()->__is_modified  = true;
+		m_pEngine->getSong()->set_is_modified( true );
 
 		AudioEngine::get_instance()->lock( RIGHT_HERE );
 		m_pEngine->setBPM( fNewVal );
@@ -982,34 +978,12 @@ void PlayerControl::bpmClicked()
 }
 
 
-void PlayerControl::bpmButtonPressed( Button* pBtn)
+void PlayerControl::bpmButtonClicked( Button* pBtn )
 {
-	if ( pBtn == m_pBPMUpBtn ) {
+	if ( pBtn == m_pBPMUpBtn )
 		m_pLCDBPMSpinbox->upBtnClicked();
-		m_nBPMIncrement = 1;
-	}
-	else {
+	else
 		m_pLCDBPMSpinbox->downBtnClicked();
-		m_nBPMIncrement = -1;
-	}
-	m_pBPMTimer->start( 100 );
-}
-
-
-void PlayerControl::bpmButtonClicked( Button* )
-{
-	m_pBPMTimer->stop();
-}
-
-
-void PlayerControl::onBpmTimerEvent()
-{
-	if (m_nBPMIncrement == 1) {
-		m_pLCDBPMSpinbox->upBtnClicked();
-	}
-	else {
-		m_pLCDBPMSpinbox->downBtnClicked();
-	}
 }
 
 
@@ -1035,7 +1009,7 @@ void PlayerControl::songLoopBtnClicked( Button* )
 	Hydrogen *pEngine = Hydrogen::get_instance();
 	Song *song = pEngine->getSong();
 	song->set_loop_enabled( ! song->is_loop_enabled() );
-	song->__is_modified = true;
+	song->set_is_modified( true );
 
 	if ( song->is_loop_enabled() ) {
 		HydrogenApp::get_instance()->setStatusBarMessage(trUtf8("Loop song = On"), 5000);
