@@ -31,7 +31,7 @@
 
 #include <QPixmap>
 #include <QFontDialog>
-#include "widgets/midiTable.h"
+#include "widgets/MidiTable.h"
 
 #include <hydrogen/midi_map.h>
 #include <hydrogen/hydrogen.h>
@@ -40,6 +40,9 @@
 #include <hydrogen/LashClient.h>
 #include <hydrogen/audio_engine.h>
 #include <hydrogen/sampler/Sampler.h>
+#include "SongEditor/SongEditor.h"
+#include "SongEditor/SongEditorPanel.h"
+
 
 using namespace H2Core;
 
@@ -116,6 +119,8 @@ PreferencesDialog::PreferencesDialog(QWidget* parent)
 	}
 
 	m_pIgnoreNoteOffCheckBox->setChecked( pPref->m_bMidiNoteOffIgnore );
+	m_pDiscardMidiMsgCheckbox->setChecked( pPref->m_bMidiDiscardNoteAfterAction );
+	m_pFixedMapping->setChecked( pPref->m_bMidiFixedMapping );
 
 	updateDriverInfo();
 
@@ -201,6 +206,24 @@ PreferencesDialog::PreferencesDialog(QWidget* parent)
 		i++;
 	}
 
+	//SongEditor coloring
+	int coloringMethod = pPref->getColoringMethod();
+	int coloringMethodAuxValue = pPref->getColoringMethodAuxValue();
+
+	coloringMethodCombo->clear();
+	coloringMethodCombo->addItem(trUtf8("Automatic"));
+	coloringMethodCombo->addItem(trUtf8("Steps"));
+	coloringMethodCombo->addItem(trUtf8("Fixed"));
+
+	coloringMethodAuxSpinBox->setMaximum(300);
+
+	coloringMethodCombo->setCurrentIndex( coloringMethod );
+	coloringMethodAuxSpinBox->setValue( coloringMethodAuxValue );
+
+	coloringMethodCombo_currentIndexChanged( coloringMethod );
+
+	connect(coloringMethodCombo, SIGNAL(currentIndexChanged(int)), this, SLOT( coloringMethodCombo_currentIndexChanged(int) ));
+
 
 	// midi tab
 	midiPortChannelComboBox->setEnabled( false );
@@ -236,6 +259,7 @@ PreferencesDialog::PreferencesDialog(QWidget* parent)
 	// General tab
 	restoreLastUsedSongCheckbox->setChecked( pPref->isRestoreLastSongEnabled() );
 	restoreLastUsedPlaylistCheckbox->setChecked( pPref->isRestoreLastPlaylistEnabled() );
+	useRelativePlaylistPathsCheckbox->setChecked( pPref->isPlaylistUsingRelativeFilenames() );
 
 	//restore the right m_bsetlash value
 	if ( pPref->m_brestartLash == true ){
@@ -336,8 +360,12 @@ void PreferencesDialog::on_okBtn_clicked()
 	// JACK
 	pPref->m_bJackConnectDefaults = connectDefaultsCheckBox->isChecked();
 
+	/*
+	 * 0: Post-Fader
+	 * 1: Pre-Fader
+	 */
 
-	if (trackOutputComboBox->currentText() == "Post-Fader")
+	if (trackOutputComboBox->currentIndex() == Preferences::POST_FADER)
 	{
 		pPref->m_nJackTrackOutputMode = Preferences::POST_FADER;
 	} else {
@@ -382,6 +410,8 @@ void PreferencesDialog::on_okBtn_clicked()
 
 
 	pPref->m_bMidiNoteOffIgnore = m_pIgnoreNoteOffCheckBox->isChecked();
+	pPref->m_bMidiFixedMapping = m_pFixedMapping->isChecked();
+	pPref->m_bMidiDiscardNoteAfterAction = m_pDiscardMidiMsgCheckbox->isChecked();
 
 	// Mixer falloff
 	QString falloffStr = mixerFalloffComboBox->currentText();
@@ -414,6 +444,7 @@ void PreferencesDialog::on_okBtn_clicked()
 	// General tab
 	pPref->setRestoreLastSongEnabled( restoreLastUsedSongCheckbox->isChecked() );
 	pPref->setRestoreLastPlaylistEnabled( restoreLastUsedPlaylistCheckbox->isChecked() );
+	pPref->setUseRelativeFilenamesForPlaylists( useRelativePlaylistPathsCheckbox->isChecked() );
 	pPref->m_bsetLash = useLashCheckbox->isChecked(); //restore m_bsetLash after saving pref.
 
 	//path to rubberband
@@ -433,6 +464,28 @@ void PreferencesDialog::on_okBtn_clicked()
 
 	pPref->setDefaultUILayout( uiLayoutComboBox->currentIndex() );
 
+	int coloringMethod = coloringMethodCombo->currentIndex();
+
+	pPref->setColoringMethod( coloringMethod );
+
+	switch( coloringMethod )
+	{
+		case 0:
+			//Automatic
+			pPref->setColoringMethodAuxValue(0);
+			break;
+		case 1:
+			pPref->setColoringMethodAuxValue( coloringMethodAuxSpinBox->value() );
+			break;
+		case 2:
+			pPref->setColoringMethodAuxValue( coloringMethodAuxSpinBox->value() );
+			break;
+	}
+
+	HydrogenApp *pH2App = HydrogenApp::get_instance();
+	SongEditorPanel* pSongEditorPanel = pH2App->getSongEditorPanel();
+	SongEditor * pSongEditor = pSongEditorPanel->getSongEditor();
+	pSongEditor->updateEditorandSetTrue();
 
 	pPref->savePreferences();
 
@@ -447,7 +500,6 @@ void PreferencesDialog::on_okBtn_clicked()
 }
 
 
-
 void PreferencesDialog::on_driverComboBox_activated( int index )
 {
 	UNUSED( index );
@@ -702,6 +754,26 @@ void PreferencesDialog::on_useLashCheckbox_clicked()
 	QMessageBox::information ( this, "Hydrogen", trUtf8 ( "Please restart hydrogen to enable/disable LASH support" ) );
 }
 
+void PreferencesDialog::coloringMethodCombo_currentIndexChanged (int index)
+{
+	switch(index)
+	{
+		case 0:
+			coloringMethodAuxLabel->setText( "" );
+			coloringMethodAuxSpinBox->hide();
+			break;
+		case 1:
+			coloringMethodAuxLabel->setText( trUtf8("Number of steps") );
+			coloringMethodAuxSpinBox->setMinimum(1);
+			coloringMethodAuxSpinBox->show();
+			break;
+		case 2:
+			coloringMethodAuxLabel->setText( trUtf8("Color (Hue value)") );
+			coloringMethodAuxSpinBox->setMinimum(0);
+			coloringMethodAuxSpinBox->show();
+			break;
+	}
+}
 
 void PreferencesDialog::on_resampleComboBox_currentIndexChanged ( int index )
 {
