@@ -32,6 +32,8 @@
 #include <hydrogen/audio_engine.h>
 #include <hydrogen/event_queue.h>
 #include <hydrogen/basics/instrument.h>
+#include <hydrogen/LocalFileMng.h>
+#include <hydrogen/timeline.h>
 using namespace H2Core;
 
 #include "UndoActions.h"
@@ -51,7 +53,9 @@ using namespace H2Core;
 #include "../SongPropertiesDialog.h"
 #include "../Skin.h"
 #include "../VirtualPatternDialog.h"
-#include <hydrogen/LocalFileMng.h>
+
+
+
 
 #ifdef WIN32
 #include <time.h>
@@ -277,7 +281,7 @@ void SongEditor::addPattern( int nColumn , int nRow )
 		}
 		pColumn->add( pPattern );
 	}
-	pSong->__is_modified = true;
+	pSong->set_is_modified( true );
 	AudioEngine::get_instance()->unlock();
 	m_bSequenceChanged = true;
 	update();
@@ -308,7 +312,7 @@ void SongEditor::deletePattern( int nColumn , int nRow, unsigned nColumnIndex )
 			break;
 		}
 	}
-	pSong->__is_modified = true;
+	pSong->set_is_modified( true );
 	AudioEngine::get_instance()->unlock();
 	m_bSequenceChanged = true;
 	update();
@@ -590,7 +594,7 @@ void SongEditor::movePatternCellAction( std::vector<QPoint> movingCells, std::ve
 		}
 	}
 
-	pEngine->getSong()->__is_modified = true;
+	pEngine->getSong()->set_is_modified( true );
 	AudioEngine::get_instance()->unlock();
 
 	m_bIsMoving = false;
@@ -813,6 +817,60 @@ void SongEditor::drawPattern( int pos, int number, bool invertColour )
 	QPainter p( m_pSequencePixmap );
 	QColor patternColor( pStyle->m_songEditor_pattern1Color.getRed(), pStyle->m_songEditor_pattern1Color.getGreen(), pStyle->m_songEditor_pattern1Color.getBlue() );
 
+	/*
+	 * The following color modes are available:
+	 *
+	 * Fixed: One color. Argument: specified color
+	 * Steps: User defined number of steps.
+	 * Automatic: Steps = Number of pattern in song
+	 */
+
+	int coloringMethod = pref->getColoringMethod();
+	int coloringMethodAuxValue = pref->getColoringMethodAuxValue();
+	int steps = 1;
+
+	/*
+	 * This coloring of the song editor "squares" is done using the hsv color model,
+	 * see http://qt-project.org/doc/qt-4.8/qcolor.html#the-hsv-color-model for details.
+	 *
+	 * The default color of the cubes in rgb is 97,167,251.
+	 * The hsv equivalent is 213,156,249.
+	 */
+	int hue = 213;
+
+	Song* song = Hydrogen::get_instance()->getSong();
+	PatternList *patList = song->get_pattern_list();
+
+	switch(coloringMethod)
+	{
+		case 0:
+			//Automatic
+			steps = patList->size();
+
+			if(steps == 0)
+			{
+				//beware of the division by zero..
+				steps = 1;
+			}
+
+			hue = ((number % steps) * (300 / steps) + 213) % 300;
+			patternColor.setHsv( hue , 156 , 249);
+			break;
+		case 1:
+			//Steps
+			steps = coloringMethodAuxValue;
+			hue = ((number % steps) * (300 / steps) + 213) % 300;
+			patternColor.setHsv( hue , 156, 249);
+			break;
+		case 2:
+			//Fixed color
+			hue = coloringMethodAuxValue;
+			patternColor.setHsv( hue , 156, 249);
+			break;
+	}
+
+
+
 	if (true == invertColour) {
 		patternColor = patternColor.darker(200);
 	}//if
@@ -857,7 +915,7 @@ void SongEditor::clearThePatternSequenseVector( QString filename )
 	}
 	pPatternGroupsVect->clear();
 
-	song->__is_modified = true;
+	song->set_is_modified( true );
 	AudioEngine::get_instance()->unlock();
 	m_bSequenceChanged = true;
 	update();
@@ -865,7 +923,7 @@ void SongEditor::clearThePatternSequenseVector( QString filename )
 
 void SongEditor::updateEditorandSetTrue()
 {
-	Hydrogen::get_instance()->getSong()->__is_modified = true;
+	Hydrogen::get_instance()->getSong()->set_is_modified( true );
 	m_bSequenceChanged = true;
 	update();
 }
@@ -932,11 +990,12 @@ void SongEditorPatternList::patternChangedEvent() {
 	update();
 	///here we check the timeline  && m_pSong->get_mode() == Song::SONG_MODE
 	Hydrogen *engine = Hydrogen::get_instance();
+	Timeline *pTimeline = engine->getTimeline();
 		if ( ( Preferences::get_instance()->getUseTimelineBpm() ) && ( engine->getSong()->get_mode() == Song::SONG_MODE ) ){
-		for ( int i = 0; i < static_cast<int>(engine->m_timelinevector.size()); i++){
-			if ( ( engine->m_timelinevector[i].m_htimelinebeat == engine->getPatternPos() )
-				&& ( engine->getNewBpmJTM() != engine->m_timelinevector[i].m_htimelinebpm ) ){
-				engine->setBPM( engine->m_timelinevector[i].m_htimelinebpm );
+		for ( int i = 0; i < static_cast<int>(pTimeline->m_timelinevector.size()); i++){
+			if ( ( pTimeline->m_timelinevector[i].m_htimelinebeat == engine->getPatternPos() )
+				&& ( engine->getNewBpmJTM() != pTimeline->m_timelinevector[i].m_htimelinebpm ) ){
+				engine->setBPM( pTimeline->m_timelinevector[i].m_htimelinebpm );
 			}//if
 		}//for
 	}//if
@@ -975,8 +1034,8 @@ void SongEditorPatternList::mousePressEvent( QMouseEvent *ev )
 ///
 void SongEditorPatternList::togglePattern( int row ) {
 
-	Hydrogen *engine = Hydrogen::get_instance();
-	engine->sequencer_setNextPattern( row, false, true );
+	Hydrogen *pEngine = Hydrogen::get_instance();
+	pEngine->sequencer_setNextPattern( row );
 	createBackground();
 	update();
 }
@@ -1017,7 +1076,6 @@ void SongEditorPatternList::inlineEditingEntered()
 												  line->text(), patternBeingEdited->get_info(), patternBeingEdited->get_category(), nSelectedPattern );
 		HydrogenApp::get_instance()->m_undoStack->push( action );
 	}
-// 	patternBeingEdited = NULL;
 }
 
 
@@ -1106,12 +1164,7 @@ void SongEditorPatternList::createBackground()
 
 		// Text
 		bool bNext = false, bActive = false;
-/*		for (uint j = 0; j < pCurrentPatternList->size(); j++) {
-			if ( pPattern == pCurrentPatternList->get(j) ) {
-				bActive = true;
-				break;
-			}
-		}*/
+
 		if ( pCurrentPatternList->index( pPattern ) != -1 ) bActive = true;
 		if ( pEngine->getNextPatterns()->index( pPattern ) != -1 ) bNext = true;
 
@@ -1127,7 +1180,6 @@ void SongEditorPatternList::createBackground()
 			p.drawPixmap( QPoint( 5, text_y + 3 ), m_playingPattern_off_Pixmap );
 		}
 		else if (bActive) {
-//			p.drawText( 5, text_y - 1 - m_nGridHeight, m_nWidth - 25, m_nGridHeight + 2, Qt::AlignVCenter, ">" );
 
 			//mark active pattern with triangular
 			if( ! pref->patternModePlaysSelected() ){
@@ -1260,7 +1312,7 @@ void SongEditorPatternList::loadPatternAction( QString afilename, int position)
 		pPatternList->replace( position, pNewPattern );
 
 		engine->setSelectedPatternNumber( position );
-		song->__is_modified = true;
+		song->set_is_modified( true );
 		createBackground();
 		HydrogenApp::get_instance()->getSongEditorPanel()->updateAll();
 	}
@@ -1344,7 +1396,7 @@ void SongEditorPatternList::acceptPatternPropertiesDialogSettings(QString newPat
 	pattern->set_name( newPatternName );
 	pattern->set_info( newPatternInfo );
 	pattern->set_category( newPatternCategory );
-	song->__is_modified = true;
+	song->set_is_modified( true );
 	EventQueue::get_instance()->push_event( EVENT_SELECTED_PATTERN_CHANGED, -1 );
 	createBackground();
 	update();
@@ -1359,7 +1411,7 @@ void SongEditorPatternList::revertPatternPropertiesDialogSettings(QString oldPat
 	H2Core::Pattern *pattern = patternList->get( patternNr );
 	pattern->set_name( oldPatternName );
 	pattern->set_category( oldPatternCategory );
-	song->__is_modified = true;
+	song->set_is_modified( true );
 	EventQueue::get_instance()->push_event( EVENT_SELECTED_PATTERN_CHANGED, -1 );
 	createBackground();
 	update();
@@ -1395,7 +1447,7 @@ void SongEditorPatternList::deletePatternFromList( QString patternFilename, QStr
 	Hydrogen *pEngine = Hydrogen::get_instance();
 
 	if ( pEngine->getSong()->get_mode() == Song::PATTERN_MODE ) {
-		pEngine->sequencer_setNextPattern( -1, false, false );	// reimposto il prossimo pattern a NULL, altrimenti viene scelto quello che sto distruggendo ora...
+		pEngine->sequencer_setNextPattern( -1 );	// reimposto il prossimo pattern a NULL, altrimenti viene scelto quello che sto distruggendo ora...
 	}
 
 	Song *song = pEngine->getSong();
@@ -1422,7 +1474,7 @@ void SongEditorPatternList::deletePatternFromList( QString patternFilename, QStr
 	//~save pattern end
 
 	H2Core::Pattern *pattern = pSongPatternList->get( patternPosition );
-	INFOLOG( QString("[patternPopup_delete] Delete pattern: %1 @%2").arg(pattern->get_name()).arg( (long)pattern ) );
+	INFOLOG( QString("[patternPopup_delete] Delete pattern: %1 @%2").arg(pattern->get_name()).arg( (long long)pattern ) );
 	pSongPatternList->del(pattern);
 
 	vector<PatternList*> *patternGroupVect = song->get_pattern_group_vector();
@@ -1477,7 +1529,7 @@ void SongEditorPatternList::deletePatternFromList( QString patternFilename, QStr
 	pSongPatternList->flattened_virtual_patterns_compute();
 
 	delete pattern;
-	song->__is_modified = true;
+	song->set_is_modified( true );
 	HydrogenApp::get_instance()->getSongEditorPanel()->updateAll();
 
 }
@@ -1506,7 +1558,7 @@ void SongEditorPatternList::restoreDeletedPatternsFromList( QString patternFilen
 		}
 
 		pPatternList->replace( tmpselectedpatternpos, pNewPattern );
-		song->__is_modified = true;
+		song->set_is_modified( true );
 		createBackground();
 		HydrogenApp::get_instance()->getSongEditorPanel()->updateAll();
 		EventQueue::get_instance()->push_event( EVENT_SELECTED_PATTERN_CHANGED, -1 );
@@ -1578,7 +1630,7 @@ void SongEditorPatternList::patternPopup_copyAction( QString patternFilename, in
 		pPatternList->replace( patternposition, pNewPattern );
 		engine->setSelectedPatternNumber( patternposition );
 
-		song->__is_modified = true;
+		song->set_is_modified( true );
 		createBackground();
 		HydrogenApp::get_instance()->getSongEditorPanel()->updateAll();
 		EventQueue::get_instance()->push_event( EVENT_SELECTED_PATTERN_CHANGED, -1 );
@@ -1676,7 +1728,7 @@ void SongEditorPatternList::fillRangeWithPattern( FillRange* pRange, int nPatter
 
 
 	// Update
-	pSong->__is_modified = true;
+	pSong->set_is_modified( true );
 	HydrogenApp::get_instance()->getSongEditorPanel()->updateAll();
 }
 
@@ -1694,7 +1746,7 @@ void SongEditorPatternList::dropEvent(QDropEvent *event)
 {
 	QString sText = event->mimeData()->text();
 
-	if( sText.startsWith("Songs:") || sText.startsWith("move instrument:") ){
+	if( sText.startsWith("Songs:") || sText.startsWith("move instrument:") || sText.startsWith("importInstrument:")){
 		event->acceptProposedAction();
 		return;
 	}
@@ -1712,11 +1764,9 @@ void SongEditorPatternList::dropEvent(QDropEvent *event)
 
 		SE_movePatternListItemAction *action = new SE_movePatternListItemAction( nSourcePattern , nTargetPattern ) ;
 		HydrogenApp::get_instance()->m_undoStack->push( action );
-		//movePatternLine( nSourcePattern , nTargetPattern );
 
 		event->acceptProposedAction();
 	}else {
-
 		QStringList tokens = sText.split( "::" );
 		QString sPatternName = tokens.at( 1 );
 
@@ -1770,7 +1820,7 @@ void SongEditorPatternList::movePatternLine( int nSourcePattern , int nTargetPat
 		}
 		engine->setSelectedPatternNumber( nTargetPattern );
 		HydrogenApp::get_instance()->getSongEditorPanel()->updateAll();
-		pSong->__is_modified = true;
+		pSong->set_is_modified( true );
 }
 
 
@@ -1868,6 +1918,8 @@ void SongEditorPositionRuler::createBackground()
 	m_pBackgroundPixmap->fill( backgroundColor );
 
 	Preferences *pref = Preferences::get_instance();
+	Timeline * pTimeline = Hydrogen::get_instance()->getTimeline();
+
 	QString family = pref->getApplicationFontFamily();
 	int size = pref->getApplicationFontPointSize();
 	QFont font( family, size );
@@ -1879,8 +1931,8 @@ void SongEditorPositionRuler::createBackground()
 	char tmp[10];
 	for (uint i = 0; i < m_nMaxPatternSequence + 1; i++) {
 		uint x = 10 + i * m_nGridWidth;
-		for ( int t = 0; t < static_cast<int>(Hydrogen::get_instance()->m_timelinetagvector.size()); t++){
-			if ( Hydrogen::get_instance()->m_timelinetagvector[t].m_htimelinetagbeat == i ) {
+		for ( int t = 0; t < static_cast<int>(pTimeline->m_timelinetagvector.size()); t++){
+			if ( pTimeline->m_timelinetagvector[t].m_htimelinetagbeat == i ) {
 				p.setPen( Qt::cyan );
 				p.drawText( x - m_nGridWidth / 2 , 12, m_nGridWidth * 2, height() , Qt::AlignCenter, "T");
 			}
@@ -1913,9 +1965,9 @@ void SongEditorPositionRuler::createBackground()
 		uint x = 10 + i * m_nGridWidth;
 		p.drawLine( x, 2, x, 5 );
 		p.drawLine( x, 19, x, 20 );
-		for ( int t = 0; t < static_cast<int>(Hydrogen::get_instance()->m_timelinevector.size()); t++){
-			if ( Hydrogen::get_instance()->m_timelinevector[t].m_htimelinebeat == i ) {
-				sprintf( tempo, "%d",  ((int)Hydrogen::get_instance()->m_timelinevector[t].m_htimelinebpm) );
+		for ( int t = 0; t < static_cast<int>(pTimeline->m_timelinevector.size()); t++){
+			if ( pTimeline->m_timelinevector[t].m_htimelinebeat == i ) {
+				sprintf( tempo, "%d",  ((int)pTimeline->m_timelinevector[t].m_htimelinebpm) );
 				p.drawText( x - m_nGridWidth, 3, m_nGridWidth * 2, height() / 2 - 5, Qt::AlignCenter, tempo );
 			}
 		}
@@ -2088,25 +2140,26 @@ void SongEditorPositionRuler::updatePosition()
 void SongEditorPositionRuler::editTimeLineAction( int newPosition, float newBpm )
 {
 	Hydrogen* engine = Hydrogen::get_instance();
+	Timeline* pTimeline = engine->getTimeline();
 
 	//erase the value to set the new value
-	if( engine->m_timelinevector.size() >= 1 ){
-		for ( int t = 0; t < engine->m_timelinevector.size(); t++){
-			if ( engine->m_timelinevector[t].m_htimelinebeat == newPosition -1 ) {
-				engine->m_timelinevector.erase( engine->m_timelinevector.begin() +  t);
+	if( pTimeline->m_timelinevector.size() >= 1 ){
+		for ( int t = 0; t < pTimeline->m_timelinevector.size(); t++){
+			if ( pTimeline->m_timelinevector[t].m_htimelinebeat == newPosition -1 ) {
+				pTimeline->m_timelinevector.erase( pTimeline->m_timelinevector.begin() +  t);
 			}
 		}
 	}
 
-	Hydrogen::HTimelineVector tlvector;
+	Timeline::HTimelineVector tlvector;
 
 	tlvector.m_htimelinebeat = newPosition -1 ;
 
 	if( newBpm < 30.0 ) newBpm = 30.0;
 	if( newBpm > 500.0 ) newBpm = 500.0;
 	tlvector.m_htimelinebpm = newBpm;
-	engine->m_timelinevector.push_back( tlvector );
-	engine->sortTimelineVector();
+	pTimeline->m_timelinevector.push_back( tlvector );
+	pTimeline->sortTimelineVector();
 	createBackground();
 }
 
@@ -2115,11 +2168,13 @@ void SongEditorPositionRuler::editTimeLineAction( int newPosition, float newBpm
 void SongEditorPositionRuler::deleteTimeLinePosition( int position )
 {
 	Hydrogen* engine = Hydrogen::get_instance();
+	Timeline* pTimeline = engine->getTimeline();
+
 	//erase the value to set the new value
-	if( engine->m_timelinevector.size() >= 1 ){
-		for ( int t = 0; t < engine->m_timelinevector.size(); t++){
-			if ( engine->m_timelinevector[t].m_htimelinebeat == position -1 ) {
-				engine->m_timelinevector.erase( engine->m_timelinevector.begin() +  t);
+	if( pTimeline->m_timelinevector.size() >= 1 ){
+		for ( int t = 0; t < pTimeline->m_timelinevector.size(); t++){
+			if ( pTimeline->m_timelinevector[t].m_htimelinebeat == position -1 ) {
+				pTimeline->m_timelinevector.erase( pTimeline->m_timelinevector.begin() +  t);
 			}
 		}
 	}
@@ -2130,21 +2185,22 @@ void SongEditorPositionRuler::deleteTimeLinePosition( int position )
 void SongEditorPositionRuler::editTagAction( QString text, int position, QString textToReplace)
 {
 	Hydrogen* engine = Hydrogen::get_instance();
+	Timeline* pTimeline = engine->getTimeline();
 
 	//check vector for old entries and remove them.
-	for( int i = 0; i < engine->m_timelinetagvector.size(); ++i ){
-		if( ( engine->m_timelinetagvector[i].m_htimelinetag == textToReplace ) &&
-			( engine->m_timelinetagvector[i].m_htimelinetagbeat == position ) ){
+	for( int i = 0; i < pTimeline->m_timelinetagvector.size(); ++i ){
+		if( ( pTimeline->m_timelinetagvector[i].m_htimelinetag == textToReplace ) &&
+			( pTimeline->m_timelinetagvector[i].m_htimelinetagbeat == position ) ){
 
-			engine->m_timelinetagvector.erase( engine->m_timelinetagvector.begin() + i );
+			pTimeline->m_timelinetagvector.erase( pTimeline->m_timelinetagvector.begin() + i );
 			break;
 		}
 	}
-	Hydrogen::HTimelineTagVector tlvector;
+	Timeline::HTimelineTagVector tlvector;
 	tlvector.m_htimelinetagbeat = position;
 	tlvector.m_htimelinetag = text;
-	engine->m_timelinetagvector.push_back( tlvector );
-	engine->sortTimelineTagVector();
+	pTimeline->m_timelinetagvector.push_back( tlvector );
+	pTimeline->sortTimelineTagVector();
 	createBackground();
 }
 
@@ -2152,14 +2208,16 @@ void SongEditorPositionRuler::deleteTagAction( QString text, int position )
 {
 
 	Hydrogen* engine = Hydrogen::get_instance();
-	for( int i = 0; i < engine->m_timelinetagvector.size(); ++i ){
-		if( ( engine->m_timelinetagvector[i].m_htimelinetag == text ) &&
-			( engine->m_timelinetagvector[i].m_htimelinetagbeat == position ) ){
+	Timeline* pTimeline = engine->getTimeline();
 
-			engine->m_timelinetagvector.erase( engine->m_timelinetagvector.begin() + i );
+	for( int i = 0; i < pTimeline->m_timelinetagvector.size(); ++i ){
+		if( ( pTimeline->m_timelinetagvector[i].m_htimelinetag == text ) &&
+			( pTimeline->m_timelinetagvector[i].m_htimelinetagbeat == position ) ){
+
+			pTimeline->m_timelinetagvector.erase( pTimeline->m_timelinetagvector.begin() + i );
 			break;
 		}
 	}
-	engine->sortTimelineTagVector();
+	pTimeline->sortTimelineTagVector();
 	createBackground();
 }
