@@ -108,12 +108,16 @@ void SoundLibraryImportDialog::updateRepositoryCombo()
 void SoundLibraryImportDialog::onRepositoryComboBoxIndexChanged(int i)
 {
 	UNUSED(i);
-	QString cacheFile = getCachedFilename();
-	if( !H2Core::Filesystem::file_exists( cacheFile ) )
+
+	if(!repositoryCombo->currentText().isEmpty())
 	{
-		SoundLibraryImportDialog::on_UpdateListBtn_clicked();
+		QString cacheFile = getCachedFilename();
+		if( !H2Core::Filesystem::file_exists( cacheFile, true ) )
+		{
+			SoundLibraryImportDialog::on_UpdateListBtn_clicked();
+		}
+		reloadRepositoryData();
 	}
-	reloadRepositoryData();
 }
 
 ///
@@ -126,6 +130,25 @@ void SoundLibraryImportDialog::on_EditListBtn_clicked()
 	updateRepositoryCombo();
 }
 
+void SoundLibraryImportDialog::clearImageCache()
+{
+	// Note: After a kit is installed the list refreshes and this gets called to 
+	// clear the image cache - maybe we want to keep the cache in this case?
+	QString cacheDir = H2Core::Filesystem::repositories_cache_dir() ;
+	INFOLOG("Deleting cached image files from " + cacheDir.toLocal8Bit() );
+
+	QDir dir( cacheDir );
+	dir.setNameFilters(QStringList() << "*.png");
+	dir.setFilter(QDir::Files);
+	foreach(QString dirFile, dir.entryList())
+	{
+		if ( !dir.remove(dirFile) )
+		{
+			WARNINGLOG("Error removing image file(s) from cache.");
+		}
+	}
+}
+
 QString SoundLibraryImportDialog::getCachedFilename()
 {
 	QString cacheDir = H2Core::Filesystem::repositories_cache_dir();
@@ -134,6 +157,15 @@ QString SoundLibraryImportDialog::getCachedFilename()
 	return cacheFile;
 }
 
+QString SoundLibraryImportDialog::getCachedImageFilename()
+{
+	QString cacheDir = H2Core::Filesystem::repositories_cache_dir();
+	QString kitNameMd5 = QString(QCryptographicHash::hash(( SoundLibraryNameLbl->text().toLatin1() ),QCryptographicHash::Md5).toHex());
+	QString cacheFile = cacheDir + "/" + kitNameMd5 + ".png";	
+	return cacheFile;
+}
+
+
 void SoundLibraryImportDialog::writeCachedData(const QString& fileName, const QString& data)
 {
 	if( data.isEmpty() )
@@ -154,6 +186,22 @@ void SoundLibraryImportDialog::writeCachedData(const QString& fileName, const QS
 	outFile.close();
 }
 
+void SoundLibraryImportDialog::writeCachedImage( const QString& imageFile, QPixmap& pixmap )
+{
+	QString cacheFile = getCachedImageFilename() ;
+
+	QFile outFile( cacheFile );
+	if( !outFile.open( QIODevice::WriteOnly ) )
+	{
+		ERRORLOG( "Failed to open file for writing repository image cache." );
+		return;
+	}
+	
+	pixmap.save(&outFile);
+
+	outFile.close();
+}
+
 QString SoundLibraryImportDialog::readCachedData(const QString& fileName)
 {
 	QString content;
@@ -177,6 +225,20 @@ QString SoundLibraryImportDialog::readCachedData(const QString& fileName)
 	return content;
 }
 
+QString SoundLibraryImportDialog::readCachedImage( const QString& imageFile )
+{
+	QString cacheFile = getCachedImageFilename() ;
+
+	QFile file( cacheFile );
+	if( !file.exists() )
+	{       
+		// no image in cache, just return NULL
+		return NULL; 
+	}       
+	
+	return cacheFile;
+}
+
 void SoundLibraryImportDialog::reloadRepositoryData()
 {
 	QString sDrumkitXML;
@@ -235,6 +297,17 @@ void SoundLibraryImportDialog::reloadRepositoryData()
 					soundLibInfo.setLicense( licenseNode.text() );
 				}
 
+				QDomElement imageNode = drumkitNode.firstChildElement( "image" );
+				if ( !imageNode.isNull() ) {
+					soundLibInfo.setImage( imageNode.text() );
+				}
+
+				QDomElement imageLicenseNode = drumkitNode.firstChildElement( "imageLicense" );
+				if ( !imageLicenseNode.isNull() ) {
+					soundLibInfo.setImageLicense( imageLicenseNode.text() );
+				}
+
+
 				m_soundLibraryList.push_back( soundLibInfo );
 			}
 		}
@@ -263,7 +336,7 @@ void SoundLibraryImportDialog::on_UpdateListBtn_clicked()
 	 *
 	 * CACHE_DIR
 	 *     +-----repositories
-	 *           +-----serverlist_$(md5(SERVER_NAME))
+	 *	   +-----serverlist_$(md5(SERVER_NAME))
 	 */
 
 
@@ -323,6 +396,10 @@ void SoundLibraryImportDialog::updateSoundLibraryList()
 			pDrumkitItem->setText( 1, trUtf8( "New" ) );
 		}
 	}
+
+	// Also clear out the image cache
+	clearImageCache();
+
 }
 
 
@@ -356,7 +433,38 @@ bool SoundLibraryImportDialog::isSoundLibraryItemAlreadyInstalled( SoundLibraryI
 	return false;
 }
 
+void SoundLibraryImportDialog::loadImage(QString img )
+{
+	QPixmap pixmap;
+	pixmap.load( img ) ;
 
+	writeCachedImage( drumkitImageLabel->text(), pixmap );
+	showImage( pixmap );
+}
+
+void SoundLibraryImportDialog::showImage( QPixmap pixmap )
+{
+	int x = (int) drumkitImageLabel->size().width();
+	int y = drumkitImageLabel->size().height();
+	float labelAspect = (float) x / y;
+	float imageAspect = (float) pixmap.width() / pixmap.height();
+
+	if ( ( x < pixmap.width() ) || ( y < pixmap.height() ) )
+	{
+		if ( labelAspect >= imageAspect )
+		{
+			// image is taller or the same as label frame
+			pixmap = pixmap.scaledToHeight( y );
+		}
+		else
+		{
+			// image is wider than label frame
+			pixmap = pixmap.scaledToWidth( x );
+		}
+	}
+	drumkitImageLabel->setPixmap( pixmap ); // TODO: Check if valid!
+
+}
 
 
 void SoundLibraryImportDialog::soundLibraryItemChanged( QTreeWidgetItem* current, QTreeWidgetItem* previous  )
@@ -381,9 +489,81 @@ void SoundLibraryImportDialog::soundLibraryItemChanged( QTreeWidgetItem* current
 
 				AuthorLbl->setText( trUtf8( "Author: %1" ).arg( info.getAuthor() ) );
 
-				LicenseLbl->setText( trUtf8( "License: %1" ).arg( info.getLicense()) );
+				LicenseLbl->setText( trUtf8( "Drumkit License: %1" ).arg( info.getLicense()) );
 
+				ImageLicenseLbl->setText( trUtf8("Image License: %1" ).arg( info.getImageLicense() ) );
 
+				// Load the drumkit image
+				// Clear any image first
+				drumkitImageLabel->setPixmap( NULL );
+				drumkitImageLabel->setText( info.getImage() );
+
+				if ( info.getImage().length() > 0 )
+				{
+					if ( isSoundLibraryItemAlreadyInstalled( info ) )
+					{
+						// get image file from local disk
+						QString sName = QFileInfo( info.getUrl() ).fileName();
+						sName = sName.left( sName.lastIndexOf( "." ) );
+
+						H2Core::Drumkit* drumkitInfo = H2Core::Drumkit::load_by_name( sName, false );
+						if ( drumkitInfo ) 
+						{
+							// get the image from the local filesystem
+							QPixmap pixmap ( drumkitInfo->get_path() + "/" + drumkitInfo->get_image() );
+							INFOLOG("Loaded image " + drumkitInfo->get_image().toLocal8Bit() + " from local filesystem");
+							showImage( pixmap );
+						} 
+						else 
+						{
+							___ERRORLOG ( "Error loading the drumkit" );
+						}
+
+					}
+					else
+					{
+						// Try from the cache
+						QString cachedFile = readCachedImage( info.getImage() );
+						
+						if ( cachedFile.length() > 0 )
+						{
+							QPixmap pixmap ( cachedFile );
+							showImage( pixmap );
+							INFOLOG( "Loaded image " + info.getImage().toLocal8Bit() + " from cache (" + cachedFile + ")" );
+						}
+						else
+						{
+							// Get the drumkit's directory name from URL
+							//
+							// Example: if the server repo URL is: http://www.hydrogen-music.org/feeds/drumkit_list.php 
+							// and the image name from the XML is Roland_TR-808_drum_machine.jpg
+							// the URL for the image will be: http://www.hydrogen-music.org/feeds/images/Roland_TR-808_drum_machine.jpg
+
+							if ( info.getImage().length() > 0 )
+							{
+								int lastSlash = info.getUrl().lastIndexOf( QString( "/" ));
+
+								QString imageUrl;
+								QString sLocalFile;
+								imageUrl = repositoryCombo->currentText().left( repositoryCombo->currentText().lastIndexOf( QString( "/" )) + 1 ) + info.getImage() ;
+								sLocalFile = QDir::tempPath() + "/" + QFileInfo( imageUrl ).fileName();
+
+								DownloadWidget dl( this, trUtf8( "" ), imageUrl, sLocalFile );
+								dl.exec();
+
+								loadImage( sLocalFile );
+								// Delete the temporary file
+								QFile::remove( sLocalFile );
+							}
+						}
+					}
+				}
+				else
+				{
+					// no image file specified in drumkit.xml
+					INFOLOG( "No image for this kit specified in drumkit.xml on remote server" );
+				}
+				
 				DownloadBtn->setEnabled( true );
 				return;
 			}
@@ -424,37 +604,49 @@ void SoundLibraryImportDialog::on_DownloadBtn_clicked()
 				sLocalFile = dataDir + "patterns/" + QFileInfo( sURL ).fileName();
 			}
 
+			bool Error = false;
+
 			for ( int i = 0; i < 30; ++i ) {
 				DownloadWidget dl( this, trUtf8( "Downloading SoundLibrary..." ), sURL, sLocalFile );
 				dl.exec();
 
 
-				QString redirect_url = dl.get_redirect_url();
-				if (redirect_url == "" ) {
+				QUrl redirect_url = dl.get_redirect_url();
+				if (redirect_url.isEmpty() ) {
 					// ok, we have all data
+					Error = dl.get_error();
 					break;
 				}
 				else {
-					sURL = redirect_url;
+					sURL = redirect_url.toEncoded();
+					Error = dl.get_error();
 				}
 			}
 
 
-			// install the new soundlibrary
-			try {
-				if ( sType == "drumkit" ) {
-					H2Core::Drumkit::install( sLocalFile );
-					QApplication::restoreOverrideCursor();
-					QMessageBox::information( this, "Hydrogen", QString( trUtf8( "SoundLibrary imported in %1" ) ).arg( dataDir ) );
-				}
+			//No 'else', error message has been already displayed by DL widget
+			if(!Error)
+			{
+				// install the new soundlibrary
+				try {
+					if ( sType == "drumkit" ) {
+						H2Core::Drumkit::install( sLocalFile );
+						QApplication::restoreOverrideCursor();
+						QMessageBox::information( this, "Hydrogen", QString( trUtf8( "SoundLibrary imported in %1" ) ).arg( dataDir ) );
+					}
 
-				if ( sType == "song" || sType == "pattern") {
+					if ( sType == "song" || sType == "pattern") {
+						QApplication::restoreOverrideCursor();
+					}
+				}
+				catch( H2Core::H2Exception ex ) {
 					QApplication::restoreOverrideCursor();
+					QMessageBox::warning( this, "Hydrogen", trUtf8( "An error occurred importing the SoundLibrary."  ) );
 				}
 			}
-			catch( H2Core::H2Exception ex ) {
+			else
+			{
 				QApplication::restoreOverrideCursor();
-				QMessageBox::warning( this, "Hydrogen", trUtf8( "An error occurred importing the SoundLibrary."  ) );
 			}
 
 			QApplication::setOverrideCursor(Qt::WaitCursor);
