@@ -35,16 +35,18 @@ class Drumkit;
 
 class SoundLibraryPropertiesDialog : public QDialog, public Ui_SoundLibraryPropertiesDialog_UI, public H2Core::Object
 {
-    H2_OBJECT
+	H2_OBJECT
 	Q_OBJECT
 	public:
 		SoundLibraryPropertiesDialog( QWidget* pParent , Drumkit *drumkitInfo, Drumkit *preDrumKit );
 		~SoundLibraryPropertiesDialog();
 
 	private slots:
-			void on_saveBtn_clicked();
+		void on_saveBtn_clicked();
+		void on_imageBrowsePushButton_clicked();
 
 	private:
+		void updateImage( QString& filename );
 };
 
 }
